@@ -18,6 +18,8 @@
  */
 package org.apache.pinot.broker.routing.segmentpruner;
 
+import com.google.common.annotations.VisibleForTesting;
+import com.google.common.base.Preconditions;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
@@ -26,6 +28,7 @@ import javax.annotation.Nullable;
 import org.apache.commons.collections.MapUtils;
 import org.apache.helix.store.zk.ZkHelixPropertyStore;
 import org.apache.helix.zookeeper.datamodel.ZNRecord;
+import org.apache.pinot.common.metadata.ZKMetadataProvider;
 import org.apache.pinot.segment.local.utils.TableConfigUtils;
 import org.apache.pinot.spi.config.table.ColumnPartitionConfig;
 import org.apache.pinot.spi.config.table.RoutingConfig;
@@ -33,6 +36,9 @@ import org.apache.pinot.spi.config.table.SegmentPartitionConfig;
 import org.apache.pinot.spi.config.table.SegmentsValidationAndRetentionConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.config.table.TableType;
+import org.apache.pinot.spi.data.DateTimeFieldSpec;
+import org.apache.pinot.spi.data.DateTimeFormatSpec;
+import org.apache.pinot.spi.data.Schema;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -52,7 +58,7 @@ public class SegmentPrunerFactory {
     boolean needsEmptySegment = TableConfigUtils.needsEmptySegmentPruner(tableConfig);
     if (needsEmptySegment) {
       // Add EmptySegmentPruner if needed
-      segmentPruners.add(new EmptySegmentPruner(tableConfig, propertyStore));
+      segmentPruners.add(new EmptySegmentPruner(tableConfig));
     }
 
     RoutingConfig routingConfig = tableConfig.getRoutingConfig();
@@ -113,8 +119,8 @@ public class SegmentPrunerFactory {
     LOGGER.info("Using PartitionSegmentPruner on partition columns: {} for table: {}", partitionColumns,
         tableNameWithType);
     return partitionColumns.size() == 1 ? new SinglePartitionColumnSegmentPruner(tableNameWithType,
-        partitionColumns.iterator().next(), propertyStore)
-        : new MultiPartitionColumnsSegmentPruner(tableNameWithType, partitionColumns, propertyStore);
+        partitionColumns.iterator().next())
+        : new MultiPartitionColumnsSegmentPruner(tableNameWithType, partitionColumns);
   }
 
   @Nullable
@@ -131,9 +137,26 @@ public class SegmentPrunerFactory {
       LOGGER.warn("Cannot enable time range pruning without time column for table: {}", tableNameWithType);
       return null;
     }
+    return createTimeSegmentPruner(tableConfig, propertyStore);
+  }
 
-    LOGGER.info("Using TimeRangePruner on time column: {} for table: {}", timeColumn, tableNameWithType);
-    return new TimeSegmentPruner(tableConfig, propertyStore);
+  @VisibleForTesting
+  static TimeSegmentPruner createTimeSegmentPruner(TableConfig tableConfig,
+      ZkHelixPropertyStore<ZNRecord> propertyStore) {
+    String tableNameWithType = tableConfig.getTableName();
+    String timeColumn = tableConfig.getValidationConfig().getTimeColumnName();
+    Preconditions.checkNotNull(timeColumn, "Time column must be configured in table config for table: %s",
+        tableNameWithType);
+    Schema schema = ZKMetadataProvider.getTableSchema(propertyStore, tableNameWithType);
+    Preconditions.checkNotNull(schema, "Failed to find schema for table: %s", tableNameWithType);
+    DateTimeFieldSpec dateTimeSpec = schema.getSpecForTimeColumn(timeColumn);
+    Preconditions.checkNotNull(dateTimeSpec, "Field spec must be specified in schema for time column: %s of table: %s",
+        timeColumn, tableNameWithType);
+    DateTimeFormatSpec timeFormatSpec = dateTimeSpec.getFormatSpec();
+
+    LOGGER.info("Using TimeRangePruner on time column: {} for table: {} with DateTimeFormatSpec: {}",
+        timeColumn, tableNameWithType, timeFormatSpec);
+    return new TimeSegmentPruner(tableConfig, timeColumn, timeFormatSpec);
   }
 
   private static List<SegmentPruner> sortSegmentPruners(List<SegmentPruner> pruners) {
