@@ -72,17 +72,6 @@ public class Connection {
     return new PreparedStatement(this, query);
   }
 
-  /**
-   * Creates a prepared statement, to escape query parameters.
-   *
-   * @param request The request for which to create a prepared statement.
-   * @return A prepared statement for this connection.
-   */
-  @Deprecated
-  public PreparedStatement prepareStatement(Request request) {
-    return new PreparedStatement(this, request);
-  }
-
   /**
    * Executes a query.
    *
@@ -94,18 +83,6 @@ public class Connection {
     return execute(null, query);
   }
 
-  /**
-   * Executes a Pinot Request.
-   * @param request The request to execute
-   * @return The result of the query
-   * @throws PinotClientException If an exception occurs while processing the query
-   */
-  @Deprecated
-  public ResultSetGroup execute(Request request)
-      throws PinotClientException {
-    return execute(null, request);
-  }
-
   /**
    * Executes a query.
    *
@@ -128,19 +105,6 @@ public class Connection {
     return new ResultSetGroup(response);
   }
 
-  /**
-   * Executes a Pinot Request.
-   *
-   * @param request The request to execute
-   * @return The result of the query
-   * @throws PinotClientException If an exception occurs while processing the query
-   */
-  @Deprecated
-  public ResultSetGroup execute(@Nullable String tableName, Request request)
-      throws PinotClientException {
-    return execute(tableName, request.getQuery());
-  }
-
   /**
    * Executes a query asynchronously.
    *
@@ -153,19 +117,6 @@ public class Connection {
     return executeAsync(null, query);
   }
 
-  /**
-   * Executes a Pinot Request asynchronously.
-   *
-   * @param request The request to execute
-   * @return A future containing the result of the query
-   * @throws PinotClientException If an exception occurs while processing the query
-   */
-  @Deprecated
-  public CompletableFuture<ResultSetGroup> executeAsync(Request request)
-      throws PinotClientException {
-    return executeAsync(null, request.getQuery());
-  }
-
   /**
    * Executes a query asynchronously.
    *
