@@ -45,14 +45,14 @@ public class JsonAsyncHttpPinotClientTransportFactory implements PinotClientTran
   private int _handshakeTimeoutMs = Integer.parseInt(DEFAULT_BROKER_HANDSHAKE_TIMEOUT_MS);
   private String _appId = null;
   private String _extraOptionString;
-  private boolean _useMultiStageEngine;
+  private boolean _useMultistageEngine;
 
   @Override
   public PinotClientTransport buildTransport() {
     ConnectionTimeouts connectionTimeouts =
         ConnectionTimeouts.create(_readTimeoutMs, _connectTimeoutMs, _handshakeTimeoutMs);
     TlsProtocols tlsProtocols = TlsProtocols.defaultProtocols(_tlsV10Enabled);
-    return new JsonAsyncHttpPinotClientTransport(_headers, _scheme, _extraOptionString, _useMultiStageEngine,
+    return new JsonAsyncHttpPinotClientTransport(_headers, _scheme, _extraOptionString, _useMultistageEngine,
         _sslContext, connectionTimeouts, tlsProtocols, _appId);
   }
 
@@ -104,7 +104,7 @@ public class JsonAsyncHttpPinotClientTransportFactory implements PinotClientTran
         System.getProperties().getProperty("broker.tlsV10Enabled", DEFAULT_BROKER_TLS_V10_ENABLED));
 
     _extraOptionString = properties.getProperty("queryOptions", "");
-    _useMultiStageEngine = Boolean.parseBoolean(properties.getProperty("useMultiStageEngine", "false"));
+    _useMultistageEngine = Boolean.parseBoolean(properties.getProperty("useMultistageEngine", "false"));
     return this;
   }
 }
