@@ -155,6 +155,20 @@ public class TableCache implements PinotConfigProvider {
     return _tableNameMap;
   }
 
+  /**
+   * Get all dimension table names.
+   * @return List of dimension table names
+   */
+  public List<String> getAllDimensionTables() {
+    List<String> dimensionTables = new ArrayList<>();
+    for (TableConfigInfo tableConfigInfo : _tableConfigInfoMap.values()) {
+      if (tableConfigInfo._tableConfig.isDimTable()) {
+        dimensionTables.add(tableConfigInfo._tableConfig.getTableName());
+      }
+    }
+    return dimensionTables;
+  }
+
   /**
    * Returns a map from column name to actual column name for the given table, or {@code null} if the table schema does
    * not exist. For case-insensitive case, the keys of the map are in lower case.
