@@ -94,6 +94,14 @@ public class SegmentZKMetadata implements ZKMetadata {
     return _endTimeMs;
   }
 
+  public String getRawStartTime() {
+    return _simpleFields.get(Segment.RAW_START_TIME);
+  }
+
+  public String getRawEndTime() {
+    return _simpleFields.get(Segment.RAW_END_TIME);
+  }
+
   public void setStartTime(long startTime) {
     setNonNegativeValue(Segment.START_TIME, startTime);
     _startTimeMsCached = false;
@@ -104,6 +112,14 @@ public class SegmentZKMetadata implements ZKMetadata {
     _endTimeMsCached = false;
   }
 
+  public void setRawStartTime(String startTime) {
+    setValue(Segment.RAW_START_TIME, startTime);
+  }
+
+  public void setRawEndTime(String endTime) {
+    setValue(Segment.RAW_END_TIME, endTime);
+  }
+
   public void setTimeUnit(TimeUnit timeUnit) {
     setValue(Segment.TIME_UNIT, timeUnit);
     _startTimeMsCached = false;
@@ -142,6 +158,18 @@ public class SegmentZKMetadata implements ZKMetadata {
     setNonNegativeValue(Segment.CRC, crc);
   }
 
+  public String getTier() {
+    return _simpleFields.get(Segment.TIER);
+  }
+
+  public void setTier(String tier) {
+    setValue(Segment.TIER, tier);
+  }
+
+  /**
+   * For uploaded segment, this is the time when the segment file is created. For real-time segment, this is the time
+   * when the consuming segment is created.
+   */
   public long getCreationTime() {
     return _znRecord.getLongField(Segment.CREATION_TIME, -1);
   }
@@ -150,6 +178,10 @@ public class SegmentZKMetadata implements ZKMetadata {
     setNonNegativeValue(Segment.CREATION_TIME, creationTime);
   }
 
+  /**
+   * Push time exists only for uploaded segments. It is the time when the segment is first pushed to the cluster (i.e.
+   * when the segment ZK metadata is created).
+   */
   public long getPushTime() {
     String pushTimeString = _simpleFields.get(Segment.PUSH_TIME);
     // Handle legacy push time key
@@ -164,6 +196,10 @@ public class SegmentZKMetadata implements ZKMetadata {
     setNonNegativeValue(Segment.PUSH_TIME, pushTime);
   }
 
+  /**
+   * Refresh time exists only for uploaded segments that have been replaced. It is the time when the segment is last
+   * replaced.
+   */
   public long getRefreshTime() {
     String refreshTimeString = _simpleFields.get(Segment.REFRESH_TIME);
     // Handle legacy refresh time key
