@@ -101,9 +101,15 @@ public enum ControllerGauge implements AbstractMetrics.Gauge {
   // Number of Tasks Status
   TASK_STATUS("taskStatus", false),
 
-  // Number of dropped minion instances
+  // Number of dropped stale minion instances
   DROPPED_MINION_INSTANCES("droppedMinionInstances", true),
 
+  // Number of dropped stale broker instances
+  DROPPED_BROKER_INSTANCES("droppedBrokerInstances", true),
+
+  // Number of dropped stale server instances
+  DROPPED_SERVER_INSTANCES("droppedServerInstances", true),
+
   // Number of online minion instances
   ONLINE_MINION_INSTANCES("onlineMinionInstances", true),
 
@@ -126,7 +132,30 @@ public enum ControllerGauge implements AbstractMetrics.Gauge {
   MAX_RECORDS_LAG("maxRecordsLag", false),
 
   // Consumption availability lag in ms at a partition level
-  MAX_RECORD_AVAILABILITY_LAG_MS("maxRecordAvailabilityLagMs", false);
+  MAX_RECORD_AVAILABILITY_LAG_MS("maxRecordAvailabilityLagMs", false),
+
+  // Number of table schema got misconfigured
+  MISCONFIGURED_SCHEMA_TABLE_COUNT("misconfiguredSchemaTableCount", true),
+
+  // Number of table without schema
+  TABLE_WITHOUT_SCHEMA_COUNT("tableWithoutSchemaCount", true),
+
+  // Number of table schema got fixed
+  FIXED_SCHEMA_TABLE_COUNT("fixedSchemaTableCount", true),
+
+  // Number of tables that we want to fix but failed to copy schema from old schema name to new schema name
+  FAILED_TO_COPY_SCHEMA_COUNT("failedToCopySchemaCount", true),
+
+  // Number of tables that we want to fix but failed to update table config
+  FAILED_TO_UPDATE_TABLE_CONFIG_COUNT("failedToUpdateTableConfigCount", true),
+
+  LLC_SEGMENTS_DEEP_STORE_UPLOAD_RETRY_QUEUE_SIZE("LLCSegmentDeepStoreUploadRetryQueueSize", false),
+
+  TABLE_CONSUMPTION_PAUSED("tableConsumptionPaused", false),
+
+  TABLE_DISABLED("tableDisabled", false),
+
+  TABLE_REBALANCE_IN_PROGRESS("tableRebalanceInProgress", false);
 
   private final String _gaugeName;
   private final String _unit;
