@@ -18,6 +18,7 @@
  */
 package org.apache.pinot.common.utils.fetcher;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.net.InetAddresses;
 import java.io.File;
 import java.io.IOException;
@@ -48,6 +49,20 @@ public class HttpSegmentFetcher extends BaseSegmentFetcher {
     _httpClient = new FileUploadDownloadClient(HttpClientConfig.newBuilder(config).build());
   }
 
+  public HttpSegmentFetcher() {
+  }
+
+  @VisibleForTesting
+  protected HttpSegmentFetcher(FileUploadDownloadClient httpClient, PinotConfiguration config) {
+    _httpClient = httpClient;
+    _retryCount = config.getProperty(RETRY_COUNT_CONFIG_KEY, DEFAULT_RETRY_COUNT);
+    _retryWaitMs = config.getProperty(RETRY_WAIT_MS_CONFIG_KEY, DEFAULT_RETRY_WAIT_MS);
+    _retryDelayScaleFactor = config.getProperty(RETRY_DELAY_SCALE_FACTOR_CONFIG_KEY, DEFAULT_RETRY_DELAY_SCALE_FACTOR);
+    _logger
+        .info("Initialized with retryCount: {}, retryWaitMs: {}, retryDelayScaleFactor: {}", _retryCount, _retryWaitMs,
+            _retryDelayScaleFactor);
+  }
+
   @Override
   public void fetchSegmentToLocal(URI downloadURI, File dest)
       throws Exception {
@@ -174,8 +189,12 @@ public class HttpSegmentFetcher extends BaseSegmentFetcher {
       throws Exception {
     try {
       int statusCode = _httpClient.downloadFile(uri, dest, _authProvider);
-      _logger.info("Downloaded segment from: {} to: {} of size: {}; Response status code: {}", uri, dest, dest.length(),
-          statusCode);
+      _logger.info("Try to download the segment from: {} to: {} of size: {}; Response status code: {}", uri, dest,
+          dest.length(), statusCode);
+      // In case of download failure, throw exception.
+      if (statusCode >= 300) {
+        throw new HttpErrorStatusException("Failed to download segment", statusCode);
+      }
     } catch (Exception e) {
       _logger.warn("Caught exception while downloading segment from: {} to: {}", uri, dest, e);
       throw e;
