@@ -53,6 +53,7 @@ import org.apache.http.entity.ContentType;
 import org.apache.http.entity.StringEntity;
 import org.apache.http.entity.mime.MultipartEntityBuilder;
 import org.apache.http.impl.client.CloseableHttpClient;
+import org.apache.http.impl.client.HttpClientBuilder;
 import org.apache.http.impl.client.HttpClients;
 import org.apache.http.util.EntityUtils;
 import org.apache.pinot.common.auth.AuthProviderUtils;
@@ -86,14 +87,18 @@ public class HttpClient implements AutoCloseable {
   private final CloseableHttpClient _httpClient;
 
   public HttpClient() {
-    this(null);
+    this(HttpClientConfig.DEFAULT_HTTP_CLIENT_CONFIG, null);
   }
 
   public HttpClient(@Nullable SSLContext sslContext) {
+    this(HttpClientConfig.DEFAULT_HTTP_CLIENT_CONFIG, sslContext);
+  }
+
+  public HttpClient(HttpClientConfig httpClientConfig, @Nullable SSLContext sslContext) {
     SSLContext context = sslContext != null ? sslContext : TlsUtils.getSslContext();
     // Set NoopHostnameVerifier to skip validating hostname when uploading/downloading segments.
     SSLConnectionSocketFactory csf = new SSLConnectionSocketFactory(context, NoopHostnameVerifier.INSTANCE);
-    _httpClient = HttpClients.custom().setSSLSocketFactory(csf).build();
+    _httpClient = buildCloseableHttpClient(httpClientConfig, csf);
   }
 
   public static HttpClient getInstance() {
@@ -101,7 +106,8 @@ public class HttpClient implements AutoCloseable {
   }
 
   private static final class HttpClientHolder {
-    static final HttpClient HTTP_CLIENT = new HttpClient(TlsUtils.getSslContext());
+    static final HttpClient HTTP_CLIENT =
+        new HttpClient(HttpClientConfig.DEFAULT_HTTP_CLIENT_CONFIG, TlsUtils.getSslContext());
   }
 
   // --------------------------------------------------------------------------
@@ -464,6 +470,18 @@ public class HttpClient implements AutoCloseable {
     requestBuilder.setConfig(requestConfig);
   }
 
+  private static CloseableHttpClient buildCloseableHttpClient(HttpClientConfig httpClientConfig,
+      SSLConnectionSocketFactory csf) {
+    HttpClientBuilder httpClientBuilder = HttpClients.custom().setSSLSocketFactory(csf);
+    if (httpClientConfig.getMaxConnTotal() > 0) {
+      httpClientBuilder.setMaxConnTotal(httpClientConfig.getMaxConnTotal());
+    }
+    if (httpClientConfig.getMaxConnPerRoute() > 0) {
+      httpClientBuilder.setMaxConnPerRoute(httpClientConfig.getMaxConnPerRoute());
+    }
+    return httpClientBuilder.build();
+  }
+
   private static String getErrorMessage(HttpUriRequest request, CloseableHttpResponse response) {
     String controllerHost = null;
     String controllerVersion = null;
@@ -475,9 +493,13 @@ public class HttpClient implements AutoCloseable {
     String reason;
     try {
       String entityStr = EntityUtils.toString(response.getEntity());
-      reason = JsonUtils.stringToObject(entityStr, SimpleHttpErrorInfo.class).getError();
+      try {
+        reason = JsonUtils.stringToObject(entityStr, SimpleHttpErrorInfo.class).getError();
+      } catch (Exception e) {
+        reason = entityStr;
+      }
     } catch (Exception e) {
-      reason = String.format("Failed to get a reason, exception: %s", e.toString());
+      reason = String.format("Failed to get a reason, exception: %s", e);
     }
     String errorMessage = String.format("Got error status code: %d (%s) with reason: \"%s\" while sending request: %s",
         statusLine.getStatusCode(), statusLine.getReasonPhrase(), reason, request.getURI());
