@@ -114,6 +114,30 @@ public class TierSegmentSelectorTest {
     Assert.assertFalse(segmentSelector.selectSegment(tableNameWithType, segmentName));
   }
 
+  @Test
+  public void testRealTimeConsumingSegmentShouldNotBeRelocated() {
+
+    long now = System.currentTimeMillis();
+
+    String segmentName = "myTable__4__1__" + now;
+    String tableNameWithType = "myTable_REALTIME";
+    SegmentZKMetadata realtimeSegmentZKMetadata = new SegmentZKMetadata(segmentName);
+    realtimeSegmentZKMetadata.setStatus(CommonConstants.Segment.Realtime.Status.IN_PROGRESS);
+
+    ZNRecord segmentZKMetadataZNRecord = realtimeSegmentZKMetadata.toZNRecord();
+
+    ZkHelixPropertyStore<ZNRecord> propertyStore = mock(ZkHelixPropertyStore.class);
+    when(propertyStore
+            .get(eq(ZKMetadataProvider.constructPropertyStorePathForSegment(tableNameWithType, segmentName)), any(),
+                    anyInt())).thenReturn(segmentZKMetadataZNRecord);
+
+    HelixManager helixManager = mock(HelixManager.class);
+    when(helixManager.getHelixPropertyStore()).thenReturn(propertyStore);
+
+    TimeBasedTierSegmentSelector segmentSelector = new TimeBasedTierSegmentSelector(helixManager, "7d");
+    Assert.assertFalse(segmentSelector.selectSegment(tableNameWithType, segmentName));
+  }
+
   @Test
   public void testFixedSegmentSelector() {
 
