@@ -36,21 +36,21 @@ import javax.ws.rs.Path;
 import javax.ws.rs.Produces;
 import javax.ws.rs.QueryParam;
 import javax.ws.rs.core.MediaType;
+import org.apache.commons.configuration2.ex.ConfigurationException;
 import org.apache.commons.io.FileUtils;
 import org.apache.pinot.common.metrics.ControllerGauge;
 import org.apache.pinot.common.protocols.SegmentCompletionProtocol;
 import org.apache.pinot.common.utils.LLCSegmentName;
 import org.apache.pinot.common.utils.TarGzCompressionUtils;
 import org.apache.pinot.common.utils.URIUtils;
-import org.apache.pinot.common.utils.fetcher.SegmentFetcherFactory;
 import org.apache.pinot.controller.api.access.AccessType;
 import org.apache.pinot.controller.api.access.Authenticate;
 import org.apache.pinot.controller.helix.core.realtime.SegmentCompletionManager;
 import org.apache.pinot.controller.helix.core.realtime.segment.CommittingSegmentDescriptor;
-import org.apache.pinot.controller.util.SegmentCompletionUtils;
 import org.apache.pinot.core.auth.Actions;
 import org.apache.pinot.core.auth.Authorize;
 import org.apache.pinot.core.auth.TargetType;
+import org.apache.pinot.core.data.manager.realtime.SegmentCompletionUtils;
 import org.apache.pinot.segment.spi.V1Constants;
 import org.apache.pinot.segment.spi.index.metadata.SegmentMetadataImpl;
 import org.apache.pinot.spi.filesystem.PinotFS;
@@ -214,57 +214,8 @@ public class LLCSegmentCompletionHandlers {
     return responseStr;
   }
 
-  // TODO: remove this API. Should not allow committing without metadata
-  @GET
-  @Path(SegmentCompletionProtocol.MSG_TYPE_COMMIT_END)
-  @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.GET_ADMIN_INFO)
-  @Produces(MediaType.APPLICATION_JSON)
-  public String segmentCommitEnd(@QueryParam(SegmentCompletionProtocol.PARAM_INSTANCE_ID) String instanceId,
-      @QueryParam(SegmentCompletionProtocol.PARAM_SEGMENT_NAME) String segmentName,
-      @QueryParam(SegmentCompletionProtocol.PARAM_SEGMENT_LOCATION) String segmentLocation,
-      @QueryParam(SegmentCompletionProtocol.PARAM_OFFSET) long offset,
-      @QueryParam(SegmentCompletionProtocol.PARAM_STREAM_PARTITION_MSG_OFFSET) String streamPartitionMsgOffset,
-      @QueryParam(SegmentCompletionProtocol.PARAM_MEMORY_USED_BYTES) long memoryUsedBytes,
-      @QueryParam(SegmentCompletionProtocol.PARAM_BUILD_TIME_MILLIS) long buildTimeMillis,
-      @QueryParam(SegmentCompletionProtocol.PARAM_WAIT_TIME_MILLIS) long waitTimeMillis,
-      @QueryParam(SegmentCompletionProtocol.PARAM_ROW_COUNT) int numRows,
-      @QueryParam(SegmentCompletionProtocol.PARAM_SEGMENT_SIZE_BYTES) long segmentSizeBytes) {
-    if (instanceId == null || segmentName == null || segmentLocation == null || (offset == -1
-        && streamPartitionMsgOffset == null)) {
-      LOGGER.error(
-          "Invalid call: offset={}, segmentName={}, instanceId={}, segmentLocation={}, streamPartitionMsgOffset={}",
-          offset, segmentName, instanceId, segmentLocation, streamPartitionMsgOffset);
-      // TODO: memoryUsedInBytes = 0 if not present in params. Add validation when we start using it
-      return SegmentCompletionProtocol.RESP_FAILED.toJsonString();
-    }
-
-    SegmentMetadataImpl segmentMetadata;
-    try {
-      segmentMetadata = extractMetadataFromSegmentFileURI(new URI(segmentLocation), segmentName);
-    } catch (Exception e) {
-      LOGGER.error("Caught exception while extracting metadata from segment: {} at location: {} from instance: {}",
-          segmentName, segmentLocation, instanceId, e);
-      return SegmentCompletionProtocol.RESP_FAILED.toJsonString();
-    }
-    SegmentCompletionProtocol.Request.Params requestParams = new SegmentCompletionProtocol.Request.Params();
-    requestParams.withInstanceId(instanceId).withSegmentName(segmentName).withSegmentLocation(segmentLocation)
-        .withSegmentSizeBytes(segmentSizeBytes).withBuildTimeMillis(buildTimeMillis).withWaitTimeMillis(waitTimeMillis)
-        .withNumRows(numRows).withMemoryUsedBytes(memoryUsedBytes);
-    extractOffsetFromParams(requestParams, streamPartitionMsgOffset, offset);
-    LOGGER.info("Processing segmentCommitEnd:{}", requestParams.toString());
-
-    final boolean isSuccess = true;
-    final boolean isSplitCommit = true;
-
-    CommittingSegmentDescriptor committingSegmentDescriptor =
-        CommittingSegmentDescriptor.fromSegmentCompletionReqParamsAndMetadata(requestParams, segmentMetadata);
-    SegmentCompletionProtocol.Response response = _segmentCompletionManager
-        .segmentCommitEnd(requestParams, isSuccess, isSplitCommit, committingSegmentDescriptor);
-    final String responseStr = response.toJsonString();
-    LOGGER.info("Response to segmentCommitEnd for segment:{} is:{}", segmentName, responseStr);
-    return responseStr;
-  }
-
+  // Remove after releasing 1.1 (server always use split commit)
+  @Deprecated
   @POST
   @Path(SegmentCompletionProtocol.MSG_TYPE_COMMIT)
   @Authorize(targetType = TargetType.CLUSTER, action = Actions.Cluster.COMMIT_SEGMENT)
@@ -343,8 +294,8 @@ public class LLCSegmentCompletionHandlers {
     }
 
     response = segmentCompletionManager.segmentCommitEnd(requestParams, success, false, committingSegmentDescriptor);
-    LOGGER.info("Response to segmentCommit: instance={}  segment={} status={} offset={}, streamMsgOffset={}",
-        requestParams.getInstanceId(), requestParams.getSegmentName(), response.getStatus(), response.getOffset(),
+    LOGGER.info("Response to segmentCommit: instance={}, segment={}, status={}, streamMsgOffset={}",
+        requestParams.getInstanceId(), requestParams.getSegmentName(), response.getStatus(),
         response.getStreamPartitionMsgOffset());
 
     return response.toJsonString();
@@ -378,7 +329,7 @@ public class LLCSegmentCompletionHandlers {
       String rawTableName = new LLCSegmentName(segmentName).getTableName();
       URI segmentFileURI = URIUtils
           .getUri(ControllerFilePathProvider.getInstance().getDataDirURI().toString(), rawTableName,
-              URIUtils.encode(SegmentCompletionUtils.generateSegmentFileName(segmentName)));
+              URIUtils.encode(SegmentCompletionUtils.generateTmpSegmentFileName(segmentName)));
       PinotFSFactory.create(segmentFileURI.getScheme()).copyFromLocalFile(localTempFile, segmentFileURI);
       SegmentCompletionProtocol.Response.Params responseParams = new SegmentCompletionProtocol.Response.Params()
           .withStreamPartitionMsgOffset(requestParams.getStreamPartitionMsgOffset())
@@ -412,6 +363,7 @@ public class LLCSegmentCompletionHandlers {
       @QueryParam(SegmentCompletionProtocol.PARAM_WAIT_TIME_MILLIS) long waitTimeMillis,
       @QueryParam(SegmentCompletionProtocol.PARAM_ROW_COUNT) int numRows,
       @QueryParam(SegmentCompletionProtocol.PARAM_SEGMENT_SIZE_BYTES) long segmentSizeBytes,
+      @QueryParam(SegmentCompletionProtocol.PARAM_REASON) String stopReason,
       FormDataMultiPart metadataFiles) {
     if (instanceId == null || segmentName == null || segmentLocation == null || metadataFiles == null || (offset == -1
         && streamPartitionMsgOffset == null)) {
@@ -425,7 +377,7 @@ public class LLCSegmentCompletionHandlers {
     SegmentCompletionProtocol.Request.Params requestParams = new SegmentCompletionProtocol.Request.Params();
     requestParams.withInstanceId(instanceId).withSegmentName(segmentName).withSegmentLocation(segmentLocation)
         .withSegmentSizeBytes(segmentSizeBytes).withBuildTimeMillis(buildTimeMillis).withWaitTimeMillis(waitTimeMillis)
-        .withNumRows(numRows).withMemoryUsedBytes(memoryUsedBytes);
+        .withNumRows(numRows).withMemoryUsedBytes(memoryUsedBytes).withReason(stopReason);
     extractOffsetFromParams(requestParams, streamPartitionMsgOffset, offset);
     LOGGER.info("Processing segmentCommitEndWithMetadata:{}", requestParams.toString());
 
@@ -508,7 +460,7 @@ public class LLCSegmentCompletionHandlers {
    * temporarily.
    */
   private static SegmentMetadataImpl extractSegmentMetadataFromForm(FormDataMultiPart form, String segmentName)
-      throws IOException {
+      throws IOException, ConfigurationException {
     File tempIndexDir = org.apache.pinot.common.utils.FileUtils.concatAndValidateFile(
         ControllerFilePathProvider.getInstance().getUntarredFileTempDir(), getTempSegmentFileName(segmentName),
         "Invalid segment name: %s", segmentName);
@@ -542,25 +494,6 @@ public class LLCSegmentCompletionHandlers {
     }
   }
 
-  /**
-   * Extracts the segment metadata from the segment file URI (could be remote). Copy the segment file to local under
-   * file upload temporary directory first, then use the untarred file temporary directory to store the metadata files
-   * temporarily.
-   */
-  private static SegmentMetadataImpl extractMetadataFromSegmentFileURI(URI segmentFileURI, String segmentName)
-      throws Exception {
-    File localTempFile = org.apache.pinot.common.utils.FileUtils.concatAndValidateFile(
-        ControllerFilePathProvider.getInstance().getFileUploadTempDir(), getTempSegmentFileName(segmentName),
-        "Invalid segment name: %s", segmentName);
-
-    try {
-      SegmentFetcherFactory.fetchSegmentToLocal(segmentFileURI, localTempFile);
-      return extractMetadataFromLocalSegmentFile(localTempFile);
-    } finally {
-      FileUtils.deleteQuietly(localTempFile);
-    }
-  }
-
   private static String getTempSegmentFileName(String segmentName) {
     return segmentName + "." + UUID.randomUUID();
   }
