@@ -57,6 +57,7 @@ import org.apache.helix.model.InstanceConfig;
 import org.apache.pinot.common.Utils;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.response.ProcessingException;
+import org.apache.pinot.common.response.broker.BrokerResponseNative;
 import org.apache.pinot.common.utils.config.TagNameUtils;
 import org.apache.pinot.common.utils.request.RequestUtils;
 import org.apache.pinot.controller.ControllerConf;
@@ -107,6 +108,10 @@ public class PinotQueryResource {
   public String handlePostSql(String requestJsonStr, @Context HttpHeaders httpHeaders) {
     try {
       JsonNode requestJson = JsonUtils.stringToJsonNode(requestJsonStr);
+      if (!requestJson.has("sql")) {
+        return constructQueryExceptionResponse(QueryException.getException(QueryException.JSON_PARSING_ERROR,
+                "JSON Payload is missing the query string field 'sql'"));
+      }
       String sqlQuery = requestJson.get("sql").asText();
       String traceEnabled = "false";
       if (requestJson.has("trace")) {
@@ -120,13 +125,13 @@ public class PinotQueryResource {
       return executeSqlQuery(httpHeaders, sqlQuery, traceEnabled, queryOptions, "/sql");
     } catch (ProcessingException pe) {
       LOGGER.error("Caught exception while processing post request {}", pe.getMessage());
-      return pe.getMessage();
+      return constructQueryExceptionResponse(pe);
     } catch (WebApplicationException wae) {
       LOGGER.error("Caught exception while processing post request", wae);
       throw wae;
     } catch (Exception e) {
       LOGGER.error("Caught exception while processing post request", e);
-      return QueryException.getException(QueryException.INTERNAL_ERROR, e).toString();
+      return constructQueryExceptionResponse(QueryException.getException(QueryException.INTERNAL_ERROR, e));
     }
   }
 
@@ -140,13 +145,13 @@ public class PinotQueryResource {
       return executeSqlQuery(httpHeaders, sqlQuery, traceEnabled, queryOptions, "/sql");
     } catch (ProcessingException pe) {
       LOGGER.error("Caught exception while processing get request {}", pe.getMessage());
-      return pe.getMessage();
+      return constructQueryExceptionResponse(pe);
     } catch (WebApplicationException wae) {
       LOGGER.error("Caught exception while processing get request", wae);
       throw wae;
     } catch (Exception e) {
       LOGGER.error("Caught exception while processing get request", e);
-      return QueryException.getException(QueryException.INTERNAL_ERROR, e).toString();
+      return constructQueryExceptionResponse(QueryException.getException(QueryException.INTERNAL_ERROR, e));
     }
   }
 
@@ -171,8 +176,7 @@ public class PinotQueryResource {
           CommonConstants.Helix.DEFAULT_MULTI_STAGE_ENGINE_ENABLED)) {
         return getMultiStageQueryResponse(sqlQuery, queryOptions, httpHeaders, endpointUrl, traceEnabled);
       } else {
-        throw new UnsupportedOperationException("V2 Multi-Stage query engine not enabled. "
-            + "Please see https://docs.pinot.apache.org/ for instruction to enable V2 engine.");
+        throw QueryException.getException(QueryException.INTERNAL_ERROR, "V2 Multi-Stage query engine not enabled.");
       }
     } else {
       PinotSqlType sqlType = sqlNodeAndOptions.getSqlType();
@@ -186,7 +190,7 @@ public class PinotQueryResource {
                   .collect(Collectors.toMap(Pair::getKey, Pair::getValue));
           return _sqlQueryExecutor.executeDMLStatement(sqlNodeAndOptions, headers).toJsonString();
         default:
-          throw new UnsupportedOperationException("Unsupported SQL type - " + sqlType);
+          throw QueryException.getException(QueryException.INTERNAL_ERROR, "Unsupported SQL type - " + sqlType);
       }
     }
   }
@@ -203,7 +207,13 @@ public class PinotQueryResource {
 
     QueryEnvironment queryEnvironment = new QueryEnvironment(new TypeFactory(new TypeSystem()),
         CalciteSchemaBuilder.asRootSchema(new PinotCatalog(_pinotHelixResourceManager.getTableCache())), null, null);
-    List<String> tableNames = queryEnvironment.getTableNamesForQuery(query);
+    List<String> tableNames;
+    try {
+      tableNames = queryEnvironment.getTableNamesForQuery(query);
+    } catch (Exception e) {
+      return QueryException.getException(QueryException.SQL_PARSING_ERROR,
+          new Exception("Unable to find table for this query", e)).toString();
+    }
     List<String> instanceIds;
     if (tableNames.size() != 0) {
       List<TableConfig> tableConfigList = getListTableConfigs(tableNames);
@@ -465,4 +475,13 @@ public class PinotQueryResource {
       throw new AssertionError("Should not reach this");
     }
   }
+
+  private static String constructQueryExceptionResponse(ProcessingException pe) {
+    try {
+      return new BrokerResponseNative(pe).toJsonString();
+    } catch (IOException ioe) {
+      Utils.rethrowException(ioe);
+      throw new AssertionError("Should not reach this");
+    }
+  }
 }
