@@ -18,12 +18,9 @@
  */
 package org.apache.pinot.controller.helix.core.realtime.segment;
 
-import com.google.common.annotations.VisibleForTesting;
-import java.util.List;
 import javax.annotation.Nullable;
 import org.apache.pinot.common.metadata.segment.SegmentZKMetadata;
-import org.apache.pinot.spi.stream.PartitionGroupMetadata;
-import org.apache.pinot.spi.stream.PartitionLevelStreamConfig;
+import org.apache.pinot.spi.stream.StreamConfig;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -33,32 +30,25 @@ import org.slf4j.LoggerFactory;
  * previous segment
  * The formula used to compute new number of rows is:
  * targetNumRows = ideal_segment_size * (a * current_rows_to_size_ratio + b * previous_rows_to_size_ratio)
- * where a = 0.25, b = 0.75, prev ratio= ratio collected over all previous segment completions
+ * where a = 0.1, b = 0.9, prev ratio= ratio collected over all previous segment completions
  * This ensures that we take into account the history of the segment size and number rows
  */
 public class SegmentSizeBasedFlushThresholdUpdater implements FlushThresholdUpdater {
   public static final Logger LOGGER = LoggerFactory.getLogger(SegmentSizeBasedFlushThresholdUpdater.class);
   private final SegmentFlushThresholdComputer _flushThresholdComputer;
 
-  @VisibleForTesting
-  double getLatestSegmentRowsToSizeRatio() {
-    return _flushThresholdComputer.getLatestSegmentRowsToSizeRatio();
-  }
-
   public SegmentSizeBasedFlushThresholdUpdater() {
     _flushThresholdComputer = new SegmentFlushThresholdComputer();
   }
 
   // synchronized since this method could be called for multiple partitions of the same table in different threads
   @Override
-  public synchronized void updateFlushThreshold(PartitionLevelStreamConfig streamConfig,
-      SegmentZKMetadata newSegmentZKMetadata, CommittingSegmentDescriptor committingSegmentDescriptor,
-      @Nullable SegmentZKMetadata committingSegmentZKMetadata, int maxNumPartitionsPerInstance,
-      List<PartitionGroupMetadata> partitionGroupMetadataList) {
-
-    int threshold = _flushThresholdComputer.computeThreshold(streamConfig, committingSegmentDescriptor,
-        committingSegmentZKMetadata, partitionGroupMetadataList, newSegmentZKMetadata.getSegmentName());
-
+  public synchronized void updateFlushThreshold(StreamConfig streamConfig, SegmentZKMetadata newSegmentZKMetadata,
+      CommittingSegmentDescriptor committingSegmentDescriptor, @Nullable SegmentZKMetadata committingSegmentZKMetadata,
+      int maxNumPartitionsPerInstance) {
+    int threshold =
+        _flushThresholdComputer.computeThreshold(streamConfig, committingSegmentDescriptor, committingSegmentZKMetadata,
+            newSegmentZKMetadata.getSegmentName());
     newSegmentZKMetadata.setSizeThresholdToFlushSegment(threshold);
   }
 }
