@@ -36,4 +36,14 @@ public class NoOpTableRebalanceObserver implements TableRebalanceObserver {
   @Override
   public void onError(String errorMsg) {
   }
+
+  @Override
+  public boolean isStopped() {
+    return false;
+  }
+
+  @Override
+  public RebalanceResult.Status getStopStatus() {
+    throw new UnsupportedOperationException();
+  }
 }
