@@ -35,6 +35,7 @@ import org.apache.commons.collections.CollectionUtils;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang.math.RandomUtils;
 import org.apache.helix.ConfigAccessor;
 import org.apache.helix.HelixAdmin;
 import org.apache.helix.HelixDataAccessor;
@@ -88,6 +89,7 @@ import static org.apache.pinot.spi.utils.CommonConstants.Helix.UNTAGGED_SERVER_I
 import static org.apache.pinot.spi.utils.CommonConstants.Server.DEFAULT_ADMIN_API_PORT;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertNotNull;
+import static org.testng.Assert.assertTrue;
 import static org.testng.Assert.fail;
 
 
@@ -210,7 +212,8 @@ public class ControllerTest {
     Map<String, Object> properties = new HashMap<>();
 
     properties.put(ControllerConf.CONTROLLER_HOST, LOCAL_HOST);
-    properties.put(ControllerConf.CONTROLLER_PORT, NetUtils.findOpenPort(DEFAULT_CONTROLLER_PORT));
+    properties.put(ControllerConf.CONTROLLER_PORT,
+        NetUtils.findOpenPort(DEFAULT_CONTROLLER_PORT + RandomUtils.nextInt(10000)));
     properties.put(ControllerConf.DATA_DIR, DEFAULT_DATA_DIR);
     properties.put(ControllerConf.LOCAL_TEMP_DIR, DEFAULT_LOCAL_TEMP_DIR);
     properties.put(ControllerConf.ZK_STR, getZkUrl());
@@ -244,6 +247,8 @@ public class ControllerTest {
     _controllerPort = DEFAULT_CONTROLLER_PORT;
     if (StringUtils.isNotBlank(_controllerConfig.getControllerPort())) {
       _controllerPort = Integer.parseInt(_controllerConfig.getControllerPort());
+    } else if (StringUtils.isNotBlank(_controllerConfig.getControllerVipPort())) {
+      _controllerPort = Integer.parseInt(_controllerConfig.getControllerVipPort());
     }
 
     _controllerBaseApiUrl = controllerScheme + "://localhost:" + _controllerPort;
@@ -288,6 +293,7 @@ public class ControllerTest {
     Preconditions.checkState(_controllerStarter != null);
     _controllerStarter.stop();
     _controllerStarter = null;
+    _controllerRequestClient = null;
     FileUtils.deleteQuietly(new File(_controllerDataDir));
   }
 
@@ -391,7 +397,8 @@ public class ControllerTest {
   public void addFakeServerInstancesToAutoJoinHelixCluster(int numInstances, boolean isSingleTenant, int baseAdminPort)
       throws Exception {
     for (int i = 0; i < numInstances; i++) {
-      addFakeServerInstanceToAutoJoinHelixCluster(SERVER_INSTANCE_ID_PREFIX + i, isSingleTenant, baseAdminPort + i);
+      addFakeServerInstanceToAutoJoinHelixCluster(SERVER_INSTANCE_ID_PREFIX + i, isSingleTenant,
+          NetUtils.findOpenPort(baseAdminPort + i + RandomUtils.nextInt(10000)));
     }
   }
 
@@ -402,7 +409,8 @@ public class ControllerTest {
 
   public void addFakeServerInstanceToAutoJoinHelixCluster(String instanceId, boolean isSingleTenant)
       throws Exception {
-    addFakeServerInstanceToAutoJoinHelixCluster(instanceId, isSingleTenant, Server.DEFAULT_ADMIN_API_PORT);
+    addFakeServerInstanceToAutoJoinHelixCluster(instanceId, isSingleTenant,
+        NetUtils.findOpenPort(Server.DEFAULT_ADMIN_API_PORT + RandomUtils.nextInt(10000)));
   }
 
   public void addFakeServerInstanceToAutoJoinHelixCluster(String instanceId, boolean isSingleTenant, int adminPort)
@@ -430,7 +438,8 @@ public class ControllerTest {
   /** Add fake server instances until total number of server instances reaches maxCount */
   public void addMoreFakeServerInstancesToAutoJoinHelixCluster(int maxCount, boolean isSingleTenant)
       throws Exception {
-    addMoreFakeServerInstancesToAutoJoinHelixCluster(maxCount, isSingleTenant, DEFAULT_ADMIN_API_PORT);
+    addMoreFakeServerInstancesToAutoJoinHelixCluster(maxCount, isSingleTenant,
+        NetUtils.findOpenPort(DEFAULT_ADMIN_API_PORT + RandomUtils.nextInt(10000)));
   }
 
   /** Add fake server instances until total number of server instances reaches maxCount */
@@ -599,7 +608,8 @@ public class ControllerTest {
 
   public void stopAndDropFakeInstance(String instanceId) {
     stopFakeInstance(instanceId);
-    _helixResourceManager.dropInstance(instanceId);
+    TestUtils.waitForCondition(aVoid -> _helixResourceManager.dropInstance(instanceId).isSuccessful(), 60_000L,
+        "Failed to drop fake instance: " + instanceId);
   }
 
   public static Schema createDummySchema(String tableName) {
@@ -680,6 +690,11 @@ public class ControllerTest {
     getControllerRequestClient().deleteTable(TableNameBuilder.REALTIME.tableNameWithType(tableName));
   }
 
+  public void waitForEVToAppear(String tableNameWithType) {
+    TestUtils.waitForCondition(aVoid -> _helixResourceManager.getTableExternalView(tableNameWithType) != null, 60_000L,
+        "Failed to create the external view for table: " + tableNameWithType);
+  }
+
   public void waitForEVToDisappear(String tableNameWithType) {
     TestUtils.waitForCondition(aVoid -> _helixResourceManager.getTableExternalView(tableNameWithType) == null, 60_000L,
         "Failed to clean up the external view for table: " + tableNameWithType);
@@ -710,14 +725,14 @@ public class ControllerTest {
     return getControllerRequestClient().getTableSize(tableName);
   }
 
-  public void reloadOfflineTable(String tableName)
+  public String reloadOfflineTable(String tableName)
       throws IOException {
-    reloadOfflineTable(tableName, false);
+    return reloadOfflineTable(tableName, false);
   }
 
-  public void reloadOfflineTable(String tableName, boolean forceDownload)
+  public String reloadOfflineTable(String tableName, boolean forceDownload)
       throws IOException {
-    getControllerRequestClient().reloadTable(tableName, TableType.OFFLINE, forceDownload);
+    return getControllerRequestClient().reloadTable(tableName, TableType.OFFLINE, forceDownload);
   }
 
   public void reloadOfflineSegment(String tableName, String segmentName, boolean forceDownload)
@@ -725,9 +740,9 @@ public class ControllerTest {
     getControllerRequestClient().reloadSegment(tableName, segmentName, forceDownload);
   }
 
-  public void reloadRealtimeTable(String tableName)
+  public String reloadRealtimeTable(String tableName)
       throws IOException {
-    getControllerRequestClient().reloadTable(tableName, TableType.REALTIME, false);
+    return getControllerRequestClient().reloadTable(tableName, TableType.REALTIME, false);
   }
 
   public void createBrokerTenant(String tenantName, int numBrokers)
@@ -787,9 +802,10 @@ public class ControllerTest {
   }
 
   public static String sendPostRequest(String urlString)
-    throws IOException {
+      throws IOException {
     return sendPostRequest(urlString, null);
   }
+
   public static String sendPostRequest(String urlString, String payload)
       throws IOException {
     return sendPostRequest(urlString, payload, Collections.emptyMap());
@@ -1025,7 +1041,9 @@ public class ControllerTest {
         DEFAULT_NUM_SERVER_INSTANCES);
 
     // No pre-existing tables
-    assertEquals(getHelixResourceManager().getAllTables().size(), 0);
+    assertTrue(CollectionUtils.isEmpty(getHelixResourceManager().getAllTables()));
+    // No pre-existing schemas
+    assertTrue(CollectionUtils.isEmpty(getHelixResourceManager().getSchemaNames()));
   }
 
   /**
