@@ -28,20 +28,18 @@ import java.io.IOException;
 import java.net.URI;
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.UUID;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
 import javax.annotation.Nullable;
 import javax.annotation.concurrent.ThreadSafe;
-import org.apache.commons.configuration.Configuration;
-import org.apache.commons.configuration.ConfigurationConverter;
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.commons.lang3.tuple.Pair;
@@ -61,8 +59,6 @@ import org.apache.pinot.core.data.manager.offline.ImmutableSegmentDataManager;
 import org.apache.pinot.core.util.PeerServerSegmentFinder;
 import org.apache.pinot.segment.local.data.manager.SegmentDataManager;
 import org.apache.pinot.segment.local.data.manager.TableDataManager;
-import org.apache.pinot.segment.local.data.manager.TableDataManagerConfig;
-import org.apache.pinot.segment.local.data.manager.TableDataManagerParams;
 import org.apache.pinot.segment.local.indexsegment.immutable.ImmutableSegmentLoader;
 import org.apache.pinot.segment.local.segment.index.loader.IndexLoadingConfig;
 import org.apache.pinot.segment.local.segment.index.loader.LoaderUtils;
@@ -74,9 +70,9 @@ import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderContext;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderRegistry;
 import org.apache.pinot.segment.spi.store.SegmentDirectory;
 import org.apache.pinot.spi.auth.AuthProvider;
+import org.apache.pinot.spi.config.instance.InstanceDataManagerConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
 import org.apache.pinot.spi.data.Schema;
-import org.apache.pinot.spi.env.PinotConfiguration;
 import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.retry.AttemptsExceededException;
 import org.slf4j.Logger;
@@ -91,8 +87,10 @@ public abstract class BaseTableDataManager implements TableDataManager {
   // Semaphore to restrict the maximum number of parallel segment downloads for a table.
   private Semaphore _segmentDownloadSemaphore;
 
-  protected TableDataManagerConfig _tableDataManagerConfig;
+  protected InstanceDataManagerConfig _instanceDataManagerConfig;
   protected String _instanceId;
+  protected TableConfig _tableConfig;
+  protected HelixManager _helixManager;
   protected ZkHelixPropertyStore<ZNRecord> _propertyStore;
   protected ServerMetrics _serverMetrics;
   protected String _tableNameWithType;
@@ -100,9 +98,9 @@ public abstract class BaseTableDataManager implements TableDataManager {
   protected File _indexDir;
   protected File _resourceTmpDir;
   protected Logger _logger;
-  protected HelixManager _helixManager;
   protected ExecutorService _segmentPreloadExecutor;
   protected AuthProvider _authProvider;
+  protected String _peerDownloadScheme;
   protected long _streamSegmentDownloadUntarRateLimitBytesPerSec;
   protected boolean _isStreamSegmentDownloadUntar;
 
@@ -115,25 +113,22 @@ public abstract class BaseTableDataManager implements TableDataManager {
   protected volatile boolean _shutDown;
 
   @Override
-  public void init(TableDataManagerConfig tableDataManagerConfig, String instanceId,
-      ZkHelixPropertyStore<ZNRecord> propertyStore, ServerMetrics serverMetrics, HelixManager helixManager,
-      @Nullable ExecutorService segmentPreloadExecutor,
-      @Nullable LoadingCache<Pair<String, String>, SegmentErrorInfo> errorCache,
-      TableDataManagerParams tableDataManagerParams) {
-    LOGGER.info("Initializing table data manager for table: {}", tableDataManagerConfig.getTableName());
+  public void init(InstanceDataManagerConfig instanceDataManagerConfig, TableConfig tableConfig,
+      HelixManager helixManager, @Nullable ExecutorService segmentPreloadExecutor,
+      @Nullable LoadingCache<Pair<String, String>, SegmentErrorInfo> errorCache) {
+    LOGGER.info("Initializing table data manager for table: {}", tableConfig.getTableName());
 
-    _tableDataManagerConfig = tableDataManagerConfig;
-    _instanceId = instanceId;
-    _propertyStore = propertyStore;
-    _serverMetrics = serverMetrics;
+    _instanceDataManagerConfig = instanceDataManagerConfig;
+    _instanceId = instanceDataManagerConfig.getInstanceId();
+    _tableConfig = tableConfig;
     _helixManager = helixManager;
+    _propertyStore = helixManager.getHelixPropertyStore();
+    _serverMetrics = ServerMetrics.get();
     _segmentPreloadExecutor = segmentPreloadExecutor;
+    _authProvider = AuthProviderUtils.extractAuthProvider(_instanceDataManagerConfig.getAuthConfig(), null);
 
-    _authProvider =
-        AuthProviderUtils.extractAuthProvider(toPinotConfiguration(_tableDataManagerConfig.getAuthConfig()), null);
-
-    _tableNameWithType = tableDataManagerConfig.getTableName();
-    _tableDataDir = tableDataManagerConfig.getDataDir();
+    _tableNameWithType = tableConfig.getTableName();
+    _tableDataDir = _instanceDataManagerConfig.getInstanceDataDir() + File.separator + _tableNameWithType;
     _indexDir = new File(_tableDataDir);
     if (!_indexDir.exists()) {
       Preconditions.checkState(_indexDir.mkdirs(), "Unable to create index directory at %s. "
@@ -149,18 +144,23 @@ public abstract class BaseTableDataManager implements TableDataManager {
     }
     _errorCache = errorCache;
     _recentlyDeletedSegments =
-        CacheBuilder.newBuilder().maximumSize(tableDataManagerConfig.getTableDeletedSegmentsCacheSize())
-            .expireAfterWrite(tableDataManagerConfig.getTableDeletedSegmentsCacheTtlMinutes(), TimeUnit.MINUTES)
-            .build();
+        CacheBuilder.newBuilder().maximumSize(instanceDataManagerConfig.getDeletedSegmentsCacheSize())
+            .expireAfterWrite(instanceDataManagerConfig.getDeletedSegmentsCacheTtlMinutes(), TimeUnit.MINUTES).build();
+
+    _peerDownloadScheme = tableConfig.getValidationConfig().getPeerSegmentDownloadScheme();
+    if (_peerDownloadScheme == null) {
+      _peerDownloadScheme = instanceDataManagerConfig.getSegmentPeerDownloadScheme();
+    }
+
     _streamSegmentDownloadUntarRateLimitBytesPerSec =
-        tableDataManagerParams.getStreamSegmentDownloadUntarRateLimitBytesPerSec();
-    _isStreamSegmentDownloadUntar = tableDataManagerParams.isStreamSegmentDownloadUntar();
+        instanceDataManagerConfig.getStreamSegmentDownloadUntarRateLimit();
+    _isStreamSegmentDownloadUntar = instanceDataManagerConfig.isStreamSegmentDownloadUntar();
     if (_isStreamSegmentDownloadUntar) {
       LOGGER.info("Using streamed download-untar for segment download! "
               + "The rate limit interval for streamed download-untar is {} bytes/s",
           _streamSegmentDownloadUntarRateLimitBytesPerSec);
     }
-    int maxParallelSegmentDownloads = tableDataManagerParams.getMaxParallelSegmentDownloads();
+    int maxParallelSegmentDownloads = instanceDataManagerConfig.getMaxParallelSegmentDownloads();
     if (maxParallelSegmentDownloads > 0) {
       LOGGER.info(
           "Construct segment download semaphore for Table: {}. Maximum number of parallel segment downloads: {}",
@@ -179,6 +179,16 @@ public abstract class BaseTableDataManager implements TableDataManager {
 
   protected abstract void doInit();
 
+  @Override
+  public String getInstanceId() {
+    return _instanceId;
+  }
+
+  @Override
+  public InstanceDataManagerConfig getInstanceDataManagerConfig() {
+    return _instanceDataManagerConfig;
+  }
+
   @Override
   public synchronized void start() {
     _logger.info("Starting table data manager for table: {}", _tableNameWithType);
@@ -206,11 +216,27 @@ public abstract class BaseTableDataManager implements TableDataManager {
    * Releases and removes all segments tracked by the table data manager.
    */
   protected void releaseAndRemoveAllSegments() {
-    Iterator<SegmentDataManager> iterator = _segmentDataManagerMap.values().iterator();
-    while (iterator.hasNext()) {
-      SegmentDataManager segmentDataManager = iterator.next();
-      iterator.remove();
-      releaseSegment(segmentDataManager);
+    List<SegmentDataManager> segmentDataManagers;
+    synchronized (_segmentDataManagerMap) {
+      segmentDataManagers = new ArrayList<>(_segmentDataManagerMap.values());
+      _segmentDataManagerMap.clear();
+    }
+    if (!segmentDataManagers.isEmpty()) {
+      int numThreads = Math.min(Runtime.getRuntime().availableProcessors(), segmentDataManagers.size());
+      ExecutorService stopExecutorService = Executors.newFixedThreadPool(numThreads);
+      for (SegmentDataManager segmentDataManager : segmentDataManagers) {
+        stopExecutorService.submit(() -> releaseSegment(segmentDataManager));
+      }
+      stopExecutorService.shutdown();
+      try {
+        // Wait at most 10 minutes before exiting this method.
+        if (!stopExecutorService.awaitTermination(10, TimeUnit.MINUTES)) {
+          stopExecutorService.shutdownNow();
+        }
+      } catch (InterruptedException e) {
+        stopExecutorService.shutdownNow();
+        Thread.currentThread().interrupt();
+      }
     }
   }
 
@@ -254,7 +280,7 @@ public abstract class BaseTableDataManager implements TableDataManager {
     Preconditions.checkState(!_shutDown, "Table data manager is already shut down, cannot add segment: %s to table: %s",
         indexDir.getName(), _tableNameWithType);
     indexLoadingConfig.setTableDataDir(_tableDataDir);
-    indexLoadingConfig.setInstanceTierConfigs(_tableDataManagerConfig.getInstanceTierConfigs());
+    indexLoadingConfig.setInstanceTierConfigs(_instanceDataManagerConfig.getTierConfigs());
     addSegment(ImmutableSegmentLoader.load(indexDir, indexLoadingConfig, indexLoadingConfig.getSchema()));
   }
 
@@ -310,6 +336,12 @@ public abstract class BaseTableDataManager implements TableDataManager {
 
   @Override
   public List<SegmentDataManager> acquireSegments(List<String> segmentNames, List<String> missingSegments) {
+    return acquireSegments(segmentNames, null, missingSegments);
+  }
+
+  @Override
+  public List<SegmentDataManager> acquireSegments(List<String> segmentNames,
+      @Nullable List<String> optionalSegmentNames, List<String> missingSegments) {
     List<SegmentDataManager> segmentDataManagers = new ArrayList<>();
     for (String segmentName : segmentNames) {
       SegmentDataManager segmentDataManager = _segmentDataManagerMap.get(segmentName);
@@ -319,6 +351,15 @@ public abstract class BaseTableDataManager implements TableDataManager {
         missingSegments.add(segmentName);
       }
     }
+    if (optionalSegmentNames != null) {
+      for (String segmentName : optionalSegmentNames) {
+        SegmentDataManager segmentDataManager = _segmentDataManagerMap.get(segmentName);
+        // Optional segments are not counted to missing segments that are reported back in query exception.
+        if (segmentDataManager != null && segmentDataManager.increaseReferenceCount()) {
+          segmentDataManagers.add(segmentDataManager);
+        }
+      }
+    }
     return segmentDataManagers;
   }
 
@@ -366,11 +407,6 @@ public abstract class BaseTableDataManager implements TableDataManager {
     return _indexDir;
   }
 
-  @Override
-  public TableDataManagerConfig getTableDataManagerConfig() {
-    return _tableDataManagerConfig;
-  }
-
   @Override
   public void addSegmentError(String segmentName, SegmentErrorInfo segmentErrorInfo) {
     _errorCache.put(Pair.of(_tableNameWithType, segmentName), segmentErrorInfo);
@@ -397,7 +433,7 @@ public abstract class BaseTableDataManager implements TableDataManager {
     String segmentTier = getSegmentCurrentTier(segmentName);
     indexLoadingConfig.setSegmentTier(segmentTier);
     indexLoadingConfig.setTableDataDir(_tableDataDir);
-    indexLoadingConfig.setInstanceTierConfigs(_tableDataManagerConfig.getInstanceTierConfigs());
+    indexLoadingConfig.setInstanceTierConfigs(_instanceDataManagerConfig.getTierConfigs());
     File indexDir = getSegmentDataDir(segmentName, segmentTier, indexLoadingConfig.getTableConfig());
     try {
       // Download segment from deep store if CRC changes or forced to download;
@@ -491,7 +527,7 @@ public abstract class BaseTableDataManager implements TableDataManager {
     String segmentTier = zkMetadata.getTier();
     indexLoadingConfig.setSegmentTier(segmentTier);
     indexLoadingConfig.setTableDataDir(_tableDataDir);
-    indexLoadingConfig.setInstanceTierConfigs(_tableDataManagerConfig.getInstanceTierConfigs());
+    indexLoadingConfig.setInstanceTierConfigs(_instanceDataManagerConfig.getTierConfigs());
     if (localMetadata == null && tryLoadExistingSegment(segmentName, indexLoadingConfig, zkMetadata)) {
       return;
     }
@@ -522,7 +558,10 @@ public abstract class BaseTableDataManager implements TableDataManager {
    */
   @Nullable
   protected SegmentDataManager registerSegment(String segmentName, SegmentDataManager segmentDataManager) {
-    SegmentDataManager oldSegmentDataManager = _segmentDataManagerMap.put(segmentName, segmentDataManager);
+    SegmentDataManager oldSegmentDataManager;
+    synchronized (_segmentDataManagerMap) {
+      oldSegmentDataManager = _segmentDataManagerMap.put(segmentName, segmentDataManager);
+    }
     _recentlyDeletedSegments.invalidate(segmentName);
     return oldSegmentDataManager;
   }
@@ -537,7 +576,9 @@ public abstract class BaseTableDataManager implements TableDataManager {
   @Nullable
   protected SegmentDataManager unregisterSegment(String segmentName) {
     _recentlyDeletedSegments.put(segmentName, segmentName);
-    return _segmentDataManagerMap.remove(segmentName);
+    synchronized (_segmentDataManagerMap) {
+      return _segmentDataManagerMap.remove(segmentName);
+    }
   }
 
   protected boolean allowDownload(String segmentName, SegmentZKMetadata zkMetadata) {
@@ -609,7 +650,7 @@ public abstract class BaseTableDataManager implements TableDataManager {
       LOGGER.error("Attempts exceeded when downloading segment: {} for table: {} from: {} to: {}", segmentName,
           _tableNameWithType, uri, tarFile);
       _serverMetrics.addMeteredTableValue(_tableNameWithType, ServerMeter.SEGMENT_DOWNLOAD_FROM_REMOTE_FAILURES, 1L);
-      if (_tableDataManagerConfig.getTablePeerDownloadScheme() == null) {
+      if (_peerDownloadScheme == null) {
         throw e;
       }
       downloadFromPeersWithoutStreaming(segmentName, zkMetadata, tarFile);
@@ -628,11 +669,9 @@ public abstract class BaseTableDataManager implements TableDataManager {
   // not thread safe. Caller should invoke it with safe concurrency control.
   protected void downloadFromPeersWithoutStreaming(String segmentName, SegmentZKMetadata zkMetadata, File destTarFile)
       throws Exception {
-    Preconditions.checkArgument(_tableDataManagerConfig.getTablePeerDownloadScheme() != null,
-        "Download peers require non null peer download scheme");
+    Preconditions.checkState(_peerDownloadScheme != null, "Download peers require non null peer download scheme");
     List<URI> peerSegmentURIs =
-        PeerServerSegmentFinder.getPeerServerURIs(segmentName, _tableDataManagerConfig.getTablePeerDownloadScheme(),
-            _helixManager, _tableNameWithType);
+        PeerServerSegmentFinder.getPeerServerURIs(segmentName, _peerDownloadScheme, _helixManager, _tableNameWithType);
     if (peerSegmentURIs.isEmpty()) {
       String msg = String.format("segment %s doesn't have any peers", segmentName);
       LOGGER.warn(msg);
@@ -724,7 +763,7 @@ public abstract class BaseTableDataManager implements TableDataManager {
       return getSegmentDataDir(segmentName);
     }
     String tierDataDir =
-        TierConfigUtils.getDataDirForTier(tableConfig, segmentTier, _tableDataManagerConfig.getInstanceTierConfigs());
+        TierConfigUtils.getDataDirForTier(tableConfig, segmentTier, _instanceDataManagerConfig.getTierConfigs());
     if (StringUtils.isEmpty(tierDataDir)) {
       return getSegmentDataDir(segmentName);
     }
@@ -912,11 +951,4 @@ public abstract class BaseTableDataManager implements TableDataManager {
       }
     }
   }
-
-  private static PinotConfiguration toPinotConfiguration(Configuration configuration) {
-    if (configuration == null) {
-      return new PinotConfiguration();
-    }
-    return new PinotConfiguration((Map<String, Object>) (Map) ConfigurationConverter.getMap(configuration));
-  }
 }
