@@ -92,6 +92,11 @@ public class AcquireReleaseColumnsSegmentOperator extends BaseOperator<BaseResul
     materializeChildOperator();
   }
 
+  @Override
+  public void postExplainPlan(ExplainPlanRows explainPlanRows) {
+    release();
+  }
+
   @Override
   public String toExplainString() {
     return EXPLAIN_NAME;
