@@ -18,18 +18,14 @@
  */
 package org.apache.pinot.core.operator.combine;
 
-import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
-import java.util.concurrent.ConcurrentLinkedQueue;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
-import org.apache.pinot.common.exception.QueryException;
-import org.apache.pinot.common.response.ProcessingException;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.core.common.Operator;
 import org.apache.pinot.core.data.table.ConcurrentIndexedTable;
@@ -47,7 +43,7 @@ import org.apache.pinot.core.query.aggregation.groupby.AggregationGroupByResult;
 import org.apache.pinot.core.query.aggregation.groupby.GroupKeyGenerator;
 import org.apache.pinot.core.query.request.context.QueryContext;
 import org.apache.pinot.core.util.GroupByUtils;
-import org.apache.pinot.spi.utils.LoopUtils;
+import org.apache.pinot.spi.trace.Tracing;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -58,9 +54,8 @@ import org.slf4j.LoggerFactory;
  *       all threads
  */
 @SuppressWarnings("rawtypes")
-public class GroupByCombineOperator extends BaseCombineOperator<GroupByResultsBlock> {
+public class GroupByCombineOperator extends BaseSingleBlockCombineOperator<GroupByResultsBlock> {
   public static final int MAX_TRIM_THRESHOLD = 1_000_000_000;
-  public static final int MAX_GROUP_BY_KEYS_MERGED_PER_INTERRUPTION_CHECK = 10_000;
 
   private static final Logger LOGGER = LoggerFactory.getLogger(GroupByCombineOperator.class);
   private static final String EXPLAIN_NAME = "COMBINE_GROUP_BY";
@@ -70,7 +65,6 @@ public class GroupByCombineOperator extends BaseCombineOperator<GroupByResultsBl
   private final int _numAggregationFunctions;
   private final int _numGroupByExpressions;
   private final int _numColumns;
-  private final ConcurrentLinkedQueue<ProcessingException> _mergedProcessingExceptions = new ConcurrentLinkedQueue<>();
   // We use a CountDownLatch to track if all Futures are finished by the query timeout, and cancel the unfinished
   // _futures (try to interrupt the execution if it already started).
   private final CountDownLatch _operatorLatch;
@@ -79,7 +73,7 @@ public class GroupByCombineOperator extends BaseCombineOperator<GroupByResultsBl
   private volatile boolean _numGroupsLimitReached;
 
   public GroupByCombineOperator(List<Operator> operators, QueryContext queryContext, ExecutorService executorService) {
-    super(operators, overrideMaxExecutionThreads(queryContext, operators.size()), executorService);
+    super(null, operators, overrideMaxExecutionThreads(queryContext, operators.size()), executorService);
 
     int minTrimSize = queryContext.getMinServerGroupTrimSize();
     if (minTrimSize > 0) {
@@ -130,7 +124,7 @@ public class GroupByCombineOperator extends BaseCombineOperator<GroupByResultsBl
   @Override
   protected void processSegments() {
     int operatorId;
-    while ((operatorId = _nextOperatorId.getAndIncrement()) < _numOperators) {
+    while (_processingException.get() == null && (operatorId = _nextOperatorId.getAndIncrement()) < _numOperators) {
       Operator operator = _operators.get(operatorId);
       try {
         if (operator instanceof AcquireReleaseColumnsSegmentOperator) {
@@ -156,12 +150,6 @@ public class GroupByCombineOperator extends BaseCombineOperator<GroupByResultsBl
           }
         }
 
-        // Merge processing exceptions.
-        List<ProcessingException> processingExceptionsToMerge = resultsBlock.getProcessingExceptions();
-        if (processingExceptionsToMerge != null) {
-          _mergedProcessingExceptions.addAll(processingExceptionsToMerge);
-        }
-
         // Set groups limit reached flag.
         if (resultsBlock.isNumGroupsLimitReached()) {
           _numGroupsLimitReached = true;
@@ -188,16 +176,16 @@ public class GroupByCombineOperator extends BaseCombineOperator<GroupByResultsBl
                 values[_numGroupByExpressions + i] = aggregationGroupByResult.getResultForGroupId(i, groupId);
               }
               _indexedTable.upsert(new Key(keys), new Record(values));
+              Tracing.ThreadAccountantOps.sampleAndCheckInterruptionPeriodically(mergedKeys);
               mergedKeys++;
-              LoopUtils.checkMergePhaseInterruption(mergedKeys);
             }
           }
         } else {
           for (IntermediateRecord intermediateResult : intermediateRecords) {
             //TODO: change upsert api so that it accepts intermediateRecord directly
             _indexedTable.upsert(intermediateResult._key, intermediateResult._record);
+            Tracing.ThreadAccountantOps.sampleAndCheckInterruptionPeriodically(mergedKeys);
             mergedKeys++;
-            LoopUtils.checkMergePhaseInterruption(mergedKeys);
           }
         }
       } finally {
@@ -209,12 +197,12 @@ public class GroupByCombineOperator extends BaseCombineOperator<GroupByResultsBl
   }
 
   @Override
-  protected void onException(Throwable t) {
-    _mergedProcessingExceptions.add(QueryException.getException(QueryException.QUERY_EXECUTION_ERROR, t));
+  public void onProcessSegmentsException(Throwable t) {
+    _processingException.compareAndSet(null, t);
   }
 
   @Override
-  protected void onFinish() {
+  public void onProcessSegmentsFinish() {
     _operatorLatch.countDown();
   }
 
@@ -232,7 +220,7 @@ public class GroupByCombineOperator extends BaseCombineOperator<GroupByResultsBl
    * </ul>
    */
   @Override
-  protected BaseResultsBlock mergeResults()
+  public BaseResultsBlock mergeResults()
       throws Exception {
     long timeoutMs = _queryContext.getEndTimeMs() - System.currentTimeMillis();
     boolean opCompleted = _operatorLatch.await(timeoutMs, TimeUnit.MILLISECONDS);
@@ -245,26 +233,21 @@ public class GroupByCombineOperator extends BaseCombineOperator<GroupByResultsBl
       return new ExceptionResultsBlock(new TimeoutException(errorMessage));
     }
 
+    Throwable processingException = _processingException.get();
+    if (processingException != null) {
+      return new ExceptionResultsBlock(processingException);
+    }
+
     IndexedTable indexedTable = _indexedTable;
     if (!_queryContext.isServerReturnFinalResult()) {
       indexedTable.finish(false);
     } else {
       indexedTable.finish(true, true);
     }
-    GroupByResultsBlock mergedBlock = new GroupByResultsBlock(indexedTable);
+    GroupByResultsBlock mergedBlock = new GroupByResultsBlock(indexedTable, _queryContext);
     mergedBlock.setNumGroupsLimitReached(_numGroupsLimitReached);
     mergedBlock.setNumResizes(indexedTable.getNumResizes());
     mergedBlock.setResizeTimeMs(indexedTable.getResizeTimeMs());
-
-    // Set the processing exceptions.
-    if (!_mergedProcessingExceptions.isEmpty()) {
-      mergedBlock.setProcessingExceptions(new ArrayList<>(_mergedProcessingExceptions));
-    }
-
     return mergedBlock;
   }
-
-  @Override
-  protected void mergeResultsBlocks(GroupByResultsBlock mergedBlock, GroupByResultsBlock blockToMerge) {
-  }
 }
