@@ -32,6 +32,7 @@ import org.apache.pinot.core.operator.ExecutionStatistics;
 import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.operator.blocks.results.GroupByResultsBlock;
 import org.apache.pinot.core.query.aggregation.function.AggregationFunction;
+import org.apache.pinot.core.query.aggregation.function.AggregationFunctionUtils.AggregationInfo;
 import org.apache.pinot.core.query.aggregation.groupby.DefaultGroupByExecutor;
 import org.apache.pinot.core.query.aggregation.groupby.GroupByExecutor;
 import org.apache.pinot.core.query.request.context.QueryContext;
@@ -51,23 +52,23 @@ public class GroupByOperator extends BaseOperator<GroupByResultsBlock> {
   private final AggregationFunction[] _aggregationFunctions;
   private final ExpressionContext[] _groupByExpressions;
   private final BaseProjectOperator<?> _projectOperator;
-  private final long _numTotalDocs;
   private final boolean _useStarTree;
+  private final long _numTotalDocs;
   private final DataSchema _dataSchema;
 
   private int _numDocsScanned = 0;
 
-  public GroupByOperator(QueryContext queryContext, ExpressionContext[] groupByExpressions,
-      BaseProjectOperator<?> projectOperator, long numTotalDocs, boolean useStarTree) {
+  public GroupByOperator(QueryContext queryContext, AggregationInfo aggregationInfo, long numTotalDocs) {
+    assert queryContext.getAggregationFunctions() != null && queryContext.getGroupByExpressions() != null;
     _queryContext = queryContext;
     _aggregationFunctions = queryContext.getAggregationFunctions();
-    _groupByExpressions = groupByExpressions;
-    _projectOperator = projectOperator;
+    _groupByExpressions = queryContext.getGroupByExpressions().toArray(new ExpressionContext[0]);
+    _projectOperator = aggregationInfo.getProjectOperator();
+    _useStarTree = aggregationInfo.isUseStarTree();
     _numTotalDocs = numTotalDocs;
-    _useStarTree = useStarTree;
 
-    // NOTE: The indexedTable expects that the the data schema will have group by columns before aggregation columns
-    int numGroupByExpressions = groupByExpressions.length;
+    // NOTE: The indexedTable expects that the data schema will have group by columns before aggregation columns
+    int numGroupByExpressions = _groupByExpressions.length;
     int numAggregationFunctions = _aggregationFunctions.length;
     int numColumns = numGroupByExpressions + numAggregationFunctions;
     String[] columnNames = new String[numColumns];
@@ -75,7 +76,7 @@ public class GroupByOperator extends BaseOperator<GroupByResultsBlock> {
 
     // Extract column names and data types for group-by columns
     for (int i = 0; i < numGroupByExpressions; i++) {
-      ExpressionContext groupByExpression = groupByExpressions[i];
+      ExpressionContext groupByExpression = _groupByExpressions[i];
       columnNames[i] = groupByExpression.toString();
       columnDataTypes[i] = DataSchema.ColumnDataType.fromDataTypeSV(
           _projectOperator.getResultColumnContext(groupByExpression).getDataType());
