@@ -46,11 +46,12 @@ public class InIdSetTransformFunction extends BaseTransformFunction {
 
   @Override
   public String getName() {
-    return TransformFunctionType.INIDSET.getName();
+    return TransformFunctionType.IN_ID_SET.getName();
   }
 
   @Override
   public void init(List<TransformFunction> arguments, Map<String, ColumnContext> columnContextMap) {
+    super.init(arguments, columnContextMap);
     Preconditions.checkArgument(arguments.size() == 2,
         "2 arguments are required for IN_ID_SET transform function: expression, base64 encoded IdSet");
     Preconditions.checkArgument(arguments.get(0).getResultMetadata().isSingleValue(),
