@@ -23,6 +23,16 @@ import java.util.List;
 import org.apache.datasketches.tuple.aninteger.IntegerSummary;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.FunctionContext;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggDistinctDoubleFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggDistinctFloatFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggDistinctIntFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggDistinctLongFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggDistinctStringFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggDoubleFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggFloatFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggIntFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggLongFunction;
+import org.apache.pinot.core.query.aggregation.function.array.ArrayAggStringFunction;
 import org.apache.pinot.core.query.aggregation.function.funnel.FunnelCountAggregationFunctionFactory;
 import org.apache.pinot.segment.spi.AggregationFunctionType;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
@@ -46,6 +56,7 @@ public class AggregationFunctionFactory {
       String upperCaseFunctionName =
           AggregationFunctionType.getNormalizedAggregationFunctionName(function.getFunctionName());
       List<ExpressionContext> arguments = function.getArguments();
+      int numArguments = arguments.size();
       ExpressionContext firstArgument = arguments.get(0);
       if (upperCaseFunctionName.startsWith("PERCENTILE")) {
         String remainingFunctionName = upperCaseFunctionName.substring(10);
@@ -64,7 +75,6 @@ public class AggregationFunctionFactory {
         if (remainingFunctionName.equals("RAWKLLMV")) {
           return new PercentileRawKLLMVAggregationFunction(arguments);
         }
-        int numArguments = arguments.size();
         if (numArguments == 1) {
           // Single argument percentile (e.g. Percentile99(foo), PercentileTDigest95(bar), etc.)
           // NOTE: This convention is deprecated. DO NOT add new functions here
@@ -111,8 +121,8 @@ public class AggregationFunctionFactory {
         } else if (numArguments == 2) {
           // Double arguments percentile (e.g. percentile(foo, 99), percentileTDigest(bar, 95), etc.) where the
           // second argument is a decimal number from 0.0 to 100.0.
-          // Have to use literal string because we need to cast int to double here.
-          double percentile = parsePercentileToDouble(arguments.get(1).getLiteral().getStringValue());
+          double percentile = arguments.get(1).getLiteral().getDoubleValue();
+          Preconditions.checkArgument(percentile >= 0 && percentile <= 100, "Invalid percentile: %s", percentile);
           if (remainingFunctionName.isEmpty()) {
             // Percentile
             return new PercentileAggregationFunction(firstArgument, percentile);
@@ -159,9 +169,10 @@ public class AggregationFunctionFactory {
           // the compression_factor for the TDigest. This can only be used for TDigest type percentile functions to
           // pass in a custom compression_factor. If the two argument version is used the default compression_factor
           // of 100.0 is used.
-          // Have to use literal string because we need to cast int to double here.
-          double percentile = parsePercentileToDouble(arguments.get(1).getLiteral().getStringValue());
-          int compressionFactor = parseCompressionFactorToInt(arguments.get(2).getLiteral().getStringValue());
+          double percentile = arguments.get(1).getLiteral().getDoubleValue();
+          Preconditions.checkArgument(percentile >= 0 && percentile <= 100, "Invalid percentile: %s", percentile);
+          int compressionFactor = arguments.get(2).getLiteral().getIntValue();
+          Preconditions.checkArgument(compressionFactor >= 0, "Invalid compressionFactor: %d", compressionFactor);
           if (remainingFunctionName.equals("TDIGEST")) {
             // PercentileTDigest
             return new PercentileTDigestAggregationFunction(firstArgument, percentile, compressionFactor);
@@ -181,88 +192,137 @@ public class AggregationFunctionFactory {
         }
         throw new IllegalArgumentException("Invalid percentile function: " + function);
       } else {
-        switch (AggregationFunctionType.valueOf(upperCaseFunctionName)) {
+        AggregationFunctionType functionType = AggregationFunctionType.valueOf(upperCaseFunctionName);
+        switch (functionType) {
           case COUNT:
-            return new CountAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new CountAggregationFunction(arguments, nullHandlingEnabled);
           case MIN:
-            return new MinAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new MinAggregationFunction(arguments, nullHandlingEnabled);
           case MAX:
-            return new MaxAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new MaxAggregationFunction(arguments, nullHandlingEnabled);
           case SUM:
           case SUM0:
-            return new SumAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new SumAggregationFunction(arguments, nullHandlingEnabled);
           case SUMPRECISION:
             return new SumPrecisionAggregationFunction(arguments, nullHandlingEnabled);
           case AVG:
-            return new AvgAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new AvgAggregationFunction(arguments, nullHandlingEnabled);
           case MODE:
             return new ModeAggregationFunction(arguments);
-          case FIRSTWITHTIME:
-            if (arguments.size() == 3) {
-              ExpressionContext timeCol = arguments.get(1);
-              ExpressionContext dataType = arguments.get(2);
-              if (dataType.getType() != ExpressionContext.Type.LITERAL) {
-                throw new IllegalArgumentException("Third argument of firstWithTime Function should be literal."
-                    + " The function can be used as firstWithTime(dataColumn, timeColumn, 'dataType')");
-              }
-              DataType fieldDataType = DataType.valueOf(dataType.getLiteral().getStringValue().toUpperCase());
-              switch (fieldDataType) {
+          case FIRSTWITHTIME: {
+            Preconditions.checkArgument(numArguments == 3,
+                "FIRST_WITH_TIME expects 3 arguments, got: %s. The function can be used as "
+                    + "firstWithTime(dataColumn, timeColumn, 'dataType')", numArguments);
+            ExpressionContext timeCol = arguments.get(1);
+            ExpressionContext dataTypeExp = arguments.get(2);
+            Preconditions.checkArgument(dataTypeExp.getType() == ExpressionContext.Type.LITERAL,
+                "FIRST_WITH_TIME expects the 3rd argument to be literal, got: %s. The function can be used as "
+                    + "firstWithTime(dataColumn, timeColumn, 'dataType')", dataTypeExp.getType());
+            DataType dataType = DataType.valueOf(dataTypeExp.getLiteral().getStringValue().toUpperCase());
+            switch (dataType) {
+              case BOOLEAN:
+                return new FirstIntValueWithTimeAggregationFunction(firstArgument, timeCol, true);
+              case INT:
+                return new FirstIntValueWithTimeAggregationFunction(firstArgument, timeCol, false);
+              case LONG:
+                return new FirstLongValueWithTimeAggregationFunction(firstArgument, timeCol);
+              case FLOAT:
+                return new FirstFloatValueWithTimeAggregationFunction(firstArgument, timeCol);
+              case DOUBLE:
+                return new FirstDoubleValueWithTimeAggregationFunction(firstArgument, timeCol);
+              case STRING:
+                return new FirstStringValueWithTimeAggregationFunction(firstArgument, timeCol);
+              default:
+                throw new IllegalArgumentException("Unsupported data type for FIRST_WITH_TIME: " + dataType);
+            }
+          }
+          case ARRAYAGG: {
+            Preconditions.checkArgument(numArguments >= 2,
+                "ARRAY_AGG expects 2 or 3 arguments, got: %s. The function can be used as "
+                    + "arrayAgg(dataColumn, 'dataType', ['isDistinct'])", numArguments);
+            ExpressionContext dataTypeExp = arguments.get(1);
+            Preconditions.checkArgument(dataTypeExp.getType() == ExpressionContext.Type.LITERAL,
+                "ARRAY_AGG expects the 2nd argument to be literal, got: %s. The function can be used as "
+                    + "arrayAgg(dataColumn, 'dataType', ['isDistinct'])", dataTypeExp.getType());
+            DataType dataType = DataType.valueOf(dataTypeExp.getLiteral().getStringValue().toUpperCase());
+            boolean isDistinct = false;
+            if (numArguments == 3) {
+              ExpressionContext isDistinctExp = arguments.get(2);
+              Preconditions.checkArgument(isDistinctExp.getType() == ExpressionContext.Type.LITERAL,
+                  "ARRAY_AGG expects the 3rd argument to be literal, got: %s. The function can be used as "
+                      + "arrayAgg(dataColumn, 'dataType', ['isDistinct'])", isDistinctExp.getType());
+              isDistinct = isDistinctExp.getLiteral().getBooleanValue();
+            }
+            if (isDistinct) {
+              switch (dataType) {
                 case BOOLEAN:
                 case INT:
-                  return new FirstIntValueWithTimeAggregationFunction(firstArgument, timeCol,
-                      fieldDataType == DataType.BOOLEAN);
+                  return new ArrayAggDistinctIntFunction(firstArgument, dataType, nullHandlingEnabled);
                 case LONG:
-                  return new FirstLongValueWithTimeAggregationFunction(firstArgument, timeCol);
+                case TIMESTAMP:
+                  return new ArrayAggDistinctLongFunction(firstArgument, dataType, nullHandlingEnabled);
                 case FLOAT:
-                  return new FirstFloatValueWithTimeAggregationFunction(firstArgument, timeCol);
+                  return new ArrayAggDistinctFloatFunction(firstArgument, nullHandlingEnabled);
                 case DOUBLE:
-                  return new FirstDoubleValueWithTimeAggregationFunction(firstArgument, timeCol);
+                  return new ArrayAggDistinctDoubleFunction(firstArgument, nullHandlingEnabled);
                 case STRING:
-                  return new FirstStringValueWithTimeAggregationFunction(firstArgument, timeCol);
+                  return new ArrayAggDistinctStringFunction(firstArgument, nullHandlingEnabled);
                 default:
-                  throw new IllegalArgumentException("Unsupported Value Type for firstWithTime Function:" + dataType);
+                  throw new IllegalArgumentException("Unsupported data type for ARRAY_AGG: " + dataType);
               }
-            } else {
-              throw new IllegalArgumentException("Three arguments are required for firstWithTime Function."
-                  + " The function can be used as firstWithTime(dataColumn, timeColumn, 'dataType')");
             }
-          case LASTWITHTIME:
-            if (arguments.size() == 3) {
-              ExpressionContext timeCol = arguments.get(1);
-              ExpressionContext dataType = arguments.get(2);
-              if (dataType.getType() != ExpressionContext.Type.LITERAL) {
-                throw new IllegalArgumentException("Third argument of lastWithTime Function should be literal."
-                    + " The function can be used as lastWithTime(dataColumn, timeColumn, 'dataType')");
-              }
-              DataType fieldDataType = DataType.valueOf(dataType.getLiteral().getStringValue().toUpperCase());
-              switch (fieldDataType) {
-                case BOOLEAN:
-                case INT:
-                  return new LastIntValueWithTimeAggregationFunction(firstArgument, timeCol,
-                      fieldDataType == DataType.BOOLEAN);
-                case LONG:
-                  return new LastLongValueWithTimeAggregationFunction(firstArgument, timeCol);
-                case FLOAT:
-                  return new LastFloatValueWithTimeAggregationFunction(firstArgument, timeCol);
-                case DOUBLE:
-                  return new LastDoubleValueWithTimeAggregationFunction(firstArgument, timeCol);
-                case STRING:
-                  return new LastStringValueWithTimeAggregationFunction(firstArgument, timeCol);
-                default:
-                  throw new IllegalArgumentException("Unsupported Value Type for lastWithTime Function:" + dataType);
-              }
-            } else {
-              throw new IllegalArgumentException("Three arguments are required for lastWithTime Function."
-                  + " The function can be used as lastWithTime(dataColumn, timeColumn, 'dataType')");
+            switch (dataType) {
+              case BOOLEAN:
+              case INT:
+                return new ArrayAggIntFunction(firstArgument, dataType, nullHandlingEnabled);
+              case LONG:
+              case TIMESTAMP:
+                return new ArrayAggLongFunction(firstArgument, dataType, nullHandlingEnabled);
+              case FLOAT:
+                return new ArrayAggFloatFunction(firstArgument, nullHandlingEnabled);
+              case DOUBLE:
+                return new ArrayAggDoubleFunction(firstArgument, nullHandlingEnabled);
+              case STRING:
+                return new ArrayAggStringFunction(firstArgument, nullHandlingEnabled);
+              default:
+                throw new IllegalArgumentException("Unsupported data type for ARRAY_AGG: " + dataType);
             }
+          }
+          case LASTWITHTIME: {
+            Preconditions.checkArgument(numArguments == 3,
+                "LAST_WITH_TIME expects 3 arguments, got: %s. The function can be used as "
+                    + "lastWithTime(dataColumn, timeColumn, 'dataType')", numArguments);
+            ExpressionContext timeCol = arguments.get(1);
+            ExpressionContext dataTypeExp = arguments.get(2);
+            Preconditions.checkArgument(dataTypeExp.getType() == ExpressionContext.Type.LITERAL,
+                "LAST_WITH_TIME expects the 3rd argument to be literal, got: %s. The function can be used as "
+                    + "lastWithTime(dataColumn, timeColumn, 'dataType')", dataTypeExp.getType());
+            DataType dataType = DataType.valueOf(dataTypeExp.getLiteral().getStringValue().toUpperCase());
+            switch (dataType) {
+              case BOOLEAN:
+                return new LastIntValueWithTimeAggregationFunction(firstArgument, timeCol, true);
+              case INT:
+                return new LastIntValueWithTimeAggregationFunction(firstArgument, timeCol, false);
+              case LONG:
+                return new LastLongValueWithTimeAggregationFunction(firstArgument, timeCol);
+              case FLOAT:
+                return new LastFloatValueWithTimeAggregationFunction(firstArgument, timeCol);
+              case DOUBLE:
+                return new LastDoubleValueWithTimeAggregationFunction(firstArgument, timeCol);
+              case STRING:
+                return new LastStringValueWithTimeAggregationFunction(firstArgument, timeCol);
+              default:
+                throw new IllegalArgumentException("Unsupported data type for LAST_WITH_TIME: " + dataType);
+            }
+          }
           case MINMAXRANGE:
-            return new MinMaxRangeAggregationFunction(firstArgument);
+            return new MinMaxRangeAggregationFunction(arguments);
           case DISTINCTCOUNT:
-            return new DistinctCountAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new DistinctCountAggregationFunction(arguments, nullHandlingEnabled);
           case DISTINCTCOUNTBITMAP:
-            return new DistinctCountBitmapAggregationFunction(firstArgument);
+            return new DistinctCountBitmapAggregationFunction(arguments);
           case SEGMENTPARTITIONEDDISTINCTCOUNT:
-            return new SegmentPartitionedDistinctCountAggregationFunction(firstArgument);
+            return new SegmentPartitionedDistinctCountAggregationFunction(arguments);
           case DISTINCTCOUNTHLL:
             return new DistinctCountHLLAggregationFunction(arguments);
           case DISTINCTCOUNTRAWHLL:
@@ -270,43 +330,51 @@ public class AggregationFunctionFactory {
           case DISTINCTCOUNTSMARTHLL:
             return new DistinctCountSmartHLLAggregationFunction(arguments);
           case FASTHLL:
-            return new FastHLLAggregationFunction(firstArgument);
+            return new FastHLLAggregationFunction(arguments);
           case DISTINCTCOUNTTHETASKETCH:
             return new DistinctCountThetaSketchAggregationFunction(arguments);
           case DISTINCTCOUNTRAWTHETASKETCH:
             return new DistinctCountRawThetaSketchAggregationFunction(arguments);
           case DISTINCTSUM:
-            return new DistinctSumAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new DistinctSumAggregationFunction(arguments, nullHandlingEnabled);
           case DISTINCTAVG:
-            return new DistinctAvgAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new DistinctAvgAggregationFunction(arguments, nullHandlingEnabled);
           case IDSET:
             return new IdSetAggregationFunction(arguments);
           case COUNTMV:
-            return new CountMVAggregationFunction(firstArgument);
+            return new CountMVAggregationFunction(arguments);
           case MINMV:
-            return new MinMVAggregationFunction(firstArgument);
+            return new MinMVAggregationFunction(arguments);
           case MAXMV:
-            return new MaxMVAggregationFunction(firstArgument);
+            return new MaxMVAggregationFunction(arguments);
           case SUMMV:
-            return new SumMVAggregationFunction(firstArgument);
+            return new SumMVAggregationFunction(arguments);
           case AVGMV:
-            return new AvgMVAggregationFunction(firstArgument);
+            return new AvgMVAggregationFunction(arguments);
           case MINMAXRANGEMV:
-            return new MinMaxRangeMVAggregationFunction(firstArgument);
+            return new MinMaxRangeMVAggregationFunction(arguments);
           case DISTINCTCOUNTMV:
-            return new DistinctCountMVAggregationFunction(firstArgument);
+            return new DistinctCountMVAggregationFunction(arguments);
           case DISTINCTCOUNTBITMAPMV:
-            return new DistinctCountBitmapMVAggregationFunction(firstArgument);
+            return new DistinctCountBitmapMVAggregationFunction(arguments);
           case DISTINCTCOUNTHLLMV:
             return new DistinctCountHLLMVAggregationFunction(arguments);
           case DISTINCTCOUNTRAWHLLMV:
             return new DistinctCountRawHLLMVAggregationFunction(arguments);
+          case DISTINCTCOUNTHLLPLUS:
+            return new DistinctCountHLLPlusAggregationFunction(arguments);
+          case DISTINCTCOUNTRAWHLLPLUS:
+            return new DistinctCountRawHLLPlusAggregationFunction(arguments);
+          case DISTINCTCOUNTHLLPLUSMV:
+            return new DistinctCountHLLPlusMVAggregationFunction(arguments);
+          case DISTINCTCOUNTRAWHLLPLUSMV:
+            return new DistinctCountRawHLLPlusMVAggregationFunction(arguments);
           case DISTINCTSUMMV:
-            return new DistinctSumMVAggregationFunction(firstArgument);
+            return new DistinctSumMVAggregationFunction(arguments);
           case DISTINCTAVGMV:
-            return new DistinctAvgMVAggregationFunction(firstArgument);
+            return new DistinctAvgMVAggregationFunction(arguments);
           case STUNION:
-            return new StUnionAggregationFunction(firstArgument);
+            return new StUnionAggregationFunction(arguments);
           case HISTOGRAM:
             return new HistogramAggregationFunction(arguments);
           case COVARPOP:
@@ -314,23 +382,23 @@ public class AggregationFunctionFactory {
           case COVARSAMP:
             return new CovarianceAggregationFunction(arguments, true);
           case BOOLAND:
-            return new BooleanAndAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new BooleanAndAggregationFunction(arguments, nullHandlingEnabled);
           case BOOLOR:
-            return new BooleanOrAggregationFunction(firstArgument, nullHandlingEnabled);
+            return new BooleanOrAggregationFunction(arguments, nullHandlingEnabled);
           case VARPOP:
-            return new VarianceAggregationFunction(firstArgument, false, false, nullHandlingEnabled);
+            return new VarianceAggregationFunction(arguments, false, false, nullHandlingEnabled);
           case VARSAMP:
-            return new VarianceAggregationFunction(firstArgument, true, false, nullHandlingEnabled);
+            return new VarianceAggregationFunction(arguments, true, false, nullHandlingEnabled);
           case STDDEVPOP:
-            return new VarianceAggregationFunction(firstArgument, false, true, nullHandlingEnabled);
+            return new VarianceAggregationFunction(arguments, false, true, nullHandlingEnabled);
           case STDDEVSAMP:
-            return new VarianceAggregationFunction(firstArgument, true, true, nullHandlingEnabled);
+            return new VarianceAggregationFunction(arguments, true, true, nullHandlingEnabled);
           case SKEWNESS:
-            return new FourthMomentAggregationFunction(firstArgument, FourthMomentAggregationFunction.Type.SKEWNESS);
+            return new FourthMomentAggregationFunction(arguments, FourthMomentAggregationFunction.Type.SKEWNESS);
           case KURTOSIS:
-            return new FourthMomentAggregationFunction(firstArgument, FourthMomentAggregationFunction.Type.KURTOSIS);
+            return new FourthMomentAggregationFunction(arguments, FourthMomentAggregationFunction.Type.KURTOSIS);
           case FOURTHMOMENT:
-            return new FourthMomentAggregationFunction(firstArgument, FourthMomentAggregationFunction.Type.MOMENT);
+            return new FourthMomentAggregationFunction(arguments, FourthMomentAggregationFunction.Type.MOMENT);
           case DISTINCTCOUNTTUPLESKETCH:
             // mode actually doesn't matter here because we only care about keys, not values
             return new DistinctCountIntegerTupleSketchAggregationFunction(arguments, IntegerSummary.Mode.Sum);
@@ -351,12 +419,23 @@ public class AggregationFunctionFactory {
           case EXPRMAX:
           case EXPRMIN:
             throw new IllegalArgumentException(
-                "Aggregation function: " + function + " is only supported in selection without alias.");
+                "Aggregation function: " + functionType + " is only supported in selection without alias.");
           case FUNNELCOUNT:
             return new FunnelCountAggregationFunctionFactory(arguments).get();
-
+          case FREQUENTSTRINGSSKETCH:
+            return new FrequentStringsSketchAggregationFunction(arguments);
+          case FREQUENTLONGSSKETCH:
+            return new FrequentLongsSketchAggregationFunction(arguments);
+          case DISTINCTCOUNTCPCSKETCH:
+            return new DistinctCountCPCSketchAggregationFunction(arguments);
+          case DISTINCTCOUNTRAWCPCSKETCH:
+            return new DistinctCountRawCPCSketchAggregationFunction(arguments);
+          case DISTINCTCOUNTULL:
+            return new DistinctCountULLAggregationFunction(arguments);
+          case DISTINCTCOUNTRAWULL:
+            return new DistinctCountRawULLAggregationFunction(arguments);
           default:
-            throw new IllegalArgumentException();
+            throw new IllegalArgumentException("Unsupported aggregation function type: " + functionType);
         }
       }
     } catch (Exception e) {
@@ -369,16 +448,4 @@ public class AggregationFunctionFactory {
     Preconditions.checkArgument(percentile >= 0 && percentile <= 100, "Invalid percentile: %s", percentile);
     return percentile;
   }
-
-  private static double parsePercentileToDouble(String percentileString) {
-    double percentile = Double.parseDouble(percentileString);
-    Preconditions.checkArgument(percentile >= 0d && percentile <= 100d, "Invalid percentile: %s", percentile);
-    return percentile;
-  }
-
-  private static int parseCompressionFactorToInt(String compressionFactorString) {
-    int compressionFactor = Integer.parseInt(compressionFactorString);
-    Preconditions.checkArgument(compressionFactor >= 0, "Invalid compressionFactor: %d", compressionFactor);
-    return compressionFactor;
-  }
 }
