@@ -32,6 +32,7 @@ import org.apache.pinot.common.CustomObject;
 import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.FilterContext;
+import org.apache.pinot.common.request.context.predicate.Predicate;
 import org.apache.pinot.common.utils.DataSchema.ColumnDataType;
 import org.apache.pinot.core.common.BlockValSet;
 import org.apache.pinot.core.common.ObjectSerDeUtils;
@@ -39,10 +40,12 @@ import org.apache.pinot.core.operator.BaseProjectOperator;
 import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.operator.filter.BaseFilterOperator;
 import org.apache.pinot.core.operator.filter.CombinedFilterOperator;
+import org.apache.pinot.core.operator.filter.predicate.PredicateEvaluator;
 import org.apache.pinot.core.plan.DocIdSetPlanNode;
 import org.apache.pinot.core.plan.FilterPlanNode;
 import org.apache.pinot.core.plan.ProjectPlanNode;
 import org.apache.pinot.core.query.request.context.QueryContext;
+import org.apache.pinot.core.startree.StarTreeUtils;
 import org.apache.pinot.segment.spi.AggregationFunctionType;
 import org.apache.pinot.segment.spi.IndexSegment;
 import org.apache.pinot.segment.spi.index.startree.AggregationFunctionColumnPair;
@@ -57,22 +60,23 @@ public class AggregationFunctionUtils {
   }
 
   /**
-   * (For Star-Tree) Creates an {@link AggregationFunctionColumnPair} from the {@link AggregationFunction}. Returns
-   * {@code null} if the {@link AggregationFunction} cannot be represented as an {@link AggregationFunctionColumnPair}
-   * (e.g. has multiple arguments, argument is not column etc.).
+   * (For Star-Tree) Creates an {@link AggregationFunctionColumnPair} in stored type from the
+   * {@link AggregationFunction}. Returns {@code null} if the {@link AggregationFunction} cannot be represented as an
+   * {@link AggregationFunctionColumnPair} (e.g. has multiple arguments, argument is not column etc.).
+   * TODO: Allow multiple arguments for aggregation functions, e.g. percentileEst
    */
   @Nullable
-  public static AggregationFunctionColumnPair getAggregationFunctionColumnPair(
-      AggregationFunction aggregationFunction) {
-    AggregationFunctionType aggregationFunctionType = aggregationFunction.getType();
-    if (aggregationFunctionType == AggregationFunctionType.COUNT) {
+  public static AggregationFunctionColumnPair getStoredFunctionColumnPair(AggregationFunction aggregationFunction) {
+    AggregationFunctionType functionType = aggregationFunction.getType();
+    if (functionType == AggregationFunctionType.COUNT) {
       return AggregationFunctionColumnPair.COUNT_STAR;
     }
     List<ExpressionContext> inputExpressions = aggregationFunction.getInputExpressions();
     if (inputExpressions.size() == 1) {
       ExpressionContext inputExpression = inputExpressions.get(0);
       if (inputExpression.getType() == ExpressionContext.Type.IDENTIFIER) {
-        return new AggregationFunctionColumnPair(aggregationFunctionType, inputExpression.getIdentifier());
+        return new AggregationFunctionColumnPair(AggregationFunctionColumnPair.getStoredType(functionType),
+            inputExpression.getIdentifier());
       }
     }
     return null;
@@ -178,21 +182,102 @@ public class AggregationFunctionUtils {
         return dataTable.getString(rowId, colId);
       case BYTES:
         return dataTable.getBytes(rowId, colId).getBytes();
+      case INT_ARRAY:
+        return dataTable.getIntArray(rowId, colId);
+      case LONG_ARRAY:
+        return dataTable.getLongArray(rowId, colId);
+      case FLOAT_ARRAY:
+        return dataTable.getFloatArray(rowId, colId);
       case DOUBLE_ARRAY:
         return dataTable.getDoubleArray(rowId, colId);
+      case BOOLEAN_ARRAY: {
+        int[] intValues = dataTable.getIntArray(rowId, colId);
+        int numValues = intValues.length;
+        boolean[] booleanValues = new boolean[numValues];
+        for (int i = 0; i < numValues; i++) {
+          booleanValues[i] = intValues[i] == 1;
+        }
+        return booleanValues;
+      }
+      case TIMESTAMP_ARRAY: {
+        long[] longValues = dataTable.getLongArray(rowId, colId);
+        int numValues = longValues.length;
+        Timestamp[] timestampValues = new Timestamp[numValues];
+        for (int i = 0; i < numValues; i++) {
+          timestampValues[i] = new Timestamp(longValues[i]);
+        }
+        return timestampValues;
+      }
+      case STRING_ARRAY:
+        return dataTable.getStringArray(rowId, colId);
       default:
         throw new IllegalStateException("Illegal column data type in final result: " + columnDataType);
     }
   }
 
+  public static class AggregationInfo {
+    private final AggregationFunction[] _functions;
+    private final BaseProjectOperator<?> _projectOperator;
+    private final boolean _useStarTree;
+
+    public AggregationInfo(AggregationFunction[] functions, BaseProjectOperator<?> projectOperator,
+        boolean useStarTree) {
+      _functions = functions;
+      _projectOperator = projectOperator;
+      _useStarTree = useStarTree;
+    }
+
+    public AggregationFunction[] getFunctions() {
+      return _functions;
+    }
+
+    public BaseProjectOperator<?> getProjectOperator() {
+      return _projectOperator;
+    }
+
+    public boolean isUseStarTree() {
+      return _useStarTree;
+    }
+  }
+
   /**
-   * Build pairs of filtered aggregation functions and corresponding project operator.
+   * Builds {@link AggregationInfo} for aggregations.
    */
-  public static List<Pair<AggregationFunction[], BaseProjectOperator<?>>> buildFilteredAggregateProjectOperators(
-      IndexSegment indexSegment, QueryContext queryContext) {
+  public static AggregationInfo buildAggregationInfo(IndexSegment indexSegment, QueryContext queryContext,
+      AggregationFunction[] aggregationFunctions, @Nullable FilterContext filter, BaseFilterOperator filterOperator,
+      List<Pair<Predicate, PredicateEvaluator>> predicateEvaluators) {
+    BaseProjectOperator<?> projectOperator = null;
+
+    // TODO: Create a short-circuit ProjectOperator when filter result is empty
+    if (!filterOperator.isResultEmpty()) {
+      projectOperator =
+          StarTreeUtils.createStarTreeBasedProjectOperator(indexSegment, queryContext, aggregationFunctions, filter,
+              predicateEvaluators);
+    }
+
+    if (projectOperator != null) {
+      return new AggregationInfo(aggregationFunctions, projectOperator, true);
+    } else {
+      Set<ExpressionContext> expressionsToTransform =
+          AggregationFunctionUtils.collectExpressionsToTransform(aggregationFunctions,
+              queryContext.getGroupByExpressions());
+      projectOperator =
+          new ProjectPlanNode(indexSegment, queryContext, expressionsToTransform, DocIdSetPlanNode.MAX_DOC_PER_CALL,
+              filterOperator).run();
+      return new AggregationInfo(aggregationFunctions, projectOperator, false);
+    }
+  }
+
+  /**
+   * Builds swim-lanes (list of {@link AggregationInfo}) for filtered aggregations.
+   */
+  public static List<AggregationInfo> buildFilteredAggregationInfos(IndexSegment indexSegment,
+      QueryContext queryContext) {
     assert queryContext.getAggregationFunctions() != null && queryContext.getFilteredAggregationFunctions() != null;
 
-    BaseFilterOperator mainFilterOperator = new FilterPlanNode(indexSegment, queryContext).run();
+    FilterPlanNode mainFilterPlan = new FilterPlanNode(indexSegment, queryContext);
+    BaseFilterOperator mainFilterOperator = mainFilterPlan.run();
+    List<Pair<Predicate, PredicateEvaluator>> mainPredicateEvaluators = mainFilterPlan.getPredicateEvaluators();
 
     // No need to process sub-filters when main filter has empty result
     if (mainFilterOperator.isResultEmpty()) {
@@ -202,68 +287,88 @@ public class AggregationFunctionUtils {
       BaseProjectOperator<?> projectOperator =
           new ProjectPlanNode(indexSegment, queryContext, expressions, DocIdSetPlanNode.MAX_DOC_PER_CALL,
               mainFilterOperator).run();
-      return Collections.singletonList(Pair.of(aggregationFunctions, projectOperator));
+      return Collections.singletonList(new AggregationInfo(aggregationFunctions, projectOperator, false));
     }
 
     // For each aggregation function, check if the aggregation function is a filtered aggregate. If so, populate the
     // corresponding filter operator.
-    Map<FilterContext, Pair<BaseFilterOperator, List<AggregationFunction>>> filterOperators = new HashMap<>();
+    Map<FilterContext, FilteredAggregationContext> filteredAggregationContexts = new HashMap<>();
     List<AggregationFunction> nonFilteredFunctions = new ArrayList<>();
+    FilterContext mainFilter = queryContext.getFilter();
     for (Pair<AggregationFunction, FilterContext> functionFilterPair : queryContext.getFilteredAggregationFunctions()) {
       AggregationFunction aggregationFunction = functionFilterPair.getLeft();
       FilterContext filter = functionFilterPair.getRight();
       if (filter != null) {
-        filterOperators.computeIfAbsent(filter, k -> {
-          BaseFilterOperator combinedFilterOperator;
-          BaseFilterOperator subFilterOperator = new FilterPlanNode(indexSegment, queryContext, filter).run();
-          if (mainFilterOperator.isResultMatchingAll()) {
-            combinedFilterOperator = subFilterOperator;
+        filteredAggregationContexts.computeIfAbsent(filter, k -> {
+          FilterContext combinedFilter;
+          if (mainFilter == null) {
+            combinedFilter = filter;
           } else {
-            if (subFilterOperator.isResultEmpty()) {
-              combinedFilterOperator = subFilterOperator;
-            } else if (subFilterOperator.isResultMatchingAll()) {
-              combinedFilterOperator = mainFilterOperator;
-            } else {
-              combinedFilterOperator =
-                  new CombinedFilterOperator(mainFilterOperator, subFilterOperator, queryContext.getQueryOptions());
-            }
+            combinedFilter = FilterContext.forAnd(List.of(mainFilter, filter));
           }
-          return Pair.of(combinedFilterOperator, new ArrayList<>());
-        }).getRight().add(aggregationFunction);
+
+          FilterPlanNode subFilterPlan = new FilterPlanNode(indexSegment, queryContext, filter);
+          BaseFilterOperator subFilterOperator = subFilterPlan.run();
+          BaseFilterOperator combinedFilterOperator;
+          if (mainFilterOperator.isResultMatchingAll() || subFilterOperator.isResultEmpty()) {
+            combinedFilterOperator = subFilterOperator;
+          } else if (subFilterOperator.isResultMatchingAll()) {
+            combinedFilterOperator = mainFilterOperator;
+          } else {
+            combinedFilterOperator =
+                new CombinedFilterOperator(mainFilterOperator, subFilterOperator, queryContext.getQueryOptions());
+          }
+
+          List<Pair<Predicate, PredicateEvaluator>> subPredicateEvaluators = subFilterPlan.getPredicateEvaluators();
+          List<Pair<Predicate, PredicateEvaluator>> combinedPredicateEvaluators =
+              new ArrayList<>(mainPredicateEvaluators.size() + subPredicateEvaluators.size());
+          combinedPredicateEvaluators.addAll(mainPredicateEvaluators);
+          combinedPredicateEvaluators.addAll(subPredicateEvaluators);
+
+          return new FilteredAggregationContext(combinedFilter, combinedFilterOperator, combinedPredicateEvaluators);
+        })._aggregationFunctions.add(aggregationFunction);
       } else {
         nonFilteredFunctions.add(aggregationFunction);
       }
     }
 
-    // Create the project operators
-    List<Pair<AggregationFunction[], BaseProjectOperator<?>>> projectOperators = new ArrayList<>();
-    List<ExpressionContext> groupByExpressions = queryContext.getGroupByExpressions();
-    for (Pair<BaseFilterOperator, List<AggregationFunction>> filterOperatorFunctionsPair : filterOperators.values()) {
-      BaseFilterOperator filterOperator = filterOperatorFunctionsPair.getLeft();
+    List<AggregationInfo> aggregationInfos = new ArrayList<>();
+    for (FilteredAggregationContext filteredAggregationContext : filteredAggregationContexts.values()) {
+      BaseFilterOperator filterOperator = filteredAggregationContext._filterOperator;
       if (filterOperator == mainFilterOperator) {
         // This can happen when the sub filter matches all documents, and we can treat the function as non-filtered
-        nonFilteredFunctions.addAll(filterOperatorFunctionsPair.getRight());
+        nonFilteredFunctions.addAll(filteredAggregationContext._aggregationFunctions);
       } else {
         AggregationFunction[] aggregationFunctions =
-            filterOperatorFunctionsPair.getRight().toArray(new AggregationFunction[0]);
-        Set<ExpressionContext> expressions = collectExpressionsToTransform(aggregationFunctions, groupByExpressions);
-        BaseProjectOperator<?> projectOperator =
-            new ProjectPlanNode(indexSegment, queryContext, expressions, DocIdSetPlanNode.MAX_DOC_PER_CALL,
-                filterOperator).run();
-        projectOperators.add(Pair.of(aggregationFunctions, projectOperator));
+            filteredAggregationContext._aggregationFunctions.toArray(new AggregationFunction[0]);
+        aggregationInfos.add(
+            buildAggregationInfo(indexSegment, queryContext, aggregationFunctions, filteredAggregationContext._filter,
+                filteredAggregationContext._filterOperator, filteredAggregationContext._predicateEvaluators));
       }
     }
 
     if (!nonFilteredFunctions.isEmpty()) {
       AggregationFunction[] aggregationFunctions = nonFilteredFunctions.toArray(new AggregationFunction[0]);
-      Set<ExpressionContext> expressions = collectExpressionsToTransform(aggregationFunctions, groupByExpressions);
-      BaseProjectOperator<?> projectOperator =
-          new ProjectPlanNode(indexSegment, queryContext, expressions, DocIdSetPlanNode.MAX_DOC_PER_CALL,
-              mainFilterOperator).run();
-      projectOperators.add(Pair.of(aggregationFunctions, projectOperator));
+      aggregationInfos.add(
+          buildAggregationInfo(indexSegment, queryContext, aggregationFunctions, mainFilter, mainFilterOperator,
+              mainPredicateEvaluators));
     }
 
-    return projectOperators;
+    return aggregationInfos;
+  }
+
+  private static class FilteredAggregationContext {
+    final FilterContext _filter;
+    final BaseFilterOperator _filterOperator;
+    final List<Pair<Predicate, PredicateEvaluator>> _predicateEvaluators;
+    final List<AggregationFunction> _aggregationFunctions = new ArrayList<>();
+
+    public FilteredAggregationContext(FilterContext filter, BaseFilterOperator filterOperator,
+        List<Pair<Predicate, PredicateEvaluator>> predicateEvaluators) {
+      _filter = filter;
+      _filterOperator = filterOperator;
+      _predicateEvaluators = predicateEvaluators;
+    }
   }
 
   public static String getResultColumnName(AggregationFunction aggregationFunction, @Nullable FilterContext filter) {
