@@ -48,6 +48,7 @@ import org.apache.pinot.core.query.aggregation.AggregationResultHolder;
 import org.apache.pinot.core.query.aggregation.ObjectAggregationResultHolder;
 import org.apache.pinot.core.query.aggregation.groupby.GroupByResultHolder;
 import org.apache.pinot.core.query.aggregation.groupby.ObjectGroupByResultHolder;
+import org.apache.pinot.segment.local.customobject.ThetaSketchAccumulator;
 import org.apache.pinot.segment.spi.AggregationFunctionType;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.sql.parsers.CalciteSqlParser;
@@ -68,29 +69,31 @@ import org.apache.pinot.sql.parsers.CalciteSqlParser;
  *     'dimName=''course'' AND dimValue=''math''', 'SET_INTERSECT($1,$2)')
  *   </li>
  * </ul>
- * Currently there is only 1 parameter for the function:
+ * Currently, there are 3 parameters to the function:
  * <ul>
  *   <li>
  *     nominalEntries: The nominal entries used to create the sketch. (Default 4096)
+ *     samplingProbability: Sets the upfront uniform sampling probability, p. (Default 1.0)
+ *     accumulatorThreshold: How many sketches should be kept in memory before merging. (Default 2)
  *   </li>
  * </ul>
  * <p>E.g. DISTINCT_COUNT_THETA_SKETCH(col, 'nominalEntries=8192')
  */
 @SuppressWarnings({"rawtypes", "unchecked"})
 public class DistinctCountThetaSketchAggregationFunction
-    extends BaseSingleInputAggregationFunction<List<Sketch>, Comparable> {
+    extends BaseSingleInputAggregationFunction<List<ThetaSketchAccumulator>, Comparable> {
   private static final String SET_UNION = "setunion";
   private static final String SET_INTERSECT = "setintersect";
   private static final String SET_DIFF = "setdiff";
   private static final String DEFAULT_SKETCH_IDENTIFIER = "$0";
-  private static final Sketch EMPTY_SKETCH = new UpdateSketchBuilder().build().compact();
-
+  private static final int DEFAULT_ACCUMULATOR_THRESHOLD = 2;
   private final List<ExpressionContext> _inputExpressions;
   private final boolean _includeDefaultSketch;
   private final List<FilterEvaluator> _filterEvaluators;
   private final ExpressionContext _postAggregationExpression;
   private final UpdateSketchBuilder _updateSketchBuilder = new UpdateSketchBuilder();
-  private final SetOperationBuilder _setOperationBuilder = new SetOperationBuilder();
+  protected final SetOperationBuilder _setOperationBuilder = new SetOperationBuilder();
+  protected int _accumulatorThreshold = DEFAULT_ACCUMULATOR_THRESHOLD;
 
   public DistinctCountThetaSketchAggregationFunction(List<ExpressionContext> arguments) {
     super(arguments.get(0));
@@ -102,9 +105,16 @@ public class DistinctCountThetaSketchAggregationFunction
       Preconditions.checkArgument(paramsExpression.getType() == ExpressionContext.Type.LITERAL,
           "Second argument of DISTINCT_COUNT_THETA_SKETCH aggregation function must be literal (parameters)");
       Parameters parameters = new Parameters(paramsExpression.getLiteral().getStringValue());
+      // Allows the user to trade-off memory usage for merge CPU; higher values use more memory
+      _accumulatorThreshold = parameters.getAccumulatorThreshold();
+      // Nominal entries controls sketch accuracy and size
       int nominalEntries = parameters.getNominalEntries();
       _updateSketchBuilder.setNominalEntries(nominalEntries);
       _setOperationBuilder.setNominalEntries(nominalEntries);
+      // Sampling probability sets the initial value of Theta, defaults to 1.0
+      float p = parameters.getSamplingProbability();
+      _setOperationBuilder.setP(p);
+      _updateSketchBuilder.setP(p);
     }
 
     if (numArguments < 4) {
@@ -132,6 +142,7 @@ public class DistinctCountThetaSketchAggregationFunction
                 + "(filter expression)");
         FilterContext filter = RequestContextUtils.getFilter(
             CalciteSqlParser.compileToExpression(filterExpression.getLiteral().getStringValue()));
+        Preconditions.checkArgument(!filter.isConstant(), "Filter must not be constant: %s", filter);
         // NOTE: Collect expressions before constructing the FilterInfo so that expressionIndexMap always include the
         //       expressions in the filter.
         collectExpressions(filter, _inputExpressions, expressionIndexMap);
@@ -146,7 +157,6 @@ public class DistinctCountThetaSketchAggregationFunction
       Expression expr = CalciteSqlParser.compileToExpression(postAggregationExpression.getLiteral().getStringValue());
       _postAggregationExpression = RequestContextUtils.getExpression(expr);
 
-
       // Validate the post-aggregation expression
       _includeDefaultSketch = validatePostAggregationExpression(_postAggregationExpression, _filterEvaluators.size());
     }
@@ -401,20 +411,20 @@ public class DistinctCountThetaSketchAggregationFunction
       }
     } else {
       // Serialized sketch
-      List<Union> unions = getUnions(aggregationResultHolder);
+      List<ThetaSketchAccumulator> thetaSketchAccumulators = getUnions(aggregationResultHolder);
       Sketch[] sketches = deserializeSketches((byte[][]) valueArrays[0], length);
       if (_includeDefaultSketch) {
-        Union defaultUnion = unions.get(0);
+        ThetaSketchAccumulator defaultThetaAccumulator = thetaSketchAccumulators.get(0);
         for (Sketch sketch : sketches) {
-          defaultUnion.union(sketch);
+          defaultThetaAccumulator.apply(sketch);
         }
       }
       for (int i = 0; i < numFilters; i++) {
         FilterEvaluator filterEvaluator = _filterEvaluators.get(i);
-        Union union = unions.get(i + 1);
+        ThetaSketchAccumulator thetaSketchAccumulator = thetaSketchAccumulators.get(i + 1);
         for (int j = 0; j < length; j++) {
           if (filterEvaluator.evaluate(singleValues, valueTypes, valueArrays, j)) {
-            union.union(sketches[j]);
+            thetaSketchAccumulator.apply(sketches[j]);
           }
         }
       }
@@ -631,14 +641,14 @@ public class DistinctCountThetaSketchAggregationFunction
       // Serialized sketch
       Sketch[] sketches = deserializeSketches((byte[][]) valueArrays[0], length);
       for (int i = 0; i < length; i++) {
-        List<Union> unions = getUnions(groupByResultHolder, groupKeyArray[i]);
+        List<ThetaSketchAccumulator> thetaSketchAccumulators = getUnions(groupByResultHolder, groupKeyArray[i]);
         Sketch sketch = sketches[i];
         if (_includeDefaultSketch) {
-          unions.get(0).union(sketch);
+          thetaSketchAccumulators.get(0).apply(sketch);
         }
         for (int j = 0; j < numFilters; j++) {
           if (_filterEvaluators.get(j).evaluate(singleValues, valueTypes, valueArrays, i)) {
-            unions.get(j + 1).union(sketch);
+            thetaSketchAccumulators.get(j + 1).apply(sketch);
           }
         }
       }
@@ -907,7 +917,7 @@ public class DistinctCountThetaSketchAggregationFunction
       if (_includeDefaultSketch) {
         for (int i = 0; i < length; i++) {
           for (int groupKey : groupKeysArray[i]) {
-            getUnions(groupByResultHolder, groupKey).get(0).union(sketches[i]);
+            getUnions(groupByResultHolder, groupKey).get(0).apply(sketches[i]);
           }
         }
       }
@@ -916,7 +926,7 @@ public class DistinctCountThetaSketchAggregationFunction
         for (int j = 0; j < length; j++) {
           if (filterEvaluator.evaluate(singleValues, valueTypes, valueArrays, j)) {
             for (int groupKey : groupKeysArray[i]) {
-              getUnions(groupByResultHolder, groupKey).get(i + 1).union(sketches[i]);
+              getUnions(groupByResultHolder, groupKey).get(i + 1).apply(sketches[i]);
             }
           }
         }
@@ -925,57 +935,72 @@ public class DistinctCountThetaSketchAggregationFunction
   }
 
   @Override
-  public List<Sketch> extractAggregationResult(AggregationResultHolder aggregationResultHolder) {
+  public List<ThetaSketchAccumulator> extractAggregationResult(AggregationResultHolder aggregationResultHolder) {
     List result = aggregationResultHolder.getResult();
     if (result == null) {
       int numSketches = _filterEvaluators.size() + 1;
-      List<Sketch> sketches = new ArrayList<>(numSketches);
+      List<ThetaSketchAccumulator> sketches = new ArrayList<>(numSketches);
       for (int i = 0; i < numSketches; i++) {
-        sketches.add(EMPTY_SKETCH);
+        sketches.add(new ThetaSketchAccumulator(_setOperationBuilder, _accumulatorThreshold));
       }
       return sketches;
     }
 
     if (result.get(0) instanceof Sketch) {
-      return result;
+      ArrayList<ThetaSketchAccumulator> thetaSketchAccumulators = new ArrayList<>(result.size());
+      for (Object o : result) {
+        ThetaSketchAccumulator thetaSketchAccumulator = convertSketchAccumulator(o);
+        thetaSketchAccumulators.add(thetaSketchAccumulator);
+      }
+      return thetaSketchAccumulators;
     } else {
-      return convertToSketches(result);
+      return result;
     }
   }
 
   @Override
-  public List<Sketch> extractGroupByResult(GroupByResultHolder groupByResultHolder, int groupKey) {
+  public List<ThetaSketchAccumulator> extractGroupByResult(GroupByResultHolder groupByResultHolder, int groupKey) {
     List result = groupByResultHolder.getResult(groupKey);
-    if (result.get(0) instanceof Sketch) {
-      return result;
-    } else {
-      return convertToSketches(result);
+    if (result == null) {
+      int numSketches = _filterEvaluators.size() + 1;
+      List<ThetaSketchAccumulator> thetaSketchAccumulators = new ArrayList<>(numSketches);
+      for (int i = 0; i < numSketches; i++) {
+        thetaSketchAccumulators.add(new ThetaSketchAccumulator(_setOperationBuilder, _accumulatorThreshold));
+      }
+      return thetaSketchAccumulators;
     }
+
+    if (result.get(0) instanceof Sketch) {
+      ArrayList<ThetaSketchAccumulator> thetaSketchAccumulators = new ArrayList<>(result.size());
+      for (Object o : result) {
+        ThetaSketchAccumulator thetaSketchAccumulator = convertSketchAccumulator(o);
+        thetaSketchAccumulators.add(thetaSketchAccumulator);
+      }
+      return thetaSketchAccumulators;
+    }
+
+    return result;
   }
 
   @Override
-  public List<Sketch> merge(List<Sketch> sketches1, List<Sketch> sketches2) {
-    int numSketches = sketches1.size();
-    List<Sketch> mergedSketches = new ArrayList<>(numSketches);
-    for (int i = 0; i < numSketches; i++) {
-      Sketch sketch1 = sketches1.get(i);
-      Sketch sketch2 = sketches2.get(i);
-      if (sketch1.isEmpty()) {
-        mergedSketches.add(sketch2);
+  public List<ThetaSketchAccumulator> merge(List<ThetaSketchAccumulator> acc1, List<ThetaSketchAccumulator> acc2) {
+    int numAccumulators = acc1.size();
+    List<ThetaSketchAccumulator> mergedAccumulators = new ArrayList<>(numAccumulators);
+    for (int i = 0; i < numAccumulators; i++) {
+      ThetaSketchAccumulator thetaSketchAccumulator1 = convertSketchAccumulator(acc1.get(i));
+      ThetaSketchAccumulator thetaSketchAccumulator2 = convertSketchAccumulator(acc2.get(i));
+      if (thetaSketchAccumulator1.isEmpty()) {
+        mergedAccumulators.add(thetaSketchAccumulator2);
         continue;
       }
-      if (sketch2.isEmpty()) {
-        mergedSketches.add(sketch1);
+      if (thetaSketchAccumulator2.isEmpty()) {
+        mergedAccumulators.add(thetaSketchAccumulator1);
         continue;
       }
-      Union union = _setOperationBuilder.buildUnion();
-      union.union(sketch1);
-      union.union(sketch2);
-      // NOTE: Compact the sketch in unsorted, on-heap fashion for performance concern.
-      //       See https://datasketches.apache.org/docs/Theta/ThetaSize.html for more details.
-      mergedSketches.add(union.getResult(false, null));
+      thetaSketchAccumulator1.merge(thetaSketchAccumulator2);
+      mergedAccumulators.add(thetaSketchAccumulator1);
     }
-    return mergedSketches;
+    return mergedAccumulators;
   }
 
   @Override
@@ -989,8 +1014,32 @@ public class DistinctCountThetaSketchAggregationFunction
   }
 
   @Override
-  public Comparable extractFinalResult(List<Sketch> sketches) {
-    return Math.round(evaluatePostAggregationExpression(_postAggregationExpression, sketches).getEstimate());
+  public Comparable extractFinalResult(List<ThetaSketchAccumulator> accumulators) {
+    int numAccumulators = accumulators.size();
+    List<Sketch> mergedSketches = new ArrayList<>(numAccumulators);
+
+    for (Object accumulatorObject : accumulators) {
+      ThetaSketchAccumulator accumulator = convertSketchAccumulator(accumulatorObject);
+      accumulator.setThreshold(_accumulatorThreshold);
+      accumulator.setSetOperationBuilder(_setOperationBuilder);
+      mergedSketches.add(accumulator.getResult());
+    }
+
+    return Math.round(evaluatePostAggregationExpression(_postAggregationExpression, mergedSketches).getEstimate());
+  }
+
+  // This ensures backward compatibility with servers that still return sketches directly.
+  // The AggregationDataTableReducer casts intermediate results to Objects and although the code compiles,
+  // types might still be incompatible at runtime due to type erasure.
+  // Due to performance overheads of redundant casts, this should be removed at some future point.
+  protected ThetaSketchAccumulator convertSketchAccumulator(Object result) {
+    if (result instanceof Sketch) {
+      Sketch sketch = (Sketch) result;
+      ThetaSketchAccumulator accumulator = new ThetaSketchAccumulator(_setOperationBuilder, _accumulatorThreshold);
+      accumulator.apply(sketch);
+      return accumulator;
+    }
+    return (ThetaSketchAccumulator) result;
   }
 
   /**
@@ -1172,8 +1221,8 @@ public class DistinctCountThetaSketchAggregationFunction
   /**
    * Returns the Union list from the result holder or creates a new one if it does not exist.
    */
-  private List<Union> getUnions(AggregationResultHolder aggregationResultHolder) {
-    List<Union> unions = aggregationResultHolder.getResult();
+  private List<ThetaSketchAccumulator> getUnions(AggregationResultHolder aggregationResultHolder) {
+    List<ThetaSketchAccumulator> unions = aggregationResultHolder.getResult();
     if (unions == null) {
       unions = buildUnions();
       aggregationResultHolder.setValue(unions);
@@ -1196,8 +1245,8 @@ public class DistinctCountThetaSketchAggregationFunction
   /**
    * Returns the Union list for the given group key or creates a new one if it does not exist.
    */
-  private List<Union> getUnions(GroupByResultHolder groupByResultHolder, int groupKey) {
-    List<Union> unions = groupByResultHolder.getResult(groupKey);
+  private List<ThetaSketchAccumulator> getUnions(GroupByResultHolder groupByResultHolder, int groupKey) {
+    List<ThetaSketchAccumulator> unions = groupByResultHolder.getResult(groupKey);
     if (unions == null) {
       unions = buildUnions();
       groupByResultHolder.setValueForKey(groupKey, unions);
@@ -1220,11 +1269,13 @@ public class DistinctCountThetaSketchAggregationFunction
   /**
    * Builds the Union list.
    */
-  private List<Union> buildUnions() {
+  private List<ThetaSketchAccumulator> buildUnions() {
     int numUnions = _filterEvaluators.size() + 1;
-    List<Union> unions = new ArrayList<>(numUnions);
+    List<ThetaSketchAccumulator> unions = new ArrayList<>(numUnions);
     for (int i = 0; i < numUnions; i++) {
-      unions.add(_setOperationBuilder.buildUnion());
+      ThetaSketchAccumulator thetaSketchAccumulator =
+          new ThetaSketchAccumulator(_setOperationBuilder, _accumulatorThreshold);
+      unions.add(thetaSketchAccumulator);
     }
     return unions;
   }
@@ -1240,20 +1291,6 @@ public class DistinctCountThetaSketchAggregationFunction
     return sketches;
   }
 
-  /**
-   * Converts the given Unions to Sketches.
-   */
-  private List<Sketch> convertToSketches(List<Union> unions) {
-    int numUnions = unions.size();
-    List<Sketch> sketches = new ArrayList<>(numUnions);
-    for (Union union : unions) {
-      // NOTE: Compact the sketch in unsorted, on-heap fashion for performance concern.
-      //       See https://datasketches.apache.org/docs/Theta/ThetaSize.html for more details.
-      sketches.add(union.getResult(false, null));
-    }
-    return sketches;
-  }
-
   /**
    * Evaluates the post-aggregation expression.
    */
@@ -1269,8 +1306,6 @@ public class DistinctCountThetaSketchAggregationFunction
       return sketches.get(extractSketchId(expression.getIdentifier()));
     }
 
-    // NOTE: Compact the sketch in unsorted, on-heap fashion for performance concern.
-    //       See https://datasketches.apache.org/docs/Theta/ThetaSize.html for more details.
     FunctionContext function = expression.getFunction();
     String functionName = function.getFunctionName();
     List<ExpressionContext> arguments = function.getArguments();
@@ -1298,14 +1333,19 @@ public class DistinctCountThetaSketchAggregationFunction
   }
 
   /**
-   * Helper class to wrap the theta-sketch parameters.
+   * Helper class to wrap the theta-sketch parameters.  The initial values for the parameters are set to the
+   * same defaults in the Apache Datasketches library.
    */
   private static class Parameters {
     private static final char PARAMETER_DELIMITER = ';';
     private static final char PARAMETER_KEY_VALUE_SEPARATOR = '=';
     private static final String NOMINAL_ENTRIES_KEY = "nominalEntries";
+    private static final String SAMPLING_PROBABILITY_KEY = "samplingProbability";
+    private static final String ACCUMULATOR_THRESHOLD_KEY = "accumulatorThreshold";
 
     private int _nominalEntries = ThetaUtil.DEFAULT_NOMINAL_ENTRIES;
+    private int _accumulatorThreshold = DEFAULT_ACCUMULATOR_THRESHOLD;
+    private float _samplingProbability = 1.0F;
 
     Parameters(String parametersString) {
       StringUtils.deleteWhitespace(parametersString);
@@ -1317,6 +1357,10 @@ public class DistinctCountThetaSketchAggregationFunction
         String value = keyAndValue[1];
         if (key.equalsIgnoreCase(NOMINAL_ENTRIES_KEY)) {
           _nominalEntries = Integer.parseInt(value);
+        } else if (key.equalsIgnoreCase(SAMPLING_PROBABILITY_KEY)) {
+          _samplingProbability = Float.parseFloat(value);
+        } else if (key.equalsIgnoreCase(ACCUMULATOR_THRESHOLD_KEY)) {
+          _accumulatorThreshold = Integer.parseInt(value);
         } else {
           throw new IllegalArgumentException("Invalid parameter key: " + key);
         }
@@ -1326,6 +1370,14 @@ public class DistinctCountThetaSketchAggregationFunction
     int getNominalEntries() {
       return _nominalEntries;
     }
+
+    float getSamplingProbability() {
+      return _samplingProbability;
+    }
+
+    int getAccumulatorThreshold() {
+      return _accumulatorThreshold;
+    }
   }
 
   /**
