@@ -20,7 +20,7 @@ package org.apache.pinot.core.query.distinct.dictionary;
 
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.core.common.BlockValSet;
-import org.apache.pinot.core.operator.blocks.TransformBlock;
+import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.query.distinct.DistinctExecutor;
 import org.apache.pinot.segment.spi.index.reader.Dictionary;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
@@ -37,9 +37,9 @@ public class DictionaryBasedSingleColumnDistinctOnlyExecutor extends BaseDiction
   }
 
   @Override
-  public boolean process(TransformBlock transformBlock) {
-    BlockValSet blockValueSet = transformBlock.getBlockValueSet(_expression);
-    int numDocs = transformBlock.getNumDocs();
+  public boolean process(ValueBlock valueBlock) {
+    BlockValSet blockValueSet = valueBlock.getBlockValueSet(_expression);
+    int numDocs = valueBlock.getNumDocs();
     if (blockValueSet.isSingleValue()) {
       int[] dictIds = blockValueSet.getDictionaryIdsSV();
       for (int i = 0; i < numDocs; i++) {
