@@ -23,7 +23,7 @@ import it.unimi.dsi.fastutil.ints.IntPriorityQueue;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.OrderByExpressionContext;
 import org.apache.pinot.core.common.BlockValSet;
-import org.apache.pinot.core.operator.blocks.TransformBlock;
+import org.apache.pinot.core.operator.blocks.ValueBlock;
 import org.apache.pinot.core.query.distinct.DistinctExecutor;
 import org.apache.pinot.segment.spi.index.reader.Dictionary;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
@@ -47,9 +47,9 @@ public class DictionaryBasedSingleColumnDistinctOrderByExecutor
   }
 
   @Override
-  public boolean process(TransformBlock transformBlock) {
-    BlockValSet blockValueSet = transformBlock.getBlockValueSet(_expression);
-    int numDocs = transformBlock.getNumDocs();
+  public boolean process(ValueBlock valueBlock) {
+    BlockValSet blockValueSet = valueBlock.getBlockValueSet(_expression);
+    int numDocs = valueBlock.getNumDocs();
     if (blockValueSet.isSingleValue()) {
       int[] dictIds = blockValueSet.getDictionaryIdsSV();
       for (int i = 0; i < numDocs; i++) {
