@@ -83,16 +83,16 @@ abstract class BaseGapfillProcessor {
 
     List<ExpressionContext> args = _gapFillSelection.getFunction().getArguments();
 
-    _dateTimeFormatter = new DateTimeFormatSpec(args.get(1).getLiteralString());
-    _gapfillDateTimeGranularity = new DateTimeGranularitySpec(args.get(4).getLiteralString());
-    if (args.get(5).getLiteralString() == null) {
+    _dateTimeFormatter = new DateTimeFormatSpec(args.get(1).getLiteral().getStringValue());
+    _gapfillDateTimeGranularity = new DateTimeGranularitySpec(args.get(4).getLiteral().getStringValue());
+    if (args.get(5).getLiteral() == null) {
       _postGapfillDateTimeGranularity = _gapfillDateTimeGranularity;
     } else {
-      _postGapfillDateTimeGranularity = new DateTimeGranularitySpec(args.get(5).getLiteralString());
+      _postGapfillDateTimeGranularity = new DateTimeGranularitySpec(args.get(5).getLiteral().getStringValue());
     }
-    String start = args.get(2).getLiteralString();
+    String start = args.get(2).getLiteral().getStringValue();
     _startMs = truncate(_dateTimeFormatter.fromFormatToMillis(start));
-    String end = args.get(3).getLiteralString();
+    String end = args.get(3).getLiteral().getStringValue();
     _endMs = truncate(_dateTimeFormatter.fromFormatToMillis(end));
     _gapfillTimeBucketSize = _gapfillDateTimeGranularity.granularityToMillis();
     _postGapfillTimeBucketSize = _postGapfillDateTimeGranularity.granularityToMillis();
@@ -201,7 +201,7 @@ abstract class BaseGapfillProcessor {
       } else {
         FunctionContext functionContext = expressionContext.getFunction();
         AggregationFunction aggregationFunction =
-            AggregationFunctionFactory.getAggregationFunction(functionContext, _queryContext);
+            AggregationFunctionFactory.getAggregationFunction(functionContext, _queryContext.isNullHandlingEnabled());
         columnDataTypes[i] = aggregationFunction.getFinalResultColumnType();
         columnNames[i] = functionContext.toString();
       }
@@ -222,8 +222,8 @@ abstract class BaseGapfillProcessor {
     return epoch / sz * sz;
   }
 
-  protected List<Object[]> gapFillAndAggregate(
-      List<Object[]> rows, DataSchema dataSchema, DataSchema resultTableSchema) {
+  protected List<Object[]> gapFillAndAggregate(List<Object[]> rows, DataSchema dataSchema,
+      DataSchema resultTableSchema) {
     throw new UnsupportedOperationException("Not supported");
   }
 }
