@@ -51,6 +51,7 @@ public class ServerQueryRequest {
   private final boolean _enableTrace;
   private final boolean _enableStreaming;
   private final List<String> _segmentsToQuery;
+  private final List<String> _optionalSegments;
   private final QueryContext _queryContext;
 
   // Request id might not be unique across brokers or for request hitting a hybrid table. To solve that we may construct
@@ -71,6 +72,7 @@ public class ServerQueryRequest {
     _enableTrace = instanceRequest.isEnableTrace();
     _enableStreaming = enableStreaming;
     _segmentsToQuery = instanceRequest.getSearchSegments();
+    _optionalSegments = instanceRequest.getOptionalSegments();
     _queryContext = getQueryContext(instanceRequest.getQuery().getPinotQuery());
     _queryId = QueryIdUtils.getQueryId(_brokerId, _requestId,
         TableNameBuilder.getTableTypeFromTableName(_queryContext.getTableName()));
@@ -88,6 +90,8 @@ public class ServerQueryRequest {
     _enableStreaming = Boolean.parseBoolean(metadata.get(Request.MetadataKeys.ENABLE_STREAMING));
 
     _segmentsToQuery = serverRequest.getSegmentsList();
+    // TODO: support optional segments for GrpcQueryServer
+    _optionalSegments = null;
 
     BrokerRequest brokerRequest;
     String payloadType = metadata.getOrDefault(Request.MetadataKeys.PAYLOAD_TYPE, Request.PayloadType.SQL);
@@ -139,6 +143,10 @@ public class ServerQueryRequest {
     return _segmentsToQuery;
   }
 
+  public List<String> getOptionalSegments() {
+    return _optionalSegments;
+  }
+
   public QueryContext getQueryContext() {
     return _queryContext;
   }
