@@ -18,6 +18,8 @@
  */
 package org.apache.pinot.core.query.selection;
 
+import com.google.common.base.Preconditions;
+import it.unimi.dsi.fastutil.objects.Object2IntOpenHashMap;
 import java.io.IOException;
 import java.math.BigDecimal;
 import java.util.ArrayList;
@@ -25,12 +27,11 @@ import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
-import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
-import java.util.Map;
 import java.util.PriorityQueue;
 import java.util.Set;
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.OrderByExpressionContext;
@@ -166,26 +167,120 @@ public class SelectionOperatorUtils {
   }
 
   /**
-   * Constructs the final selection DataSchema based on the order of selection columns (data schema can have a
-   * different order, depending on order by clause)
-   * @param dataSchema data schema used for execution and ordering
-   * @param selectionColumns the selection order
-   * @return data schema for final results
+   * Returns the data schema and column indices of the final selection results based on the query and the data schema of
+   * the server response. See {@link #extractExpressions} for the column orders on the server side.
+   * NOTE: DO NOT rely on column name lookup across query context and data schema because the string representation of
+   *       expression can change, which will cause backward incompatibility.
    */
-  public static DataSchema getResultTableDataSchema(DataSchema dataSchema, List<String> selectionColumns) {
-    Map<String, ColumnDataType> columnNameToDataType = new HashMap<>();
-    String[] columnNames = dataSchema.getColumnNames();
-    ColumnDataType[] columnDataTypes = dataSchema.getColumnDataTypes();
-    int numColumns = columnNames.length;
-    for (int i = 0; i < numColumns; i++) {
-      columnNameToDataType.put(columnNames[i], columnDataTypes[i]);
+  public static Pair<DataSchema, int[]> getResultTableDataSchemaAndColumnIndices(QueryContext queryContext,
+      DataSchema dataSchema) {
+    List<ExpressionContext> selectExpressions = queryContext.getSelectExpressions();
+    int numSelectExpressions = selectExpressions.size();
+    ColumnDataType[] columnDataTypesInDataSchema = dataSchema.getColumnDataTypes();
+    int numColumnsInDataSchema = columnDataTypesInDataSchema.length;
+
+    // No order-by expression
+    // NOTE: Order-by expressions are ignored for queries with LIMIT 0.
+    List<OrderByExpressionContext> orderByExpressions = queryContext.getOrderByExpressions();
+    if (orderByExpressions == null || queryContext.getLimit() == 0) {
+      // For 'SELECT *', use the server response data schema as the final results data schema.
+      if ((numSelectExpressions == 1 && selectExpressions.get(0).equals(IDENTIFIER_STAR))) {
+        int[] columnIndices = new int[numColumnsInDataSchema];
+        for (int i = 0; i < numColumnsInDataSchema; i++) {
+          columnIndices[i] = i;
+        }
+        return Pair.of(dataSchema, columnIndices);
+      }
+
+      // For select without duplicate columns, the order of the final selection columns is the same as the order of the
+      // columns in the data schema.
+      if (numSelectExpressions == numColumnsInDataSchema) {
+        String[] columnNames = new String[numSelectExpressions];
+        int[] columnIndices = new int[numSelectExpressions];
+        for (int i = 0; i < numSelectExpressions; i++) {
+          columnNames[i] = selectExpressions.get(i).toString();
+          columnIndices[i] = i;
+        }
+        return Pair.of(new DataSchema(columnNames, columnDataTypesInDataSchema), columnIndices);
+      }
+
+      // For select with duplicate columns, construct a map from expression to index with the same order as the data
+      // schema, then look up the selection expressions.
+      Object2IntOpenHashMap<ExpressionContext> expressionIndexMap = new Object2IntOpenHashMap<>(numColumnsInDataSchema);
+      for (ExpressionContext selectExpression : selectExpressions) {
+        expressionIndexMap.putIfAbsent(selectExpression, expressionIndexMap.size());
+      }
+      Preconditions.checkState(expressionIndexMap.size() == numColumnsInDataSchema,
+          "BUG: Expect same number of deduped columns in SELECT clause and in data schema, got %s before dedup and %s"
+              + " after dedup in SELECT clause, %s in data schema", numSelectExpressions, expressionIndexMap.size(),
+          numColumnsInDataSchema);
+      String[] columnNames = new String[numSelectExpressions];
+      ColumnDataType[] columnDataTypes = new ColumnDataType[numSelectExpressions];
+      int[] columnIndices = new int[numSelectExpressions];
+      for (int i = 0; i < numSelectExpressions; i++) {
+        ExpressionContext selectExpression = selectExpressions.get(i);
+        int columnIndex = expressionIndexMap.getInt(selectExpression);
+        columnNames[i] = selectExpression.toString();
+        columnDataTypes[i] = columnDataTypesInDataSchema[columnIndex];
+        columnIndices[i] = columnIndex;
+      }
+      return Pair.of(new DataSchema(columnNames, columnDataTypes), columnIndices);
     }
-    int numResultColumns = selectionColumns.size();
-    ColumnDataType[] finalColumnDataTypes = new ColumnDataType[numResultColumns];
-    for (int i = 0; i < numResultColumns; i++) {
-      finalColumnDataTypes[i] = columnNameToDataType.get(selectionColumns.get(i));
+
+    // For 'SELECT *' with order-by, exclude transform functions from the returned columns and sort.
+    if (numSelectExpressions == 1 && selectExpressions.get(0).equals(IDENTIFIER_STAR)) {
+      String[] columnNamesInDataSchema = dataSchema.getColumnNames();
+      List<Integer> columnIndexList = new ArrayList<>(columnNamesInDataSchema.length);
+      for (int i = 0; i < columnNamesInDataSchema.length; i++) {
+        if (columnNamesInDataSchema[i].indexOf('(') == -1) {
+          columnIndexList.add(i);
+        }
+      }
+      columnIndexList.sort(Comparator.comparing(o -> columnNamesInDataSchema[o]));
+      int numColumns = columnIndexList.size();
+      String[] columnNames = new String[numColumns];
+      ColumnDataType[] columnDataTypes = new ColumnDataType[numColumns];
+      int[] columnIndices = new int[numColumns];
+      for (int i = 0; i < numColumns; i++) {
+        int columnIndex = columnIndexList.get(i);
+        columnNames[i] = columnNamesInDataSchema[columnIndex];
+        columnDataTypes[i] = columnDataTypesInDataSchema[columnIndex];
+        columnIndices[i] = columnIndex;
+      }
+      return Pair.of(new DataSchema(columnNames, columnDataTypes), columnIndices);
     }
-    return new DataSchema(selectionColumns.toArray(new String[0]), finalColumnDataTypes);
+
+    // For other order-by queries, construct a map from expression to index with the same order as the data schema,
+    // then look up the selection expressions.
+    Object2IntOpenHashMap<ExpressionContext> expressionIndexMap = new Object2IntOpenHashMap<>(numColumnsInDataSchema);
+    // NOTE: Order-by expressions are already deduped in QueryContext.
+    for (OrderByExpressionContext orderByExpression : orderByExpressions) {
+      expressionIndexMap.put(orderByExpression.getExpression(), expressionIndexMap.size());
+    }
+    for (ExpressionContext selectExpression : selectExpressions) {
+      expressionIndexMap.putIfAbsent(selectExpression, expressionIndexMap.size());
+    }
+    String[] columnNames = new String[numSelectExpressions];
+    ColumnDataType[] columnDataTypes = new ColumnDataType[numSelectExpressions];
+    int[] columnIndices = new int[numSelectExpressions];
+    if (expressionIndexMap.size() == numColumnsInDataSchema) {
+      for (int i = 0; i < numSelectExpressions; i++) {
+        ExpressionContext selectExpression = selectExpressions.get(i);
+        int columnIndex = expressionIndexMap.getInt(selectExpression);
+        columnNames[i] = selectExpression.toString();
+        columnDataTypes[i] = columnDataTypesInDataSchema[columnIndex];
+        columnIndices[i] = columnIndex;
+      }
+    } else {
+      // When all segments are pruned on the server side, the data schema will only contain the columns in the SELECT
+      // clause, and data type for all columns are set to STRING. See ResultBlocksUtils for details.
+      for (int i = 0; i < numSelectExpressions; i++) {
+        columnNames[i] = selectExpressions.get(i).toString();
+        columnDataTypes[i] = ColumnDataType.STRING;
+        columnIndices[i] = i;
+      }
+    }
+    return Pair.of(new DataSchema(columnNames, columnDataTypes), columnIndices);
   }
 
   /**
@@ -259,6 +354,8 @@ public class SelectionOperatorUtils {
 
   /**
    * Build a {@link DataTable} from a {@link Collection} of selection rows with {@link DataSchema}. (Server side)
+   *
+   * This method is allowed to modify the given rows. Specifically, it may remove nulls cells from it.
    */
   public static DataTable getDataTableFromRows(Collection<Object[]> rows, DataSchema dataSchema,
       boolean nullHandlingEnabled)
@@ -288,7 +385,9 @@ public class SelectionOperatorUtils {
       }
     }
 
+    int numRowsAdded = 0;
     for (Object[] row : rows) {
+      Tracing.ThreadAccountantOps.sampleAndCheckInterruptionPeriodically(numRowsAdded);
       dataTableBuilder.startRow();
       for (int i = 0; i < numColumns; i++) {
         Object columnValue = row[i];
@@ -343,6 +442,7 @@ public class SelectionOperatorUtils {
         }
       }
       dataTableBuilder.finishRow();
+      numRowsAdded++;
     }
 
     if (nullHandlingEnabled) {
@@ -477,82 +577,26 @@ public class SelectionOperatorUtils {
   }
 
   /**
-   * Render the selection rows to a {@link ResultTable} object
-   * for selection queries without <code>ORDER BY</code>
-   * <p>{@link ResultTable} object will be used to set in the broker response.
-   * <p>Should be called after method "reduceWithoutOrdering()".
-   *
-   * @param rows selection rows.
-   * @param dataSchema data schema.
-   * @param selectionColumns selection columns.
-   * @return {@link ResultTable} object results.
+   * Renders the selection rows to a {@link ResultTable} object for selection queries without <code>ORDER BY</code>.
+   * (Broker side)
    */
   public static ResultTable renderResultTableWithoutOrdering(List<Object[]> rows, DataSchema dataSchema,
-      List<String> selectionColumns) {
+      int[] columnIndices) {
     int numRows = rows.size();
     List<Object[]> resultRows = new ArrayList<>(numRows);
-
-    DataSchema resultDataSchema = dataSchema;
-    Map<String, Integer> columnNameToIndexMap = null;
-    if (dataSchema.getColumnNames().length != selectionColumns.size()) {
-      // Create updated data schema since one column can be selected multiple times.
-      columnNameToIndexMap = new HashMap<>(dataSchema.getColumnNames().length);
-      String[] columnNames = dataSchema.getColumnNames();
-      ColumnDataType[] columnDataTypes = dataSchema.getColumnDataTypes();
-      for (int i = 0; i < columnNames.length; i++) {
-        columnNameToIndexMap.put(columnNames[i], i);
-      }
-
-      ColumnDataType[] newColumnDataTypes = new ColumnDataType[selectionColumns.size()];
-      for (int i = 0; i < newColumnDataTypes.length; i++) {
-        int index = columnNameToIndexMap.get(selectionColumns.get(i));
-        newColumnDataTypes[i] = columnDataTypes[index];
-      }
-
-      resultDataSchema = new DataSchema(selectionColumns.toArray(new String[0]), newColumnDataTypes);
-    }
-
-    int numColumns = resultDataSchema.getColumnNames().length;
-    ColumnDataType[] resultColumnDataTypes = resultDataSchema.getColumnDataTypes();
+    ColumnDataType[] columnDataTypes = dataSchema.getColumnDataTypes();
+    int numColumns = columnDataTypes.length;
     for (Object[] row : rows) {
       Object[] resultRow = new Object[numColumns];
       for (int i = 0; i < numColumns; i++) {
-        int index = (columnNameToIndexMap != null) ? columnNameToIndexMap.get(selectionColumns.get(i)) : i;
-        Object value = row[index];
+        Object value = row[columnIndices[i]];
         if (value != null) {
-          resultRow[i] = resultColumnDataTypes[i].convertAndFormat(value);
+          resultRow[i] = columnDataTypes[i].convertAndFormat(value);
         }
       }
       resultRows.add(resultRow);
     }
-
-    return new ResultTable(resultDataSchema, resultRows);
-  }
-
-  /**
-   * Helper method to compute column indices from selection columns and the data schema for selection queries
-   * @param selectionColumns selection columns.
-   * @param dataSchema data schema.
-   * @return column indices
-   */
-  public static int[] getColumnIndices(List<String> selectionColumns, DataSchema dataSchema) {
-    String[] columnNames = dataSchema.getColumnNames();
-    Map<String, Integer> columnToIndexMap = getColumnToIndexMap(columnNames);
-    int numSelectionColumns = selectionColumns.size();
-    int[] columnIndices = new int[numSelectionColumns];
-    for (int i = 0; i < numSelectionColumns; i++) {
-      columnIndices[i] = columnToIndexMap.get(selectionColumns.get(i));
-    }
-    return columnIndices;
-  }
-
-  public static Map<String, Integer> getColumnToIndexMap(String[] columns) {
-    Map<String, Integer> columnToIndexMap = new HashMap<>();
-    int numColumns = columns.length;
-    for (int i = 0; i < numColumns; i++) {
-      columnToIndexMap.put(columns[i], i);
-    }
-    return columnToIndexMap;
+    return new ResultTable(dataSchema, resultRows);
   }
 
   /**
@@ -571,19 +615,4 @@ public class SelectionOperatorUtils {
       queue.offer(value);
     }
   }
-
-  public static DataSchema getSchemaForProjection(DataSchema dataSchema, int[] columnIndices) {
-    int numColumns = columnIndices.length;
-
-    String[] columnNames = dataSchema.getColumnNames();
-    ColumnDataType[] columnDataTypes = dataSchema.getColumnDataTypes();
-    String[] resultColumnNames = new String[numColumns];
-    ColumnDataType[] resultColumnDataTypes = new ColumnDataType[numColumns];
-    for (int i = 0; i < numColumns; i++) {
-      int columnIndex = columnIndices[i];
-      resultColumnNames[i] = columnNames[columnIndex];
-      resultColumnDataTypes[i] = columnDataTypes[columnIndex];
-    }
-    return new DataSchema(resultColumnNames, resultColumnDataTypes);
-  }
 }
