@@ -32,13 +32,17 @@ import org.apache.commons.lang3.tuple.Pair;
 import org.apache.pinot.common.request.context.ExpressionContext;
 import org.apache.pinot.common.request.context.FilterContext;
 import org.apache.pinot.common.request.context.predicate.Predicate;
+import org.apache.pinot.core.operator.BaseProjectOperator;
 import org.apache.pinot.core.operator.filter.predicate.PredicateEvaluator;
 import org.apache.pinot.core.query.aggregation.function.AggregationFunction;
 import org.apache.pinot.core.query.aggregation.function.AggregationFunctionUtils;
+import org.apache.pinot.core.query.request.context.QueryContext;
+import org.apache.pinot.core.startree.plan.StarTreeProjectPlanNode;
 import org.apache.pinot.segment.spi.IndexSegment;
 import org.apache.pinot.segment.spi.datasource.DataSource;
 import org.apache.pinot.segment.spi.index.reader.Dictionary;
 import org.apache.pinot.segment.spi.index.startree.AggregationFunctionColumnPair;
+import org.apache.pinot.segment.spi.index.startree.StarTreeV2;
 import org.apache.pinot.segment.spi.index.startree.StarTreeV2Metadata;
 
 
@@ -60,7 +64,7 @@ public class StarTreeUtils {
         new AggregationFunctionColumnPair[numAggregationFunctions];
     for (int i = 0; i < numAggregationFunctions; i++) {
       AggregationFunctionColumnPair aggregationFunctionColumnPair =
-          AggregationFunctionUtils.getAggregationFunctionColumnPair(aggregationFunctions[i]);
+          AggregationFunctionUtils.getStoredFunctionColumnPair(aggregationFunctions[i]);
       if (aggregationFunctionColumnPair != null) {
         aggregationFunctionColumnPairs[i] = aggregationFunctionColumnPair;
       } else {
@@ -115,10 +119,10 @@ public class StarTreeUtils {
           return null;
         case PREDICATE:
           Predicate predicate = filterNode.getPredicate();
-          PredicateEvaluator predicateEvaluator = getPredicateEvaluator(indexSegment, predicate,
-              predicateEvaluatorMapping);
-          if (predicateEvaluator == null) {
-            // The predicate cannot be solved with star-tree
+          PredicateEvaluator predicateEvaluator =
+              getPredicateEvaluator(indexSegment, predicate, predicateEvaluatorMapping);
+          // Do not use star-tree when the predicate cannot be solved with star-tree or is always false
+          if (predicateEvaluator == null || predicateEvaluator.isAlwaysFalse()) {
             return null;
           }
           if (!predicateEvaluator.isAlwaysTrue()) {
@@ -210,6 +214,10 @@ public class StarTreeUtils {
         predicateEvaluators.add(predicateEvaluator);
       }
     }
+    // When all predicates are always false, do not use star-tree
+    if (predicateEvaluators.isEmpty()) {
+      return null;
+    }
     return Pair.of(identifier, predicateEvaluators);
   }
 
@@ -230,7 +238,6 @@ public class StarTreeUtils {
           if (!extractOrClausePredicates(child, predicates)) {
             return false;
           }
-          predicates.add(child.getPredicate());
           break;
         case PREDICATE:
           predicates.add(child.getPredicate());
@@ -274,10 +281,44 @@ public class StarTreeUtils {
         break;
     }
     for (Pair<Predicate, PredicateEvaluator> pair : predicatesEvaluatorMapping) {
-      if (pair.getKey().equals(predicate)) {
+      if (pair.getKey() == predicate) {
         return pair.getValue();
       }
     }
     return null;
   }
+
+  /**
+   * Returns a {@link BaseProjectOperator} when the filter can be solved with star-tree, or {@code null} otherwise.
+   */
+  @Nullable
+  public static BaseProjectOperator<?> createStarTreeBasedProjectOperator(IndexSegment indexSegment,
+      QueryContext queryContext, AggregationFunction[] aggregationFunctions, @Nullable FilterContext filter,
+      List<Pair<Predicate, PredicateEvaluator>> predicateEvaluators) {
+    List<StarTreeV2> starTrees = indexSegment.getStarTrees();
+    if (starTrees == null || queryContext.isSkipStarTree() || queryContext.isNullHandlingEnabled()) {
+      return null;
+    }
+    AggregationFunctionColumnPair[] aggregationFunctionColumnPairs =
+        extractAggregationFunctionPairs(aggregationFunctions);
+    if (aggregationFunctionColumnPairs == null) {
+      return null;
+    }
+    Map<String, List<CompositePredicateEvaluator>> predicateEvaluatorsMap =
+        extractPredicateEvaluatorsMap(indexSegment, filter, predicateEvaluators);
+    if (predicateEvaluatorsMap == null) {
+      return null;
+    }
+    ExpressionContext[] groupByExpressions =
+        queryContext.getGroupByExpressions() != null ? queryContext.getGroupByExpressions()
+            .toArray(new ExpressionContext[0]) : null;
+    for (StarTreeV2 starTreeV2 : starTrees) {
+      if (isFitForStarTree(starTreeV2.getMetadata(), aggregationFunctionColumnPairs, groupByExpressions,
+          predicateEvaluatorsMap.keySet())) {
+        return new StarTreeProjectPlanNode(queryContext, starTreeV2, aggregationFunctionColumnPairs, groupByExpressions,
+            predicateEvaluatorsMap).run();
+      }
+    }
+    return null;
+  }
 }
