@@ -19,20 +19,30 @@
 package org.apache.pinot.core.common;
 
 import com.clearspring.analytics.stream.cardinality.HyperLogLog;
+import com.dynatrace.hash4j.distinctcount.UltraLogLog;
 import com.tdunning.math.stats.TDigest;
 import it.unimi.dsi.fastutil.doubles.Double2LongOpenHashMap;
 import it.unimi.dsi.fastutil.doubles.DoubleArrayList;
 import it.unimi.dsi.fastutil.floats.Float2LongOpenHashMap;
+import it.unimi.dsi.fastutil.floats.FloatArrayList;
 import it.unimi.dsi.fastutil.ints.Int2LongOpenHashMap;
+import it.unimi.dsi.fastutil.ints.IntArrayList;
 import it.unimi.dsi.fastutil.ints.IntOpenHashSet;
 import it.unimi.dsi.fastutil.ints.IntSet;
 import it.unimi.dsi.fastutil.longs.Long2LongOpenHashMap;
+import it.unimi.dsi.fastutil.longs.LongArrayList;
+import it.unimi.dsi.fastutil.objects.ObjectArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Random;
 import org.apache.commons.lang.RandomStringUtils;
+import org.apache.datasketches.cpc.CpcSketch;
+import org.apache.datasketches.theta.SetOperationBuilder;
+import org.apache.datasketches.theta.Sketch;
+import org.apache.datasketches.theta.Sketches;
+import org.apache.datasketches.theta.UpdateSketch;
 import org.apache.pinot.core.query.aggregation.function.PercentileEstAggregationFunction;
 import org.apache.pinot.core.query.aggregation.function.PercentileTDigestAggregationFunction;
 import org.apache.pinot.segment.local.customobject.AvgPair;
@@ -43,7 +53,9 @@ import org.apache.pinot.segment.local.customobject.LongLongPair;
 import org.apache.pinot.segment.local.customobject.MinMaxRangePair;
 import org.apache.pinot.segment.local.customobject.QuantileDigest;
 import org.apache.pinot.segment.local.customobject.StringLongPair;
+import org.apache.pinot.segment.local.customobject.ThetaSketchAccumulator;
 import org.apache.pinot.segment.local.customobject.ValueLongPair;
+import org.apache.pinot.segment.local.utils.UltraLogLogUtils;
 import org.testng.annotations.Test;
 
 import static org.testng.Assert.assertEquals;
@@ -374,4 +386,140 @@ public class ObjectSerDeUtilsTest {
       assertEquals(actual, expected, ERROR_MESSAGE);
     }
   }
+
+  @Test
+  public void testCpcSketch() {
+    for (int i = 0; i < NUM_ITERATIONS; i++) {
+      CpcSketch sketch = new CpcSketch();
+      int size = RANDOM.nextInt(100) + 1;
+      for (int j = 0; j < size; j++) {
+        sketch.update(RANDOM.nextLong());
+      }
+
+      byte[] bytes = ObjectSerDeUtils.serialize(sketch);
+      CpcSketch actual =
+          ObjectSerDeUtils.deserialize(bytes, ObjectSerDeUtils.ObjectType.CompressedProbabilisticCounting);
+
+      assertEquals(actual.getEstimate(), sketch.getEstimate(), ERROR_MESSAGE);
+    }
+  }
+
+  @Test
+  public void testIntArrayList() {
+    for (int i = 0; i < NUM_ITERATIONS; i++) {
+      int size = RANDOM.nextInt(100);
+      IntArrayList expected = new IntArrayList(size);
+      for (int j = 0; j < size; j++) {
+        expected.add(RANDOM.nextInt());
+      }
+      byte[] bytes = ObjectSerDeUtils.serialize(expected);
+      IntArrayList actual = ObjectSerDeUtils.deserialize(bytes, ObjectSerDeUtils.ObjectType.IntArrayList);
+      assertEquals(actual, expected, ERROR_MESSAGE);
+    }
+  }
+
+  @Test
+  public void testLongArrayList() {
+    for (int i = 0; i < NUM_ITERATIONS; i++) {
+      int size = RANDOM.nextInt(100);
+      LongArrayList expected = new LongArrayList(size);
+      for (int j = 0; j < size; j++) {
+        expected.add(RANDOM.nextLong());
+      }
+      byte[] bytes = ObjectSerDeUtils.serialize(expected);
+      LongArrayList actual = ObjectSerDeUtils.deserialize(bytes, ObjectSerDeUtils.ObjectType.LongArrayList);
+      assertEquals(actual, expected, ERROR_MESSAGE);
+    }
+  }
+
+  @Test
+  public void testFloatArrayList() {
+    for (int i = 0; i < NUM_ITERATIONS; i++) {
+      int size = RANDOM.nextInt(100);
+      FloatArrayList expected = new FloatArrayList(size);
+      for (int j = 0; j < size; j++) {
+        expected.add(RANDOM.nextFloat());
+      }
+      byte[] bytes = ObjectSerDeUtils.serialize(expected);
+      FloatArrayList actual = ObjectSerDeUtils.deserialize(bytes, ObjectSerDeUtils.ObjectType.FloatArrayList);
+      assertEquals(actual, expected, ERROR_MESSAGE);
+    }
+  }
+
+  @Test
+  public void testStringArrayList() {
+    for (int i = 0; i < NUM_ITERATIONS; i++) {
+      int size = RANDOM.nextInt(100);
+      ObjectArrayList<String> expected = new ObjectArrayList<>(size);
+      for (int j = 0; j < size; j++) {
+        expected.add(RandomStringUtils.random(RANDOM.nextInt(20)));
+      }
+      byte[] bytes = ObjectSerDeUtils.serialize(expected);
+      ObjectArrayList<String> actual = ObjectSerDeUtils.deserialize(bytes, ObjectSerDeUtils.ObjectType.StringArrayList);
+      assertEquals(actual, expected, ERROR_MESSAGE);
+    }
+  }
+
+  @Test
+  public void testULL() {
+    for (int i = 0; i < NUM_ITERATIONS; i++) {
+      UltraLogLog ull = UltraLogLog.create(12);
+      int size = RANDOM.nextInt(100) + 1;
+      for (int j = 0; j < size; j++) {
+        UltraLogLogUtils.hashObject(RANDOM.nextLong()).ifPresent(ull::add);
+      }
+
+      byte[] bytes = ObjectSerDeUtils.serialize(ull);
+      UltraLogLog actual = ObjectSerDeUtils.deserialize(bytes, ObjectSerDeUtils.ObjectType.UltraLogLog);
+
+      assertEquals(actual.getDistinctCountEstimate(), ull.getDistinctCountEstimate(), ERROR_MESSAGE);
+      assertEquals(actual.getState(), ull.getState(), ERROR_MESSAGE);
+    }
+  }
+
+  @Test
+  public void testThetaSketch() {
+    for (int i = 0; i < NUM_ITERATIONS; i++) {
+      UpdateSketch input = Sketches.updateSketchBuilder().build();
+      int size = RANDOM.nextInt(100) + 10;
+      boolean shouldOrder = RANDOM.nextBoolean();
+
+      for (int j = 0; j < size; j++) {
+        input.update(j);
+      }
+
+      Sketch sketch = input.compact(shouldOrder, null);
+
+      byte[] bytes = ObjectSerDeUtils.serialize(sketch);
+      Sketch actual = ObjectSerDeUtils.deserialize(bytes, ObjectSerDeUtils.ObjectType.DataSketch);
+
+      assertEquals(actual.getEstimate(), sketch.getEstimate(), ERROR_MESSAGE);
+      assertEquals(actual.toByteArray(), sketch.toByteArray(), ERROR_MESSAGE);
+      assertEquals(actual.isOrdered(), shouldOrder, ERROR_MESSAGE);
+    }
+  }
+
+  @Test
+  public void testThetaSketchAccumulator() {
+    for (int i = 0; i < NUM_ITERATIONS; i++) {
+      UpdateSketch input = Sketches.updateSketchBuilder().build();
+      int size = RANDOM.nextInt(100) + 10;
+
+      for (int j = 0; j < size; j++) {
+        input.update(j);
+      }
+
+      SetOperationBuilder setOperationBuilder = new SetOperationBuilder();
+      ThetaSketchAccumulator accumulator = new ThetaSketchAccumulator(setOperationBuilder, 2);
+      Sketch sketch = input.compact(false, null);
+      accumulator.apply(sketch);
+
+      byte[] bytes = ObjectSerDeUtils.serialize(accumulator);
+      ThetaSketchAccumulator actual =
+          ObjectSerDeUtils.deserialize(bytes, ObjectSerDeUtils.ObjectType.ThetaSketchAccumulator);
+
+      assertEquals(actual.getResult().getEstimate(), sketch.getEstimate(), ERROR_MESSAGE);
+      assertEquals(actual.getResult().toByteArray(), sketch.toByteArray(), ERROR_MESSAGE);
+    }
+  }
 }
