@@ -241,6 +241,14 @@ public class DateTimeFunctionsTest {
         "fromDateTime(dateTime, 'yyyy-MM-dd''T''HH:mm:ss.SSS''Z''')", Lists.newArrayList("dateTime"), row113, null
     });
 
+    // fromDateTime with malformed dateTime and default Value should return -1
+    GenericRow row114 = new GenericRow();
+    row114.putValue("dateTime", "malformed_string");
+    inputs.add(new Object[]{
+        "fromDateTime(dateTime, 'yyyy-MM-dd''T''HH:mm:ss.SSS''Z''', 'UTC', -1)", Lists.newArrayList("dateTime"),
+        row114, -1L
+    });
+
     // timezone_hour and timezone_minute
     List<String> expectedArguments = Collections.singletonList("tz");
     GenericRow row120 = new GenericRow();
@@ -584,7 +592,7 @@ public class DateTimeFunctionsTest {
     row.putValue("timeCol", timeValue);
     List<String> arguments = Collections.singletonList("timeCol");
     testFunction(String.format("dateTimeConvert(timeCol, '%s', '%s', '%s')", inputFormatStr, outputFormatStr,
-        outputGranularityStr), arguments, row, expectedResult == null ? null : expectedResult.toString());
+        outputGranularityStr), arguments, row, expectedResult == null ? null : expectedResult);
   }
 
   private void testMultipleInvocations(String functionExpression, List<GenericRow> rows, List<Object> expectedResults) {
@@ -592,7 +600,7 @@ public class DateTimeFunctionsTest {
     int numInvocations = rows.size();
     assertEquals(expectedResults.size(), numInvocations);
     for (int i = 0; i < numInvocations; i++) {
-      assertEquals(evaluator.evaluate(rows.get(i)), expectedResults.get(i).toString());
+      assertEquals(evaluator.evaluate(rows.get(i)), expectedResults.get(i));
     }
   }
 
