@@ -26,7 +26,6 @@ import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import org.apache.commons.io.FileUtils;
-import org.apache.pinot.common.datatable.DataTableFactory;
 import org.apache.pinot.common.response.broker.BrokerResponseNative;
 import org.apache.pinot.common.response.broker.ResultTable;
 import org.apache.pinot.common.utils.DataSchema;
@@ -285,7 +284,6 @@ public class AllNullQueriesTest extends BaseQueriesTest {
 
   public void testQueries(ColumnDataType columnDataType, File indexDir)
       throws IOException {
-    DataTableBuilderFactory.setDataTableVersion(DataTableFactory.VERSION_4);
     Map<String, String> queryOptions = new HashMap<>();
     queryOptions.put("enableNullHandling", "true");
     DataType dataType = columnDataType.toDataType();
@@ -562,7 +560,7 @@ public class AllNullQueriesTest extends BaseQueriesTest {
           } else if (dataType == DataType.LONG) {
             assertEquals(((Double) row[1]).longValue(), Long.MIN_VALUE);
           }
-          assertEquals(row[2], 1);
+          assertEquals(row[2], 0);
         }
       }
       {
