@@ -27,7 +27,6 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import org.apache.commons.io.FileUtils;
-import org.apache.pinot.common.datatable.DataTableFactory;
 import org.apache.pinot.common.response.broker.BrokerResponseNative;
 import org.apache.pinot.common.response.broker.ResultTable;
 import org.apache.pinot.common.utils.DataSchema;
@@ -181,7 +180,6 @@ public class BooleanNullEnabledQueriesTest extends BaseQueriesTest {
   }
 
   public void testQueries() {
-    DataTableBuilderFactory.setDataTableVersion(DataTableFactory.VERSION_4);
     Map<String, String> queryOptions = new HashMap<>();
     queryOptions.put("enableNullHandling", "true");
     HashSet<Integer> trueIndices = new HashSet<Integer>(Arrays.asList(1, 3, 5));
@@ -361,7 +359,7 @@ public class BooleanNullEnabledQueriesTest extends BaseQueriesTest {
       }
     }
     {
-      String query = "SELECT * FROM testTable ORDER BY booleanColumn DESC LIMIT 4000";
+      String query = "SELECT * FROM testTable ORDER BY booleanColumn DESC NULLS LAST LIMIT 4000";
       BrokerResponseNative brokerResponse = getBrokerResponse(query, queryOptions);
       ResultTable resultTable = brokerResponse.getResultTable();
       DataSchema dataSchema = resultTable.getDataSchema();
@@ -382,7 +380,6 @@ public class BooleanNullEnabledQueriesTest extends BaseQueriesTest {
       for (int i = _trueValuesCount * 4 + _falseValuesCount * 4; i < 4000; i++) {
         Object[] row = rows.get(i);
         assertEquals(row.length, 1);
-        // Note 2: The default null ordering is 'NULLS LAST', regardless of the ordering direction.
         assertNull(row[0]);
       }
     }
