@@ -41,6 +41,7 @@ import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.data.readers.GenericRow;
+import org.apache.pinot.spi.exception.BadQueryRequestException;
 import org.apache.pinot.spi.utils.ReadMode;
 import org.apache.pinot.spi.utils.builder.TableConfigBuilder;
 import org.apache.pinot.sql.parsers.rewriter.QueryRewriterFactory;
@@ -56,7 +57,7 @@ import static org.testng.Assert.fail;
 
 
 /**
- * Queries test for argMin/argMax functions.
+ * Queries test for exprmin/exprmax functions.
  */
 public class ExprMinMaxTest extends BaseQueriesTest {
   private static final File INDEX_DIR = new File(FileUtils.getTempDirectory(), "ExprMinMaxTest");
@@ -201,7 +202,7 @@ public class ExprMinMaxTest extends BaseQueriesTest {
     query = "SELECT expr_max(mvDoubleColumn, jsonColumn) FROM testTable";
     brokerResponse = getBrokerResponse(query);
     Assert.assertTrue(brokerResponse.getProcessingExceptions().get(0).getMessage().contains(
-        "Cannot compute ArgMinMax measuring on non-comparable type: JSON"
+        "Cannot compute exprminMax measuring on non-comparable type: JSON"
     ));
   }
 
@@ -256,8 +257,8 @@ public class ExprMinMaxTest extends BaseQueriesTest {
     assertEquals(rows.get(1)[4], new String[]{"a0", "a01", "a02"});
     assertEquals(rows.get(0)[5], 0);
     assertEquals(rows.get(1)[5], 0);
-    assertEquals(rows.get(0)[6], new BigDecimal(360000));
-    assertEquals(rows.get(1)[6], new BigDecimal(360000));
+    assertEquals(rows.get(0)[6], "360000");
+    assertEquals(rows.get(1)[6], "360000");
     assertEquals(rows.get(0)[7], 600D);
     assertEquals(rows.get(1)[7], 600D);
     assertEquals(rows.get(0)[8], 1683138373879L - 1999L);
@@ -532,7 +533,7 @@ public class ExprMinMaxTest extends BaseQueriesTest {
     assertEquals(rows.get(4)[2], new Object[]{27});
 
     //  TODO: The following query works because whenever we find an empty array in the result, we use null
-    //        (see ArgMinMaxProjectionValSetWrapper). Ideally, we should be able to serialize empty array.
+    //        (see exprminMaxProjectionValSetWrapper). Ideally, we should be able to serialize empty array.
     //        requires fix for empty int arrays ser/de in DataBlock
     query =
         "SELECT stringColumn, expr_min(VALUE_IN(mvIntColumn,16,17,18,19,20,21,22,23,24,25,26,27), intColumn), "
@@ -610,9 +611,9 @@ public class ExprMinMaxTest extends BaseQueriesTest {
       ResultTable resultTable = brokerResponse.getResultTable();
       List<Object[]> rows = resultTable.getRows();
       fail();
-    } catch (Exception e) {
-      assertTrue(e.getMessage().contains("Aggregation function: exprmax(longColumn,intColumn) "
-          + "is only supported in selection without alias."));
+    } catch (BadQueryRequestException e) {
+      assertTrue(
+          e.getMessage().contains("Aggregation function: EXPRMAX is only supported in selection without alias."));
     }
   }
 
@@ -627,8 +628,8 @@ public class ExprMinMaxTest extends BaseQueriesTest {
       List<Object[]> rows = resultTable.getRows();
       fail();
     } catch (Exception e) {
-      assertTrue(e.getMessage().contains("Aggregation function: exprmax(longColumn,intColumn) "
-          + "is only supported in selection without alias."));
+      assertTrue(
+          e.getMessage().contains("Aggregation function: EXPRMAX is only supported in selection without alias."));
     }
   }
 
