@@ -58,7 +58,6 @@ import org.apache.pinot.spi.config.table.TableType;
 import org.apache.pinot.spi.config.table.UpsertConfig;
 import org.apache.pinot.spi.config.table.ingestion.IngestionConfig;
 import org.apache.pinot.spi.data.Schema;
-import org.apache.pinot.spi.stream.StreamConfig;
 import org.apache.pinot.spi.stream.StreamConfigProperties;
 import org.apache.pinot.spi.stream.StreamDataProducer;
 import org.apache.pinot.spi.stream.StreamDataProvider;
@@ -86,12 +85,9 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
       "On_Time_On_Time_Performance_2014_100k_subset_nonulls.tar.gz";
   protected static final long DEFAULT_COUNT_STAR_RESULT = 115545L;
   protected static final int DEFAULT_LLC_SEGMENT_FLUSH_SIZE = 5000;
-  protected static final int DEFAULT_HLC_SEGMENT_FLUSH_SIZE = 20000;
   protected static final int DEFAULT_TRANSACTION_NUM_KAFKA_BROKERS = 3;
   protected static final int DEFAULT_LLC_NUM_KAFKA_BROKERS = 2;
-  protected static final int DEFAULT_HLC_NUM_KAFKA_BROKERS = 1;
   protected static final int DEFAULT_LLC_NUM_KAFKA_PARTITIONS = 2;
-  protected static final int DEFAULT_HLC_NUM_KAFKA_PARTITIONS = 10;
   protected static final int DEFAULT_MAX_NUM_KAFKA_MESSAGES_PER_BATCH = 10000;
   protected static final List<String> DEFAULT_NO_DICTIONARY_COLUMNS =
       Arrays.asList("ActualElapsedTime", "ArrDelay", "DepDelay", "CRSDepTime");
@@ -120,10 +116,6 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
     return DEFAULT_TABLE_NAME;
   }
 
-  protected String getSchemaName() {
-    return DEFAULT_SCHEMA_NAME;
-  }
-
   protected String getSchemaFileName() {
     return DEFAULT_SCHEMA_FILE_NAME;
   }
@@ -141,10 +133,6 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
     return DEFAULT_COUNT_STAR_RESULT;
   }
 
-  protected boolean useLlc() {
-    return true;
-  }
-
   protected boolean useKafkaTransaction() {
     return false;
   }
@@ -154,22 +142,11 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
   }
 
   protected int getRealtimeSegmentFlushSize() {
-    if (useLlc()) {
-      return DEFAULT_LLC_SEGMENT_FLUSH_SIZE;
-    } else {
-      return DEFAULT_HLC_SEGMENT_FLUSH_SIZE;
-    }
+    return DEFAULT_LLC_SEGMENT_FLUSH_SIZE;
   }
 
   protected int getNumKafkaBrokers() {
-    if (useKafkaTransaction()) {
-      return DEFAULT_TRANSACTION_NUM_KAFKA_BROKERS;
-    }
-    if (useLlc()) {
-      return DEFAULT_LLC_NUM_KAFKA_BROKERS;
-    } else {
-      return DEFAULT_HLC_NUM_KAFKA_BROKERS;
-    }
+    return useKafkaTransaction() ? DEFAULT_TRANSACTION_NUM_KAFKA_BROKERS : DEFAULT_LLC_NUM_KAFKA_BROKERS;
   }
 
   protected int getKafkaPort() {
@@ -182,11 +159,7 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
   }
 
   protected int getNumKafkaPartitions() {
-    if (useLlc()) {
-      return DEFAULT_LLC_NUM_KAFKA_PARTITIONS;
-    } else {
-      return DEFAULT_HLC_NUM_KAFKA_PARTITIONS;
-    }
+    return DEFAULT_LLC_NUM_KAFKA_PARTITIONS;
   }
 
   protected String getKafkaTopic() {
@@ -273,7 +246,7 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
 
   protected QueryConfig getQueryConfig() {
     // Enable groovy for tables used in the tests
-    return new QueryConfig(null, false, null, null);
+    return new QueryConfig(null, false, null, null, null, null);
   }
 
   protected boolean getNullHandlingEnabled() {
@@ -293,14 +266,9 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
     InputStream inputStream =
         BaseClusterIntegrationTest.class.getClassLoader().getResourceAsStream(getSchemaFileName());
     Assert.assertNotNull(inputStream);
-    return Schema.fromInputStream(inputStream);
-  }
-
-  /**
-   * Returns the schema in the cluster.
-   */
-  protected Schema getSchema() {
-    return getSchema(getSchemaName());
+    Schema schema = Schema.fromInputStream(inputStream);
+    schema.setSchemaName(getTableName());
+    return schema;
   }
 
   protected Schema createSchema(File schemaFile)
@@ -321,7 +289,7 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
    * Creates a new OFFLINE table config.
    */
   protected TableConfig createOfflineTableConfig() {
-    return new TableConfigBuilder(TableType.OFFLINE).setTableName(getTableName()).setSchemaName(getSchemaName())
+    return new TableConfigBuilder(TableType.OFFLINE).setTableName(getTableName())
         .setTimeColumnName(getTimeColumnName()).setSortedColumn(getSortedColumn())
         .setInvertedIndexColumns(getInvertedIndexColumns()).setNoDictionaryColumns(getNoDictionaryColumns())
         .setRangeIndexColumns(getRangeIndexColumns()).setBloomFilterColumns(getBloomFilterColumns())
@@ -347,30 +315,13 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
     Map<String, String> streamConfigMap = new HashMap<>();
     String streamType = "kafka";
     streamConfigMap.put(StreamConfigProperties.STREAM_TYPE, streamType);
-    boolean useLlc = useLlc();
-    if (useLlc) {
-      // LLC
-      streamConfigMap.put(
-          StreamConfigProperties.constructStreamProperty(streamType, StreamConfigProperties.STREAM_CONSUMER_TYPES),
-          StreamConfig.ConsumerType.LOWLEVEL.toString());
+    streamConfigMap.put(KafkaStreamConfigProperties.constructStreamProperty(
+            KafkaStreamConfigProperties.LowLevelConsumer.KAFKA_BROKER_LIST),
+        "localhost:" + _kafkaStarters.get(0).getPort());
+    if (useKafkaTransaction()) {
       streamConfigMap.put(KafkaStreamConfigProperties.constructStreamProperty(
-              KafkaStreamConfigProperties.LowLevelConsumer.KAFKA_BROKER_LIST),
-          "localhost:" + _kafkaStarters.get(0).getPort());
-      if (useKafkaTransaction()) {
-        streamConfigMap.put(KafkaStreamConfigProperties.constructStreamProperty(
-                KafkaStreamConfigProperties.LowLevelConsumer.KAFKA_ISOLATION_LEVEL),
-            KafkaStreamConfigProperties.LowLevelConsumer.KAFKA_ISOLATION_LEVEL_READ_COMMITTED);
-      }
-    } else {
-      // HLC
-      streamConfigMap.put(
-          StreamConfigProperties.constructStreamProperty(streamType, StreamConfigProperties.STREAM_CONSUMER_TYPES),
-          StreamConfig.ConsumerType.HIGHLEVEL.toString());
-      streamConfigMap.put(KafkaStreamConfigProperties.constructStreamProperty(
-          KafkaStreamConfigProperties.HighLevelConsumer.KAFKA_HLC_ZK_CONNECTION_STRING), getKafkaZKAddress());
-      streamConfigMap.put(KafkaStreamConfigProperties.constructStreamProperty(
-              KafkaStreamConfigProperties.HighLevelConsumer.KAFKA_HLC_BOOTSTRAP_SERVER),
-          "localhost:" + _kafkaStarters.get(0).getPort());
+              KafkaStreamConfigProperties.LowLevelConsumer.KAFKA_ISOLATION_LEVEL),
+          KafkaStreamConfigProperties.LowLevelConsumer.KAFKA_ISOLATION_LEVEL_READ_COMMITTED);
     }
     streamConfigMap.put(StreamConfigProperties.constructStreamProperty(streamType,
         StreamConfigProperties.STREAM_CONSUMER_FACTORY_CLASS), getStreamConsumerFactoryClassName());
@@ -392,14 +343,14 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
    */
   protected TableConfig createRealtimeTableConfig(File sampleAvroFile) {
     AvroFileSchemaKafkaAvroMessageDecoder._avroFile = sampleAvroFile;
-    return new TableConfigBuilder(TableType.REALTIME).setTableName(getTableName()).setSchemaName(getSchemaName())
+    return new TableConfigBuilder(TableType.REALTIME).setTableName(getTableName())
         .setTimeColumnName(getTimeColumnName()).setSortedColumn(getSortedColumn())
         .setInvertedIndexColumns(getInvertedIndexColumns()).setNoDictionaryColumns(getNoDictionaryColumns())
         .setRangeIndexColumns(getRangeIndexColumns()).setBloomFilterColumns(getBloomFilterColumns())
         .setFieldConfigList(getFieldConfigs()).setNumReplicas(getNumReplicas()).setSegmentVersion(getSegmentVersion())
         .setLoadMode(getLoadMode()).setTaskConfig(getTaskConfig()).setBrokerTenant(getBrokerTenant())
         .setServerTenant(getServerTenant()).setIngestionConfig(getIngestionConfig()).setQueryConfig(getQueryConfig())
-        .setLLC(useLlc()).setStreamConfigs(getStreamConfigs()).setNullHandlingEnabled(getNullHandlingEnabled()).build();
+        .setStreamConfigs(getStreamConfigs()).setNullHandlingEnabled(getNullHandlingEnabled()).build();
   }
 
   /**
@@ -414,12 +365,12 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
     UpsertConfig upsertConfig = new UpsertConfig(UpsertConfig.Mode.FULL);
     upsertConfig.setDeleteRecordColumn(deleteColumn);
 
-    return new TableConfigBuilder(TableType.REALTIME).setTableName(getTableName()).setSchemaName(getSchemaName())
+    return new TableConfigBuilder(TableType.REALTIME).setTableName(getTableName())
         .setTimeColumnName(getTimeColumnName()).setFieldConfigList(getFieldConfigs()).setNumReplicas(getNumReplicas())
         .setSegmentVersion(getSegmentVersion()).setLoadMode(getLoadMode()).setTaskConfig(getTaskConfig())
         .setBrokerTenant(getBrokerTenant()).setServerTenant(getServerTenant()).setIngestionConfig(getIngestionConfig())
-        .setLLC(useLlc()).setStreamConfigs(getStreamConfigs()).setNullHandlingEnabled(getNullHandlingEnabled())
-        .setRoutingConfig(new RoutingConfig(null, null, RoutingConfig.STRICT_REPLICA_GROUP_INSTANCE_SELECTOR_TYPE))
+        .setStreamConfigs(getStreamConfigs()).setNullHandlingEnabled(getNullHandlingEnabled()).setRoutingConfig(
+            new RoutingConfig(null, null, RoutingConfig.STRICT_REPLICA_GROUP_INSTANCE_SELECTOR_TYPE, false))
         .setSegmentPartitionConfig(new SegmentPartitionConfig(columnPartitionConfigMap))
         .setReplicaGroupStrategyConfig(new ReplicaGroupStrategyConfig(primaryKeyColumn, 1))
         .setUpsertConfig(upsertConfig).build();
@@ -446,17 +397,15 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
   /**
    * Creates a new Upsert enabled table config.
    */
-  protected TableConfig createCSVUpsertTableConfig(String tableName, @Nullable String schemaName,
-      @Nullable String kafkaTopicName, int numPartitions, Map<String, String> streamDecoderProperties,
-      UpsertConfig upsertConfig, String primaryKeyColumn) {
-    if (schemaName == null) {
-      schemaName = getSchemaName();
-    }
+  protected TableConfig createCSVUpsertTableConfig(String tableName, @Nullable String kafkaTopicName,
+      int numPartitions, Map<String, String> streamDecoderProperties, UpsertConfig upsertConfig,
+      String primaryKeyColumn) {
     Map<String, ColumnPartitionConfig> columnPartitionConfigMap = new HashMap<>();
     columnPartitionConfigMap.put(primaryKeyColumn, new ColumnPartitionConfig("Murmur", numPartitions));
 
     if (upsertConfig == null) {
       upsertConfig = new UpsertConfig(UpsertConfig.Mode.FULL);
+      upsertConfig.setEnableSnapshot(true);
     }
     if (kafkaTopicName == null) {
       kafkaTopicName = getKafkaTopic();
@@ -468,13 +417,13 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
         kafkaTopicName);
     streamConfigsMap.putAll(streamDecoderProperties);
 
-    return new TableConfigBuilder(TableType.REALTIME).setTableName(tableName).setSchemaName(schemaName)
-        .setTimeColumnName(getTimeColumnName()).setFieldConfigList(getFieldConfigs()).setNumReplicas(getNumReplicas())
-        .setSegmentVersion(getSegmentVersion()).setLoadMode(getLoadMode()).setTaskConfig(getTaskConfig())
-        .setBrokerTenant(getBrokerTenant()).setServerTenant(getServerTenant()).setIngestionConfig(getIngestionConfig())
-        .setLLC(useLlc()).setStreamConfigs(streamConfigsMap)
+    return new TableConfigBuilder(TableType.REALTIME).setTableName(tableName).setTimeColumnName(getTimeColumnName())
+        .setFieldConfigList(getFieldConfigs()).setNumReplicas(getNumReplicas()).setSegmentVersion(getSegmentVersion())
+        .setLoadMode(getLoadMode()).setTaskConfig(getTaskConfig()).setBrokerTenant(getBrokerTenant())
+        .setServerTenant(getServerTenant()).setIngestionConfig(getIngestionConfig()).setStreamConfigs(streamConfigsMap)
         .setNullHandlingEnabled(UpsertConfig.Mode.PARTIAL.equals(upsertConfig.getMode()) || getNullHandlingEnabled())
-        .setRoutingConfig(new RoutingConfig(null, null, RoutingConfig.STRICT_REPLICA_GROUP_INSTANCE_SELECTOR_TYPE))
+        .setRoutingConfig(
+            new RoutingConfig(null, null, RoutingConfig.STRICT_REPLICA_GROUP_INSTANCE_SELECTOR_TYPE, false))
         .setSegmentPartitionConfig(new SegmentPartitionConfig(columnPartitionConfigMap))
         .setReplicaGroupStrategyConfig(new ReplicaGroupStrategyConfig(primaryKeyColumn, 1))
         .setUpsertConfig(upsertConfig).build();
@@ -488,12 +437,12 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
     Map<String, ColumnPartitionConfig> columnPartitionConfigMap = new HashMap<>();
     columnPartitionConfigMap.put(primaryKeyColumn, new ColumnPartitionConfig("Murmur", numPartitions));
 
-    return new TableConfigBuilder(TableType.REALTIME).setTableName(getTableName()).setSchemaName(getSchemaName())
+    return new TableConfigBuilder(TableType.REALTIME).setTableName(getTableName())
         .setTimeColumnName(getTimeColumnName()).setFieldConfigList(getFieldConfigs()).setNumReplicas(getNumReplicas())
         .setSegmentVersion(getSegmentVersion()).setLoadMode(getLoadMode()).setTaskConfig(getTaskConfig())
         .setBrokerTenant(getBrokerTenant()).setServerTenant(getServerTenant()).setIngestionConfig(getIngestionConfig())
-        .setLLC(useLlc()).setStreamConfigs(getStreamConfigs()).setNullHandlingEnabled(getNullHandlingEnabled())
-        .setRoutingConfig(new RoutingConfig(null, null, RoutingConfig.STRICT_REPLICA_GROUP_INSTANCE_SELECTOR_TYPE))
+        .setStreamConfigs(getStreamConfigs()).setNullHandlingEnabled(getNullHandlingEnabled()).setRoutingConfig(
+            new RoutingConfig(null, null, RoutingConfig.STRICT_REPLICA_GROUP_INSTANCE_SELECTOR_TYPE, false))
         .setSegmentPartitionConfig(new SegmentPartitionConfig(columnPartitionConfigMap))
         .setReplicaGroupStrategyConfig(new ReplicaGroupStrategyConfig(primaryKeyColumn, 1))
         .setDedupConfig(new DedupConfig(true, HashFunction.NONE)).build();
@@ -516,7 +465,7 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
     if (useMultiStageQueryEngine()) {
       if (_pinotConnectionV2 == null) {
         Properties properties = getPinotConnectionProperties();
-        properties.put("useMultiStageEngine", "true");
+        properties.put("useMultistageEngine", "true");
         _pinotConnectionV2 = ConnectionFactory.fromZookeeper(getZkUrl() + "/" + getHelixClusterName(),
             new JsonAsyncHttpPinotClientTransportFactory()
                 .withConnectionProperties(properties)
@@ -801,11 +750,7 @@ public abstract class BaseClusterIntegrationTest extends ClusterTest {
   }
 
   protected String getType(JsonNode jsonNode, int colIndex) {
-    return jsonNode.get("resultTable")
-        .get("dataSchema")
-        .get("columnDataTypes")
-        .get(colIndex)
-        .asText();
+    return jsonNode.get("resultTable").get("dataSchema").get("columnDataTypes").get(colIndex).asText();
   }
 
   protected <T> T getCellValue(JsonNode jsonNode, int colIndex, int rowIndex, Function<JsonNode, T> extract) {
