@@ -46,6 +46,7 @@ import org.apache.avro.io.DatumReader;
 import org.apache.avro.io.DecoderFactory;
 import org.apache.commons.collections.MapUtils;
 import org.apache.commons.io.FileUtils;
+import org.apache.commons.lang.math.RandomUtils;
 import org.apache.http.Header;
 import org.apache.http.HttpStatus;
 import org.apache.http.NameValuePair;
@@ -69,6 +70,7 @@ import org.apache.pinot.spi.data.readers.GenericRow;
 import org.apache.pinot.spi.data.readers.RecordExtractor;
 import org.apache.pinot.spi.env.PinotConfiguration;
 import org.apache.pinot.spi.stream.StreamMessageDecoder;
+import org.apache.pinot.spi.utils.CommonConstants;
 import org.apache.pinot.spi.utils.CommonConstants.Broker;
 import org.apache.pinot.spi.utils.CommonConstants.Helix;
 import org.apache.pinot.spi.utils.CommonConstants.Minion;
@@ -103,6 +105,25 @@ public abstract class ClusterTest extends ControllerTest {
 
   private boolean _useMultiStageQueryEngine = false;
 
+  private String _baseInstanceDataDir =
+      System.getProperty("java.io.tmpdir") + File.separator + System.currentTimeMillis();
+
+  private int _serverGrpcPort;
+  private int _serverAdminApiPort;
+  private int _serverNettyPort;
+
+  protected int getServerGrpcPort() {
+    return _serverGrpcPort;
+  }
+
+  protected int getServerAdminApiPort() {
+    return _serverAdminApiPort;
+  }
+
+  protected int getServerNettyPort() {
+    return _serverNettyPort;
+  }
+
   protected String getBrokerBaseApiUrl() {
     return _brokerBaseApiUrl;
   }
@@ -136,7 +157,8 @@ public abstract class ClusterTest extends ControllerTest {
     brokerConf.setProperty(Helix.CONFIG_OF_CLUSTER_NAME, getHelixClusterName());
     brokerConf.setProperty(Helix.CONFIG_OF_ZOOKEEPR_SERVER, getZkUrl());
     brokerConf.setProperty(Broker.CONFIG_OF_BROKER_TIMEOUT_MS, 60 * 1000L);
-    brokerConf.setProperty(Helix.KEY_OF_BROKER_QUERY_PORT, NetUtils.findOpenPort(DEFAULT_BROKER_PORT + brokerId));
+    brokerConf.setProperty(Helix.KEY_OF_BROKER_QUERY_PORT,
+        NetUtils.findOpenPort(DEFAULT_BROKER_PORT + brokerId + RandomUtils.nextInt(10000)));
     brokerConf.setProperty(Broker.CONFIG_OF_DELAY_SHUTDOWN_TIME_MS, 0);
     overrideBrokerConf(brokerConf);
     return brokerConf;
@@ -184,10 +206,15 @@ public abstract class ClusterTest extends ControllerTest {
     brokerStarter.init(brokerConf);
     brokerStarter.start();
     _brokerStarters.add(brokerStarter);
-
-    // TLS configs require hard-coding
-    _brokerPorts.add(DEFAULT_BROKER_PORT);
-    _brokerBaseApiUrl = "https://localhost:" + DEFAULT_BROKER_PORT;
+    _brokerPorts.add(brokerStarter.getPort());
+    if (brokerConf.containsKey("pinot.broker.client.access.protocols.internal.port")) {
+      _brokerPorts.add(Integer.parseInt(brokerConf.getProperty("pinot.broker.client.access.protocols.internal.port")));
+    }
+    if (brokerConf.containsKey("pinot.broker.client.access.protocols.external.port")) {
+      _brokerPorts.add(Integer.parseInt(brokerConf.getProperty("pinot.broker.client.access.protocols.external.port")));
+    }
+    // TLS port is the last one in the list
+    _brokerBaseApiUrl = "https://localhost:" + _brokerPorts.get(_brokerPorts.size() - 1);
   }
 
   protected int getRandomBrokerPort() {
@@ -210,12 +237,19 @@ public abstract class ClusterTest extends ControllerTest {
     PinotConfiguration serverConf = getDefaultServerConfiguration();
     serverConf.setProperty(Helix.CONFIG_OF_CLUSTER_NAME, getHelixClusterName());
     serverConf.setProperty(Helix.CONFIG_OF_ZOOKEEPR_SERVER, getZkUrl());
-    serverConf.setProperty(Server.CONFIG_OF_INSTANCE_DATA_DIR, Server.DEFAULT_INSTANCE_DATA_DIR + "-" + serverId);
+    serverConf.setProperty(Server.CONFIG_OF_INSTANCE_DATA_DIR,
+        _baseInstanceDataDir + File.separator + "PinotServer" + File.separator + "dataDir-" + serverId);
     serverConf.setProperty(Server.CONFIG_OF_INSTANCE_SEGMENT_TAR_DIR,
-        Server.DEFAULT_INSTANCE_SEGMENT_TAR_DIR + "-" + serverId);
-    serverConf.setProperty(Server.CONFIG_OF_ADMIN_API_PORT, Server.DEFAULT_ADMIN_API_PORT - serverId);
-    serverConf.setProperty(Helix.KEY_OF_SERVER_NETTY_PORT, Helix.DEFAULT_SERVER_NETTY_PORT + serverId);
-    serverConf.setProperty(Server.CONFIG_OF_GRPC_PORT, Server.DEFAULT_GRPC_PORT + serverId);
+        _baseInstanceDataDir + File.separator + "PinotServer" + File.separator + "segmentTar-" + serverId);
+
+    _serverAdminApiPort = NetUtils.findOpenPort(Server.DEFAULT_ADMIN_API_PORT + new Random().nextInt(10000) + serverId);
+    serverConf.setProperty(Server.CONFIG_OF_ADMIN_API_PORT, _serverAdminApiPort);
+    _serverNettyPort = NetUtils.findOpenPort(Helix.DEFAULT_SERVER_NETTY_PORT + new Random().nextInt(10000) + serverId);
+    serverConf.setProperty(Helix.KEY_OF_SERVER_NETTY_PORT, _serverNettyPort);
+    _serverGrpcPort =
+        NetUtils.findOpenPort(Server.DEFAULT_GRPC_PORT + new Random().nextInt(10000) + serverId);
+    serverConf.setProperty(Server.CONFIG_OF_GRPC_PORT, _serverGrpcPort);
+
     // Thread time measurement is disabled by default, enable it in integration tests.
     // TODO: this can be removed when we eventually enable thread time measurement by default.
     serverConf.setProperty(Server.CONFIG_OF_ENABLE_THREAD_CPU_TIME_MEASUREMENT, true);
@@ -230,7 +264,7 @@ public abstract class ClusterTest extends ControllerTest {
 
   protected void startServers(int numServers)
       throws Exception {
-    FileUtils.deleteQuietly(new File(Server.DEFAULT_INSTANCE_BASE_DIR));
+    FileUtils.deleteQuietly(new File(_baseInstanceDataDir + File.separator + "PinotServer"));
     _serverStarters = new ArrayList<>(numServers);
     for (int i = 0; i < numServers; i++) {
       _serverStarters.add(startOneServer(i));
@@ -245,9 +279,17 @@ public abstract class ClusterTest extends ControllerTest {
     return serverStarter;
   }
 
+  protected BaseServerStarter startOneServer(PinotConfiguration serverConfig)
+      throws Exception {
+    HelixServerStarter serverStarter = new HelixServerStarter();
+    serverStarter.init(serverConfig);
+    serverStarter.start();
+    return serverStarter;
+  }
+
   protected void startServerHttps()
       throws Exception {
-    FileUtils.deleteQuietly(new File(Server.DEFAULT_INSTANCE_BASE_DIR));
+    FileUtils.deleteQuietly(new File(_baseInstanceDataDir + File.separator + "PinotServer"));
     _serverStarters = new ArrayList<>();
 
     PinotConfiguration serverConf = getDefaultServerConfiguration();
@@ -269,10 +311,12 @@ public abstract class ClusterTest extends ControllerTest {
   //       to manage the instance level configs
   protected void startMinion()
       throws Exception {
-    FileUtils.deleteQuietly(new File(Minion.DEFAULT_INSTANCE_BASE_DIR));
+    FileUtils.deleteQuietly(new File(_baseInstanceDataDir + File.separator + "PinotMinion"));
     PinotConfiguration minionConf = getDefaultMinionConfiguration();
     minionConf.setProperty(Helix.CONFIG_OF_CLUSTER_NAME, getHelixClusterName());
     minionConf.setProperty(Helix.CONFIG_OF_ZOOKEEPR_SERVER, getZkUrl());
+    minionConf.setProperty(CommonConstants.Helix.KEY_OF_MINION_PORT,
+        NetUtils.findOpenPort(CommonConstants.Minion.DEFAULT_HELIX_PORT));
     _minionStarter = new MinionStarter();
     _minionStarter.init(minionConf);
     _minionStarter.start();
@@ -284,6 +328,8 @@ public abstract class ClusterTest extends ControllerTest {
       brokerStarter.stop();
     }
     _brokerStarters = null;
+    _brokerPorts = null;
+    _brokerBaseApiUrl = null;
   }
 
   protected void stopServer() {
@@ -291,8 +337,11 @@ public abstract class ClusterTest extends ControllerTest {
     for (BaseServerStarter serverStarter : _serverStarters) {
       serverStarter.stop();
     }
-    FileUtils.deleteQuietly(new File(Server.DEFAULT_INSTANCE_BASE_DIR));
+    FileUtils.deleteQuietly(new File(_baseInstanceDataDir + File.separator + "PinotServer"));
     _serverStarters = null;
+    _serverGrpcPort = 0;
+    _serverAdminApiPort = 0;
+    _serverNettyPort = 0;
   }
 
   protected void stopMinion() {
@@ -305,16 +354,21 @@ public abstract class ClusterTest extends ControllerTest {
   protected void restartServers()
       throws Exception {
     assertNotNull(_serverStarters, "Servers are not started");
-    for (BaseServerStarter serverStarter : _serverStarters) {
-      serverStarter.stop();
-    }
+    List<BaseServerStarter> oldServers = new ArrayList<>(_serverStarters);
     int numServers = _serverStarters.size();
     _serverStarters.clear();
     for (int i = 0; i < numServers; i++) {
-      _serverStarters.add(startOneServer(i));
+      _serverStarters.add(restartServer(oldServers.get(i)));
     }
   }
 
+  protected BaseServerStarter restartServer(BaseServerStarter serverStarter)
+      throws Exception {
+    PinotConfiguration serverConfig = serverStarter.getConfig();
+    serverStarter.stop();
+    return startOneServer(serverConfig);
+  }
+
   /**
    * Upload all segments inside the given directory to the cluster.
    */
@@ -323,6 +377,14 @@ public abstract class ClusterTest extends ControllerTest {
     uploadSegments(tableName, TableType.OFFLINE, tarDir);
   }
 
+  /**
+   * Upload all segments inside the given directory to the cluster.
+   */
+  protected void uploadSegments(String tableName, List<File> tarDirs)
+      throws Exception {
+    uploadSegments(tableName, TableType.OFFLINE, tarDirs);
+  }
+
   /**
    * Upload all segments inside the given directory to the cluster.
    */
@@ -545,7 +607,7 @@ public abstract class ClusterTest extends ControllerTest {
 
   @DataProvider(name = "systemColumns")
   public Object[][] systemColumns() {
-    return new Object[][] {
+    return new Object[][]{
         {"$docId"},
         {"$hostName"},
         {"$segmentName"}
