@@ -86,11 +86,6 @@ public class DedupIntegrationTest extends BaseClusterIntegrationTestSet {
     return "dedupIngestionTestSchema.schema";
   }
 
-  @Override
-  protected String getSchemaName() {
-    return "dedupSchema";
-  }
-
   @Override
   protected String getAvroTarFileName() {
     return "dedupIngestionTestData.tar.gz";
