@@ -157,7 +157,7 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
       LOGGER.info("Start generating task configs for table: {} for task: {}", tableNameWithType, taskType);
 
       // Get all segment metadata
-      List<SegmentZKMetadata> allSegments = _clusterInfoAccessor.getSegmentsZKMetadata(tableNameWithType);
+      List<SegmentZKMetadata> allSegments = getSegmentsZKMetadataForTable(tableNameWithType);
       // Filter segments based on status
       List<SegmentZKMetadata> preSelectedSegmentsBasedOnStatus
           = filterSegmentsBasedOnStatus(tableConfig.getTableType(), allSegments);
@@ -859,8 +859,8 @@ public class MergeRollupTaskGenerator extends BaseTaskGenerator {
       if (v == null) {
         LOGGER.info(
             "Creating the gauge metric for tracking the merge/roll-up number buckets to process for table: {} "
-                + "and mergeLevel: {}.(bufferTimeMs={}, bucketTimeMs={}, numTimeBucketsToProcess={})"
-                + tableNameWithType, mergeLevel, bufferTimeMs, bucketTimeMs, finalCount);
+                + "and mergeLevel: {}.(bufferTimeMs={}, bucketTimeMs={}, numTimeBucketsToProcess={})",
+                tableNameWithType, mergeLevel, bufferTimeMs, bucketTimeMs, finalCount);
         controllerMetrics.setOrUpdateGauge(getMetricNameForNumBucketsToProcess(tableNameWithType, mergeLevel),
             () -> _tableNumberBucketsToProcess.get(tableNameWithType).getOrDefault(mergeLevel, finalCount));
       }
