@@ -67,7 +67,7 @@ public class PurgeTaskGenerator extends BaseTaskGenerator {
         continue;
       }
       taskConfigs = tableTaskConfig.getConfigsForTaskType(MinionConstants.PurgeTask.TASK_TYPE);
-      Preconditions.checkNotNull(taskConfigs, "Task config shouldn't be null for Table: {}", tableName);
+      Preconditions.checkNotNull(taskConfigs, "Task config shouldn't be null for Table: %s", tableName);
 
       String deltaTimePeriod =
           taskConfigs.getOrDefault(MinionConstants.PurgeTask.LAST_PURGE_TIME_THREESOLD_PERIOD,
@@ -90,7 +90,7 @@ public class PurgeTaskGenerator extends BaseTaskGenerator {
       }
       List<SegmentZKMetadata> segmentsZKMetadata = new ArrayList<>();
       if (tableConfig.getTableType() == TableType.REALTIME) {
-        List<SegmentZKMetadata> segmentsZKMetadataAll = _clusterInfoAccessor.getSegmentsZKMetadata(tableName);
+        List<SegmentZKMetadata> segmentsZKMetadataAll = getSegmentsZKMetadataForTable(tableName);
         for (SegmentZKMetadata segmentZKMetadata : segmentsZKMetadataAll) {
           CommonConstants.Segment.Realtime.Status status = segmentZKMetadata.getStatus();
           if (status.isCompleted()) {
@@ -98,7 +98,7 @@ public class PurgeTaskGenerator extends BaseTaskGenerator {
           }
         }
       } else {
-        segmentsZKMetadata = _clusterInfoAccessor.getSegmentsZKMetadata(tableName);
+        segmentsZKMetadata = getSegmentsZKMetadataForTable(tableName);
       }
 
       List<SegmentZKMetadata> purgedSegmentsZKMetadata = new ArrayList<>();
