@@ -19,9 +19,11 @@
 package org.apache.pinot.plugin.stream.pulsar;
 
 import com.google.common.base.Preconditions;
+import com.google.common.collect.Sets;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.Set;
 import java.util.UUID;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
@@ -31,6 +33,7 @@ import org.apache.pinot.spi.stream.PartitionGroupMetadata;
 import org.apache.pinot.spi.stream.StreamConfig;
 import org.apache.pinot.spi.stream.StreamMetadataProvider;
 import org.apache.pinot.spi.stream.StreamPartitionMsgOffset;
+import org.apache.pinot.spi.stream.TransientConsumerException;
 import org.apache.pulsar.client.api.Consumer;
 import org.apache.pulsar.client.api.Message;
 import org.apache.pulsar.client.api.MessageId;
@@ -64,12 +67,24 @@ public class PulsarStreamMetadataProvider extends PulsarPartitionLevelConnection
   @Override
   public int fetchPartitionCount(long timeoutMillis) {
     try {
-      return _pulsarClient.getPartitionsForTopic(_topic).get().size();
+      return _pulsarClient.getPartitionsForTopic(_topic).get(timeoutMillis, TimeUnit.MILLISECONDS).size();
+    } catch (TimeoutException e) {
+      throw new TransientConsumerException(e);
     } catch (Exception e) {
-      throw new RuntimeException("Cannot fetch partitions for topic: " + _topic, e);
+      throw new RuntimeException("Failed to fetch partitions for topic: " + _topic, e);
     }
   }
 
+  @Override
+  public Set<Integer> fetchPartitionIds(long timeoutMillis) {
+    int partitionCount = fetchPartitionCount(timeoutMillis);
+    Set<Integer> partitionIds = Sets.newHashSetWithExpectedSize(partitionCount);
+    for (int i = 0; i < partitionCount; i++) {
+      partitionIds.add(i);
+    }
+    return partitionIds;
+  }
+
   /**
    * Fetch the messageId and use it as offset.
    * If offset criteria is smallest, the message id of earliest record in the partition is returned.
@@ -84,17 +99,15 @@ public class PulsarStreamMetadataProvider extends PulsarPartitionLevelConnection
   public StreamPartitionMsgOffset fetchStreamPartitionOffset(OffsetCriteria offsetCriteria, long timeoutMillis) {
     Preconditions.checkNotNull(offsetCriteria);
     String subscription = "Pinot_" + UUID.randomUUID();
-    MessageId offset = null;
-    try (Consumer consumer =
-        _pulsarClient.newConsumer().topic(_topic)
-            .subscriptionInitialPosition(PulsarUtils.offsetCriteriaToSubscription(offsetCriteria))
-            .subscriptionMode(SubscriptionMode.NonDurable)  // automatically deletes subscription on consumer close
-            .subscriptionName(subscription).subscribe()) {
-
+    MessageId offset;
+    try (Consumer consumer = _pulsarClient.newConsumer().topic(_topic)
+        .subscriptionInitialPosition(PulsarUtils.offsetCriteriaToSubscription(offsetCriteria))
+        .subscriptionMode(SubscriptionMode.NonDurable)  // automatically deletes subscription on consumer close
+        .subscriptionName(subscription).subscribe()) {
       if (offsetCriteria.isLargest()) {
         offset = consumer.getLastMessageId();
       } else if (offsetCriteria.isSmallest()) {
-        offset = consumer.receive().getMessageId();
+        offset = consumer.receive((int) timeoutMillis, TimeUnit.MILLISECONDS).getMessageId();
       } else {
         throw new IllegalArgumentException("Unknown initial offset value " + offsetCriteria);
       }
@@ -131,8 +144,7 @@ public class PulsarStreamMetadataProvider extends PulsarPartitionLevelConnection
         for (int p = newPartitionStartIndex; p < partitionedTopicNameList.size(); p++) {
           try (Consumer consumer = _pulsarClient.newConsumer().topic(partitionedTopicNameList.get(p))
               .subscriptionInitialPosition(_config.getInitialSubscriberPosition())
-              .subscriptionMode(SubscriptionMode.NonDurable)
-              .subscriptionName(subscription).subscribe()) {
+              .subscriptionMode(SubscriptionMode.NonDurable).subscriptionName(subscription).subscribe()) {
 
             Message message = consumer.receive(timeoutMillis, TimeUnit.MILLISECONDS);
             if (message != null) {
@@ -149,8 +161,9 @@ public class PulsarStreamMetadataProvider extends PulsarPartitionLevelConnection
                   new PartitionGroupMetadata(p, new MessageIdStreamOffset(lastMessageId)));
             }
           } catch (PulsarClientException pce) {
-            LOGGER.warn("Error encountered while calculating partition group metadata for topic "
-                + _config.getPulsarTopicName() + " partition " + partitionedTopicNameList.get(p), pce);
+            LOGGER.warn(
+                "Error encountered while calculating partition group metadata for topic " + _config.getPulsarTopicName()
+                    + " partition " + partitionedTopicNameList.get(p), pce);
           }
         }
       }
