@@ -49,15 +49,10 @@ public class PinotQueryRuleSets {
           // push project through set operation
           CoreRules.PROJECT_SET_OP_TRANSPOSE,
 
-          // aggregation and projection rules
-          CoreRules.AGGREGATE_PROJECT_MERGE, CoreRules.AGGREGATE_PROJECT_PULL_UP_CONSTANTS,
-          // push a projection past a filter or vice versa
-          CoreRules.PROJECT_FILTER_TRANSPOSE, CoreRules.FILTER_PROJECT_TRANSPOSE,
-          // push a projection to the children of a join
-          // push all expressions to handle the time indicator correctly
+          // push a filter past a project
+          CoreRules.FILTER_PROJECT_TRANSPOSE,
+          // push parts of the join condition to its inputs
           CoreRules.JOIN_CONDITION_PUSH,
-          // merge projections
-          CoreRules.PROJECT_MERGE,
           // remove identity project
           CoreRules.PROJECT_REMOVE,
 
@@ -66,14 +61,19 @@ public class PinotQueryRuleSets {
           // push project through WINDOW
           CoreRules.PROJECT_WINDOW_TRANSPOSE,
 
-          // TODO: Revisit and see if they can be replaced with CoreRules.PROJECT_REDUCE_EXPRESSIONS and
-          //       CoreRules.FILTER_REDUCE_EXPRESSIONS
+          // literal rules
+          // TODO: Revisit and see if they can be replaced with
+          //     CoreRules.PROJECT_REDUCE_EXPRESSIONS and
+          //     CoreRules.FILTER_REDUCE_EXPRESSIONS
           PinotEvaluateLiteralRule.Project.INSTANCE, PinotEvaluateLiteralRule.Filter.INSTANCE,
 
+          // sort join rules
           // TODO: evaluate the SORT_JOIN_TRANSPOSE and SORT_JOIN_COPY rules
 
           // join rules
           CoreRules.JOIN_PUSH_EXPRESSIONS,
+
+          // join and semi-join rules
           CoreRules.PROJECT_TO_SEMI_JOIN,
           PinotAggregateToSemiJoinRule.INSTANCE,
 
@@ -102,7 +102,9 @@ public class PinotQueryRuleSets {
 
   // The pruner rules run top-down to ensure Calcite restarts from root node after applying a transformation.
   public static final Collection<RelOptRule> PRUNE_RULES = ImmutableList.of(
+      CoreRules.AGGREGATE_PROJECT_MERGE,
       CoreRules.PROJECT_MERGE,
+      CoreRules.FILTER_MERGE,
       CoreRules.AGGREGATE_REMOVE,
       CoreRules.SORT_REMOVE,
       PruneEmptyRules.AGGREGATE_INSTANCE, PruneEmptyRules.FILTER_INSTANCE, PruneEmptyRules.JOIN_LEFT_INSTANCE,
@@ -110,12 +112,11 @@ public class PinotQueryRuleSets {
       PruneEmptyRules.UNION_INSTANCE
   );
 
-  // Pinot specific rules that should be run BEFORE all other rules
-  public static final Collection<RelOptRule> PINOT_PRE_RULES = ImmutableList.of(
+  // Pinot specific rules to run using a single RuleCollection since we attach aggregate info after optimizer.
+  public static final Collection<RelOptRule> PINOT_AGG_PROCESS_RULES = ImmutableList.of(
       PinotAggregateLiteralAttachmentRule.INSTANCE
   );
 
-
   // Pinot specific rules that should be run AFTER all other rules
   public static final Collection<RelOptRule> PINOT_POST_RULES = ImmutableList.of(
       // Evaluate the Literal filter nodes
@@ -135,6 +136,9 @@ public class PinotQueryRuleSets {
       PinotSetOpExchangeNodeInsertRule.INSTANCE,
 
       // apply dynamic broadcast rule after exchange is inserted/
-      PinotJoinToDynamicBroadcastRule.INSTANCE
+      PinotJoinToDynamicBroadcastRule.INSTANCE,
+
+      // remove exchanges when there's duplicates
+      PinotExchangeEliminationRule.INSTANCE
   );
 }
