@@ -19,7 +19,6 @@
 package org.apache.pinot.query;
 
 import com.google.common.collect.ImmutableList;
-import com.google.common.collect.ImmutableSet;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.HashMap;
@@ -29,10 +28,9 @@ import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.stream.Collectors;
 import org.apache.calcite.rel.RelDistribution;
-import org.apache.pinot.query.planner.DispatchablePlanFragment;
-import org.apache.pinot.query.planner.DispatchableSubPlan;
-import org.apache.pinot.query.planner.PhysicalExplainPlanVisitor;
 import org.apache.pinot.query.planner.PlannerUtils;
+import org.apache.pinot.query.planner.physical.DispatchablePlanFragment;
+import org.apache.pinot.query.planner.physical.DispatchableSubPlan;
 import org.apache.pinot.query.planner.plannode.AbstractPlanNode;
 import org.apache.pinot.query.planner.plannode.AggregateNode;
 import org.apache.pinot.query.planner.plannode.FilterNode;
@@ -40,59 +38,43 @@ import org.apache.pinot.query.planner.plannode.JoinNode;
 import org.apache.pinot.query.planner.plannode.MailboxReceiveNode;
 import org.apache.pinot.query.planner.plannode.PlanNode;
 import org.apache.pinot.query.planner.plannode.ProjectNode;
-import org.testng.Assert;
+import org.apache.pinot.query.routing.QueryServerInstance;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
+import static org.testng.Assert.*;
+
 
 public class QueryCompilationTest extends QueryEnvironmentTestBase {
 
   @Test(dataProvider = "testQueryLogicalPlanDataProvider")
-  public void testQueryPlanExplainLogical(String query, String digest)
-      throws Exception {
-    testQueryPlanExplain(query, digest);
-  }
-
-  @Test(dataProvider = "testQueryPhysicalPlanDataProvider")
-  public void testQueryPlanExplainPhysical(String query, String digest)
-      throws Exception {
+  public void testQueryPlanExplainLogical(String query, String digest) {
     testQueryPlanExplain(query, digest);
   }
 
   private void testQueryPlanExplain(String query, String digest) {
-    try {
-      long requestId = RANDOM_REQUEST_ID_GEN.nextLong();
-      String explainedPlan = _queryEnvironment.explainQuery(query, requestId);
-      Assert.assertEquals(explainedPlan, digest);
-    } catch (RuntimeException e) {
-      Assert.fail("failed to explain query: " + query, e);
-    }
+    long requestId = RANDOM_REQUEST_ID_GEN.nextLong();
+    String explainedPlan = _queryEnvironment.explainQuery(query, requestId);
+    assertEquals(explainedPlan, digest);
   }
 
   @Test(dataProvider = "testQueryDataProvider")
-  public void testQueryPlanWithoutException(String query)
-      throws Exception {
-    try {
-      DispatchableSubPlan dispatchableSubPlan = _queryEnvironment.planQuery(query);
-      Assert.assertNotNull(dispatchableSubPlan);
-    } catch (RuntimeException e) {
-      Assert.fail("failed to plan query: " + query, e);
-    }
+  public void testQueryPlanWithoutException(String query) {
+    DispatchableSubPlan dispatchableSubPlan = _queryEnvironment.planQuery(query);
+    assertNotNull(dispatchableSubPlan);
   }
 
   @Test(dataProvider = "testQueryExceptionDataProvider")
   public void testQueryWithException(String query, String exceptionSnippet) {
     try {
       _queryEnvironment.planQuery(query);
-      Assert.fail("query plan should throw exception");
+      fail("query plan should throw exception");
     } catch (RuntimeException e) {
-      Assert.assertTrue(e.getCause().getMessage().contains(exceptionSnippet));
+      assertTrue(e.getCause().getMessage().contains(exceptionSnippet));
     }
   }
 
-  private static void assertGroupBySingletonAfterJoin(DispatchableSubPlan dispatchableSubPlan, boolean shouldRewrite)
-      throws Exception {
-
+  private static void assertGroupBySingletonAfterJoin(DispatchableSubPlan dispatchableSubPlan, boolean shouldRewrite) {
     for (int stageId = 0; stageId < dispatchableSubPlan.getQueryStageList().size(); stageId++) {
       if (dispatchableSubPlan.getTableNames().size() == 0 && !PlannerUtils.isRootPlanFragment(stageId)) {
         PlanNode node = dispatchableSubPlan.getQueryStageList().get(stageId).getPlanFragment().getFragmentRoot();
@@ -101,17 +83,17 @@ public class QueryCompilationTest extends QueryEnvironmentTestBase {
             // JOIN is exchanged with hash distribution (data shuffle)
             MailboxReceiveNode left = (MailboxReceiveNode) node.getInputs().get(0);
             MailboxReceiveNode right = (MailboxReceiveNode) node.getInputs().get(1);
-            Assert.assertEquals(left.getDistributionType(), RelDistribution.Type.HASH_DISTRIBUTED);
-            Assert.assertEquals(right.getDistributionType(), RelDistribution.Type.HASH_DISTRIBUTED);
+            assertEquals(left.getDistributionType(), RelDistribution.Type.HASH_DISTRIBUTED);
+            assertEquals(right.getDistributionType(), RelDistribution.Type.HASH_DISTRIBUTED);
             break;
           }
           if (node instanceof AggregateNode && node.getInputs().get(0) instanceof MailboxReceiveNode) {
             // AGG is exchanged with singleton since it has already been distributed by JOIN.
             MailboxReceiveNode input = (MailboxReceiveNode) node.getInputs().get(0);
             if (shouldRewrite) {
-              Assert.assertEquals(input.getDistributionType(), RelDistribution.Type.SINGLETON);
+              assertEquals(input.getDistributionType(), RelDistribution.Type.SINGLETON);
             } else {
-              Assert.assertNotEquals(input.getDistributionType(), RelDistribution.Type.SINGLETON);
+              assertNotEquals(input.getDistributionType(), RelDistribution.Type.SINGLETON);
             }
             break;
           }
@@ -122,34 +104,40 @@ public class QueryCompilationTest extends QueryEnvironmentTestBase {
   }
 
   @Test
-  public void testQueryAndAssertStageContentForJoin()
-      throws Exception {
+  public void testQueryAndAssertStageContentForJoin() {
     String query = "SELECT * FROM a JOIN b ON a.col1 = b.col2";
     DispatchableSubPlan dispatchableSubPlan = _queryEnvironment.planQuery(query);
-    Assert.assertEquals(dispatchableSubPlan.getQueryStageList().size(), 4);
-
-    for (int stageId = 0; stageId < dispatchableSubPlan.getQueryStageList().size(); stageId++) {
-      DispatchablePlanFragment dispatchablePlanFragment = dispatchableSubPlan.getQueryStageList().get(stageId);
-      String tableName = dispatchablePlanFragment.getTableName();
+    List<DispatchablePlanFragment> stagePlans = dispatchableSubPlan.getQueryStageList();
+    int numStages = stagePlans.size();
+    assertEquals(numStages, 4);
+    for (int stageId = 0; stageId < numStages; stageId++) {
+      DispatchablePlanFragment stagePlan = stagePlans.get(stageId);
+      Map<QueryServerInstance, List<Integer>> serverToWorkerIdsMap = stagePlan.getServerInstanceToWorkerIdMap();
+      int numServers = serverToWorkerIdsMap.size();
+      String tableName = stagePlan.getTableName();
       if (tableName != null) {
         // table scan stages; for tableA it should have 2 hosts, for tableB it should have only 1
-        Assert.assertEquals(dispatchablePlanFragment.getServerInstanceToWorkerIdMap().entrySet().stream()
-                .map(PhysicalExplainPlanVisitor::stringifyQueryServerInstanceToWorkerIdsEntry)
-                .collect(Collectors.toSet()),
-            tableName.equals("a") ? ImmutableList.of("localhost@{1,1}|[1]", "localhost@{2,2}|[0]")
-                : ImmutableList.of("localhost@{1,1}|[0]"));
+        if (tableName.equals("a")) {
+          assertEquals(numServers, 2);
+          for (QueryServerInstance server : serverToWorkerIdsMap.keySet()) {
+            int port = server.getQueryMailboxPort();
+            assertTrue(port == 1 || port == 2);
+          }
+        } else {
+          assertEquals(numServers, 1);
+          assertEquals(serverToWorkerIdsMap.keySet().iterator().next().getQueryMailboxPort(), 1);
+        }
       } else if (!PlannerUtils.isRootPlanFragment(stageId)) {
         // join stage should have both servers used.
-        Assert.assertEquals(dispatchablePlanFragment.getServerInstanceToWorkerIdMap().entrySet().stream()
-                .map(PhysicalExplainPlanVisitor::stringifyQueryServerInstanceToWorkerIdsEntry)
-                .collect(Collectors.toSet()),
-            ImmutableSet.of("localhost@{1,1}|[1]", "localhost@{2,2}|[0]"));
+        assertEquals(numServers, 2);
+        for (QueryServerInstance server : serverToWorkerIdsMap.keySet()) {
+          int port = server.getQueryMailboxPort();
+          assertTrue(port == 1 || port == 2);
+        }
       } else {
         // reduce stage should have the reducer instance.
-        Assert.assertEquals(dispatchablePlanFragment.getServerInstanceToWorkerIdMap().entrySet().stream()
-                .map(PhysicalExplainPlanVisitor::stringifyQueryServerInstanceToWorkerIdsEntry)
-                .collect(Collectors.toSet()),
-            ImmutableSet.of("localhost@{3,3}|[0]"));
+        assertEquals(numServers, 1);
+        assertEquals(serverToWorkerIdsMap.keySet().iterator().next().getQueryMailboxPort(), 3);
       }
     }
   }
@@ -173,24 +161,27 @@ public class QueryCompilationTest extends QueryEnvironmentTestBase {
   public void testQueryRoutingManagerCompilation() {
     String query = "SELECT * FROM d_OFFLINE";
     DispatchableSubPlan dispatchableSubPlan = _queryEnvironment.planQuery(query);
-    List<DispatchablePlanFragment> tableScanMetadataList = dispatchableSubPlan.getQueryStageList().stream()
-        .filter(stageMetadata -> stageMetadata.getTableName() != null).collect(Collectors.toList());
-    Assert.assertEquals(tableScanMetadataList.size(), 1);
-    Assert.assertEquals(tableScanMetadataList.get(0).getServerInstanceToWorkerIdMap().size(), 2);
+    List<DispatchablePlanFragment> tableScanMetadataList =
+        dispatchableSubPlan.getQueryStageList().stream().filter(stageMetadata -> stageMetadata.getTableName() != null)
+            .collect(Collectors.toList());
+    assertEquals(tableScanMetadataList.size(), 1);
+    assertEquals(tableScanMetadataList.get(0).getServerInstanceToWorkerIdMap().size(), 2);
 
     query = "SELECT * FROM d_REALTIME";
     dispatchableSubPlan = _queryEnvironment.planQuery(query);
-    tableScanMetadataList = dispatchableSubPlan.getQueryStageList().stream()
-        .filter(stageMetadata -> stageMetadata.getTableName() != null).collect(Collectors.toList());
-    Assert.assertEquals(tableScanMetadataList.size(), 1);
-    Assert.assertEquals(tableScanMetadataList.get(0).getServerInstanceToWorkerIdMap().size(), 1);
+    tableScanMetadataList =
+        dispatchableSubPlan.getQueryStageList().stream().filter(stageMetadata -> stageMetadata.getTableName() != null)
+            .collect(Collectors.toList());
+    assertEquals(tableScanMetadataList.size(), 1);
+    assertEquals(tableScanMetadataList.get(0).getServerInstanceToWorkerIdMap().size(), 1);
 
     query = "SELECT * FROM d";
     dispatchableSubPlan = _queryEnvironment.planQuery(query);
-    tableScanMetadataList = dispatchableSubPlan.getQueryStageList().stream()
-        .filter(stageMetadata -> stageMetadata.getTableName() != null).collect(Collectors.toList());
-    Assert.assertEquals(tableScanMetadataList.size(), 1);
-    Assert.assertEquals(tableScanMetadataList.get(0).getServerInstanceToWorkerIdMap().size(), 2);
+    tableScanMetadataList =
+        dispatchableSubPlan.getQueryStageList().stream().filter(stageMetadata -> stageMetadata.getTableName() != null)
+            .collect(Collectors.toList());
+    assertEquals(tableScanMetadataList.size(), 1);
+    assertEquals(tableScanMetadataList.get(0).getServerInstanceToWorkerIdMap().size(), 2);
   }
 
   // Test that plan query can be run as multi-thread.
@@ -238,34 +229,36 @@ public class QueryCompilationTest extends QueryEnvironmentTestBase {
     }
     for (ArrayList<DispatchableSubPlan> plans : queryPlans.values()) {
       for (DispatchableSubPlan plan : plans) {
-        Assert.assertTrue(plan.equals(plans.get(0)));
+        assertTrue(plan.equals(plans.get(0)));
       }
     }
   }
 
   @Test
-  public void testQueryWithHint()
-      throws Exception {
+  public void testQueryWithHint() {
     // Hinting the query to use final stage aggregation makes server directly return final result
     // This is useful when data is already partitioned by col1
     String query = "SELECT /*+ aggOptionsInternal(agg_type='DIRECT') */ col1, COUNT(*) FROM b GROUP BY col1";
     DispatchableSubPlan dispatchableSubPlan = _queryEnvironment.planQuery(query);
-    Assert.assertEquals(dispatchableSubPlan.getQueryStageList().size(), 2);
-    for (int stageId = 0; stageId < dispatchableSubPlan.getQueryStageList().size(); stageId++) {
-      DispatchablePlanFragment dispatchablePlanFragment = dispatchableSubPlan.getQueryStageList().get(stageId);
-      String tableName = dispatchablePlanFragment.getTableName();
+    List<DispatchablePlanFragment> stagePlans = dispatchableSubPlan.getQueryStageList();
+    int numStages = stagePlans.size();
+    assertEquals(numStages, 2);
+    for (int stageId = 0; stageId < numStages; stageId++) {
+      DispatchablePlanFragment stagePlan = stagePlans.get(stageId);
+      Map<QueryServerInstance, List<Integer>> serverToWorkerIdsMap = stagePlan.getServerInstanceToWorkerIdMap();
+      int numServers = serverToWorkerIdsMap.size();
+      String tableName = stagePlan.getTableName();
       if (tableName != null) {
         // table scan stages; for tableB it should have only 1
-        Assert.assertEquals(dispatchablePlanFragment.getServerInstanceToWorkerIdMap().entrySet().stream()
-                .map(PhysicalExplainPlanVisitor::stringifyQueryServerInstanceToWorkerIdsEntry)
-                .collect(Collectors.toSet()),
-            ImmutableList.of("localhost@{1,1}|[0]"));
+        assertEquals(numServers, 1);
+        assertEquals(stagePlan.getServerInstanceToWorkerIdMap().keySet().iterator().next().getQueryMailboxPort(), 1);
       } else if (!PlannerUtils.isRootPlanFragment(stageId)) {
         // join stage should have both servers used.
-        Assert.assertEquals(dispatchablePlanFragment.getServerInstanceToWorkerIdMap().entrySet().stream()
-                .map(PhysicalExplainPlanVisitor::stringifyQueryServerInstanceToWorkerIdsEntry)
-                .collect(Collectors.toSet()),
-            ImmutableList.of("localhost@{1,1}|[1]", "localhost@{2,2}|[0]"));
+        assertEquals(numServers, 2);
+        for (QueryServerInstance server : serverToWorkerIdsMap.keySet()) {
+          int port = server.getQueryMailboxPort();
+          assertTrue(port == 1 || port == 2);
+        }
       }
     }
   }
@@ -275,105 +268,103 @@ public class QueryCompilationTest extends QueryEnvironmentTestBase {
     // A simple filter query with one table
     String query = "Select * from a where col1 = 'a'";
     List<String> tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 1);
-    Assert.assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.size(), 1);
+    assertEquals(tableNames.get(0), "a");
 
     // query with IN / NOT IN clause
-    query = "SELECT COUNT(*) FROM a WHERE col1 IN (SELECT col1 FROM b) "
-        + "and col1 NOT IN (SELECT col1 from c)";
+    query = "SELECT COUNT(*) FROM a WHERE col1 IN (SELECT col1 FROM b) " + "and col1 NOT IN (SELECT col1 from c)";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 3);
+    assertEquals(tableNames.size(), 3);
     Collections.sort(tableNames);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
-    Assert.assertEquals(tableNames.get(2), "c");
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.get(2), "c");
 
     // query with JOIN clause
     query = "SELECT a.col1, b.col2 FROM a JOIN b ON a.col3 = b.col3 WHERE a.col1 = 'a'";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 2);
+    assertEquals(tableNames.size(), 2);
     Collections.sort(tableNames);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
 
     // query with WHERE clause JOIN
     query = "SELECT a.col1, b.col2 FROM a, b WHERE a.col3 = b.col3 AND a.col1 = 'a'";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 2);
+    assertEquals(tableNames.size(), 2);
     Collections.sort(tableNames);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
 
     // query with JOIN clause and table alias
     query = "SELECT A.col1, B.col2 FROM a AS A JOIN b AS B ON A.col3 = B.col3 WHERE A.col1 = 'a'";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 2);
+    assertEquals(tableNames.size(), 2);
     Collections.sort(tableNames);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
 
     // query with UNION clause
     query = "SELECT * FROM a UNION ALL SELECT * FROM b UNION ALL SELECT * FROM c";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 3);
+    assertEquals(tableNames.size(), 3);
     Collections.sort(tableNames);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
-    Assert.assertEquals(tableNames.get(2), "c");
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.get(2), "c");
 
     // query with UNION clause and table alias
     query = "SELECT * FROM (SELECT * FROM a) AS t1 UNION SELECT * FROM ( SELECT * FROM b) AS t2";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 2);
+    assertEquals(tableNames.size(), 2);
     Collections.sort(tableNames);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
 
     // query with UNION clause and table alias using WITH clause
-    query = "WITH tmp1 AS (SELECT * FROM a), \n"
-        + "tmp2 AS (SELECT * FROM b) \n"
+    query = "WITH tmp1 AS (SELECT * FROM a), \n" + "tmp2 AS (SELECT * FROM b) \n"
         + "SELECT * FROM tmp1 UNION ALL SELECT * FROM tmp2";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 2);
+    assertEquals(tableNames.size(), 2);
     Collections.sort(tableNames);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
 
     // query with aliases, JOIN, IN/NOT-IN, group-by
     query = "with tmp as (select col1, sum(col3) as col3, count(*) from a where col1 = 'a' group by col1), "
         + "tmp2 as (select A.col1, B.col3 from b as A JOIN c AS B on A.col1 = B.col1) "
         + "select sum(col3) from tmp where col1 in (select col1 from tmp2) and col1 not in (select col1 from d)";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 4);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
-    Assert.assertEquals(tableNames.get(2), "c");
-    Assert.assertEquals(tableNames.get(3), "d");
+    assertEquals(tableNames.size(), 4);
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.get(2), "c");
+    assertEquals(tableNames.get(3), "d");
 
     // query with aliases, JOIN, IN/NOT-IN, group-by and explain
     query = "explain plan for with tmp as (select col1, sum(col3) as col3, count(*) from a where col1 = 'a' "
         + "group by col1), tmp2 as (select A.col1, B.col3 from b as A JOIN c AS B on A.col1 = B.col1) "
         + "select sum(col3) from tmp where col1 in (select col1 from tmp2) and col1 not in (select col1 from d)";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 4);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
-    Assert.assertEquals(tableNames.get(2), "c");
-    Assert.assertEquals(tableNames.get(3), "d");
+    assertEquals(tableNames.size(), 4);
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.get(2), "c");
+    assertEquals(tableNames.get(3), "d");
 
     // lateral join query
     query = "EXPLAIN PLAN FOR SELECT a.col1, newb.sum_col3 FROM a JOIN LATERAL "
         + "(SELECT SUM(col3) as sum_col3 FROM b WHERE col2 = a.col2) AS newb ON TRUE";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 2);
-    Assert.assertEquals(tableNames.get(0), "a");
-    Assert.assertEquals(tableNames.get(1), "b");
+    assertEquals(tableNames.size(), 2);
+    assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.get(1), "b");
 
     // test for self join queries
     query = "SELECT a.col1 FROM a JOIN(SELECT col2 FROM a) as self ON a.col1=self.col2 ";
     tableNames = _queryEnvironment.getTableNamesForQuery(query);
-    Assert.assertEquals(tableNames.size(), 1);
-    Assert.assertEquals(tableNames.get(0), "a");
+    assertEquals(tableNames.size(), 1);
+    assertEquals(tableNames.get(0), "a");
   }
 
   // --------------------------------------------------------------------------
@@ -381,7 +372,7 @@ public class QueryCompilationTest extends QueryEnvironmentTestBase {
   // --------------------------------------------------------------------------
 
   private static void assertNodeTypeNotIn(PlanNode node, List<Class<? extends AbstractPlanNode>> bannedNodeType) {
-    Assert.assertFalse(isOneOf(bannedNodeType, node));
+    assertFalse(isOneOf(bannedNodeType, node));
     for (PlanNode child : node.getInputs()) {
       assertNodeTypeNotIn(child, bannedNodeType);
     }
@@ -466,49 +457,4 @@ public class QueryCompilationTest extends QueryEnvironmentTestBase {
     };
     //@formatter:on
   }
-
-  @DataProvider(name = "testQueryPhysicalPlanDataProvider")
-  private Object[][] provideQueriesWithExplainedPhysicalPlan() {
-    //@formatter:off
-    return new Object[][] {
-new Object[]{"EXPLAIN IMPLEMENTATION PLAN INCLUDING ALL ATTRIBUTES FOR SELECT col1, col3 FROM a",
-  "[0]@localhost:3 MAIL_RECEIVE(BROADCAST_DISTRIBUTED)\n"
-+ "├── [1]@localhost:1 MAIL_SEND(BROADCAST_DISTRIBUTED)->{[0]@localhost@{3,3}|[0]}\n"
-+ "│   └── [1]@localhost:1 PROJECT\n"
-+ "│       └── [1]@localhost:1 TABLE SCAN (a) null\n"
-+ "└── [1]@localhost:2 MAIL_SEND(BROADCAST_DISTRIBUTED)->{[0]@localhost@{3,3}|[0]}\n"
-+ "    └── [1]@localhost:2 PROJECT\n"
-+ "        └── [1]@localhost:2 TABLE SCAN (a) null\n"},
-new Object[]{"EXPLAIN IMPLEMENTATION PLAN EXCLUDING ATTRIBUTES FOR SELECT col1, COUNT(*) FROM a GROUP BY col1",
-"[0]@localhost:3 MAIL_RECEIVE(BROADCAST_DISTRIBUTED)\n"
-+ "├── [1]@localhost:1 MAIL_SEND(BROADCAST_DISTRIBUTED)->{[0]@localhost@{3,3}|[0]} (Subtree Omitted)\n"
-+ "└── [1]@localhost:2 MAIL_SEND(BROADCAST_DISTRIBUTED)->{[0]@localhost@{3,3}|[0]}\n"
-+ "    └── [1]@localhost:2 AGGREGATE_FINAL\n"
-+ "        └── [1]@localhost:2 MAIL_RECEIVE(HASH_DISTRIBUTED)\n"
-+ "            ├── [2]@localhost:1 MAIL_SEND(HASH_DISTRIBUTED)->{[1]@localhost@{1,1}|[1],[1]@localhost@{2,2}|[0]}\n"
-+ "            │   └── [2]@localhost:1 AGGREGATE_LEAF\n"
-+ "            │       └── [2]@localhost:1 TABLE SCAN (a) null\n"
-+ "            └── [2]@localhost:2 MAIL_SEND(HASH_DISTRIBUTED)->{[1]@localhost@{1,1}|[1],[1]@localhost@{2,2}|[0]}\n"
-+ "                └── [2]@localhost:2 AGGREGATE_LEAF\n"
-+ "                    └── [2]@localhost:2 TABLE SCAN (a) null\n"},
-new Object[]{"EXPLAIN IMPLEMENTATION PLAN FOR SELECT a.col1, b.col3 FROM a JOIN b ON a.col1 = b.col1",
-  "[0]@localhost:3 MAIL_RECEIVE(BROADCAST_DISTRIBUTED)\n"
-+ "├── [1]@localhost:1 MAIL_SEND(BROADCAST_DISTRIBUTED)->{[0]@localhost@{3,3}|[0]} (Subtree Omitted)\n"
-+ "└── [1]@localhost:2 MAIL_SEND(BROADCAST_DISTRIBUTED)->{[0]@localhost@{3,3}|[0]}\n"
-+ "    └── [1]@localhost:2 PROJECT\n"
-+ "        └── [1]@localhost:2 JOIN\n"
-+ "            ├── [1]@localhost:2 MAIL_RECEIVE(HASH_DISTRIBUTED)\n"
-+ "            │   ├── [2]@localhost:1 MAIL_SEND(HASH_DISTRIBUTED)->{[1]@localhost@{1,1}|[1],[1]@localhost@{2,2}|[0]}\n"
-+ "            │   │   └── [2]@localhost:1 PROJECT\n"
-+ "            │   │       └── [2]@localhost:1 TABLE SCAN (a) null\n"
-+ "            │   └── [2]@localhost:2 MAIL_SEND(HASH_DISTRIBUTED)->{[1]@localhost@{1,1}|[1],[1]@localhost@{2,2}|[0]}\n"
-+ "            │       └── [2]@localhost:2 PROJECT\n"
-+ "            │           └── [2]@localhost:2 TABLE SCAN (a) null\n"
-+ "            └── [1]@localhost:2 MAIL_RECEIVE(HASH_DISTRIBUTED)\n"
-+ "                └── [3]@localhost:1 MAIL_SEND(HASH_DISTRIBUTED)->{[1]@localhost@{1,1}|[1],[1]@localhost@{2,2}|[0]}\n"
-+ "                    └── [3]@localhost:1 PROJECT\n"
-+ "                        └── [3]@localhost:1 TABLE SCAN (b) null\n"}
-    };
-    //@formatter:on
-  }
 }
