@@ -39,8 +39,8 @@ public final class TransferableBlockUtils {
     return new TransferableBlock(DataBlockUtils.getEndOfStreamDataBlock());
   }
 
-  public static TransferableBlock getNoOpTransferableBlock() {
-    return new TransferableBlock(DataBlockUtils.getNoOpBlock());
+  public static TransferableBlock getEndOfStreamTransferableBlock(Map<String, String> statsMap) {
+    return new TransferableBlock(DataBlockUtils.getEndOfStreamDataBlock(statsMap));
   }
 
   public static TransferableBlock getErrorTransferableBlock(Exception e) {
@@ -55,10 +55,6 @@ public final class TransferableBlockUtils {
     return transferableBlock.isEndOfStreamBlock();
   }
 
-  public static boolean isNoOpBlock(TransferableBlock transferableBlock) {
-    return transferableBlock.isNoOpBlock();
-  }
-
   /**
    *
    *  Split a block into multiple block so that each block size is within maxBlockSize. Currently,
