@@ -22,9 +22,8 @@ import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.ImmutableSet;
 import java.util.ArrayList;
-import java.util.Collections;
+import java.util.BitSet;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -32,13 +31,14 @@ import javax.annotation.Nullable;
 import org.apache.calcite.rel.core.JoinRelType;
 import org.apache.calcite.rel.hint.PinotHintOptions;
 import org.apache.pinot.common.datablock.DataBlock;
+import org.apache.pinot.common.datatable.DataTable;
 import org.apache.pinot.common.exception.QueryException;
 import org.apache.pinot.common.response.ProcessingException;
 import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.common.utils.config.QueryOptionsUtils;
-import org.apache.pinot.core.data.table.Key;
 import org.apache.pinot.query.planner.logical.RexExpression;
 import org.apache.pinot.query.planner.partitioning.KeySelector;
+import org.apache.pinot.query.planner.partitioning.KeySelectorFactory;
 import org.apache.pinot.query.planner.plannode.AbstractPlanNode;
 import org.apache.pinot.query.planner.plannode.JoinNode;
 import org.apache.pinot.query.runtime.blocks.TransferableBlock;
@@ -46,7 +46,6 @@ import org.apache.pinot.query.runtime.blocks.TransferableBlockUtils;
 import org.apache.pinot.query.runtime.operator.operands.TransformOperand;
 import org.apache.pinot.query.runtime.operator.operands.TransformOperandFactory;
 import org.apache.pinot.query.runtime.plan.OpChainExecutionContext;
-import org.apache.pinot.query.runtime.plan.StageMetadata;
 import org.apache.pinot.spi.utils.BooleanUtils;
 import org.apache.pinot.spi.utils.CommonConstants.MultiStageQueryRunner.JoinOverFlowMode;
 
@@ -75,12 +74,11 @@ public class HashJoinOperator extends MultiStageOperator {
       ImmutableSet.of(JoinRelType.INNER, JoinRelType.LEFT, JoinRelType.RIGHT, JoinRelType.FULL, JoinRelType.SEMI,
           JoinRelType.ANTI);
 
-  private final HashMap<Key, ArrayList<Object[]>> _broadcastRightTable;
+  private final Map<Object, ArrayList<Object[]>> _broadcastRightTable;
 
   // Used to track matched right rows.
   // Only used for right join and full join to output non-matched right rows.
-  // TODO: Replace hashset with rolling bit map.
-  private final HashMap<Key, HashSet<Integer>> _matchedRightRows;
+  private final Map<Object, BitSet> _matchedRightRows;
 
   private final MultiStageOperator _leftTableOperator;
   private final MultiStageOperator _rightTableOperator;
@@ -96,8 +94,8 @@ public class HashJoinOperator extends MultiStageOperator {
   // TODO: Remove this special handling by fixing data block EOS abstraction or operator's invariant.
   private boolean _isTerminated;
   private TransferableBlock _upstreamErrorBlock;
-  private final KeySelector<Object[], Object[]> _leftKeySelector;
-  private final KeySelector<Object[], Object[]> _rightKeySelector;
+  private final KeySelector<?> _leftKeySelector;
+  private final KeySelector<?> _rightKeySelector;
 
   // Below are specific parameters to protect the hash table from growing too large.
   // Once the hash table reaches the limit, we will throw exception or break the right table build process.
@@ -113,7 +111,6 @@ public class HashJoinOperator extends MultiStageOperator {
   private final JoinOverFlowMode _joinOverflowMode;
 
   private int _currentRowsInHashTable = 0;
-  private ProcessingException _resourceLimitExceededException = null;
 
   public HashJoinOperator(OpChainExecutionContext context, MultiStageOperator leftTableOperator,
       MultiStageOperator rightTableOperator, DataSchema leftSchema, JoinNode node) {
@@ -121,10 +118,9 @@ public class HashJoinOperator extends MultiStageOperator {
     Preconditions.checkState(SUPPORTED_JOIN_TYPES.contains(node.getJoinRelType()),
         "Join type: " + node.getJoinRelType() + " is not supported!");
     _joinType = node.getJoinRelType();
-    _leftKeySelector = node.getJoinKeys().getLeftJoinKeySelector();
-    _rightKeySelector = node.getJoinKeys().getRightJoinKeySelector();
-    Preconditions.checkState(_leftKeySelector != null, "LeftKeySelector for join cannot be null");
-    Preconditions.checkState(_rightKeySelector != null, "RightKeySelector for join cannot be null");
+    JoinNode.JoinKeys joinKeys = node.getJoinKeys();
+    _leftKeySelector = KeySelectorFactory.getKeySelector(joinKeys.getLeftKeys());
+    _rightKeySelector = KeySelectorFactory.getKeySelector(joinKeys.getRightKeys());
     _leftColumnSize = leftSchema.size();
     Preconditions.checkState(_leftColumnSize > 0, "leftColumnSize has to be greater than zero:" + _leftColumnSize);
     _resultSchema = node.getDataSchema();
@@ -145,14 +141,12 @@ public class HashJoinOperator extends MultiStageOperator {
     } else {
       _matchedRightRows = null;
     }
-    StageMetadata stageMetadata = context.getStageMetadata();
-    Map<String, String> customProperties =
-        stageMetadata != null ? stageMetadata.getCustomProperties() : Collections.emptyMap();
-    _maxRowsInHashTable = getMaxRowInJoin(customProperties, node.getJoinHints());
-    _joinOverflowMode = getJoinOverflowMode(customProperties, node.getJoinHints());
+    Map<String, String> metadata = context.getOpChainMetadata();
+    _maxRowsInHashTable = getMaxRowInJoin(metadata, node.getJoinHints());
+    _joinOverflowMode = getJoinOverflowMode(metadata, node.getJoinHints());
   }
 
-  private int getMaxRowInJoin(Map<String, String> customProperties, @Nullable AbstractPlanNode.NodeHint nodeHint) {
+  private int getMaxRowInJoin(Map<String, String> opChainMetadata, @Nullable AbstractPlanNode.NodeHint nodeHint) {
     if (nodeHint != null) {
       Map<String, String> joinOptions = nodeHint._hintOptions.get(PinotHintOptions.JOIN_HINT_OPTIONS);
       if (joinOptions != null) {
@@ -162,11 +156,11 @@ public class HashJoinOperator extends MultiStageOperator {
         }
       }
     }
-    Integer maxRowsInJoin = QueryOptionsUtils.getMaxRowsInJoin(customProperties);
+    Integer maxRowsInJoin = QueryOptionsUtils.getMaxRowsInJoin(opChainMetadata);
     return maxRowsInJoin != null ? maxRowsInJoin : DEFAULT_MAX_ROWS_IN_JOIN;
   }
 
-  private JoinOverFlowMode getJoinOverflowMode(Map<String, String> customProperties,
+  private JoinOverFlowMode getJoinOverflowMode(Map<String, String> contextMetadata,
       @Nullable AbstractPlanNode.NodeHint nodeHint) {
     if (nodeHint != null) {
       Map<String, String> joinOptions = nodeHint._hintOptions.get(PinotHintOptions.JOIN_HINT_OPTIONS);
@@ -177,7 +171,7 @@ public class HashJoinOperator extends MultiStageOperator {
         }
       }
     }
-    JoinOverFlowMode joinOverflowMode = QueryOptionsUtils.getJoinOverflowMode(customProperties);
+    JoinOverFlowMode joinOverflowMode = QueryOptionsUtils.getJoinOverflowMode(contextMetadata);
     return joinOverflowMode != null ? joinOverflowMode : DEFAULT_JOIN_OVERFLOW_MODE;
   }
 
@@ -194,24 +188,21 @@ public class HashJoinOperator extends MultiStageOperator {
   }
 
   @Override
-  protected TransferableBlock getNextBlock() {
-    try {
-      if (_isTerminated) {
-        return setPartialResultExceptionToBlock(TransferableBlockUtils.getEndOfStreamTransferableBlock());
-      }
-      if (!_isHashTableBuilt) {
-        // Build JOIN hash table
-        buildBroadcastHashTable();
-      }
-      if (_upstreamErrorBlock != null) {
-        return _upstreamErrorBlock;
-      }
-      TransferableBlock leftBlock = _leftTableOperator.nextBlock();
-      // JOIN each left block with the right block.
-      return setPartialResultExceptionToBlock(buildJoinedDataBlock(leftBlock));
-    } catch (Exception e) {
-      return TransferableBlockUtils.getErrorTransferableBlock(e);
+  protected TransferableBlock getNextBlock()
+      throws ProcessingException {
+    if (_isTerminated) {
+      return TransferableBlockUtils.getEndOfStreamTransferableBlock();
     }
+    if (!_isHashTableBuilt) {
+      // Build JOIN hash table
+      buildBroadcastHashTable();
+    }
+    if (_upstreamErrorBlock != null) {
+      return _upstreamErrorBlock;
+    }
+    TransferableBlock leftBlock = _leftTableOperator.nextBlock();
+    // JOIN each left block with the constructed right hash table.
+    return buildJoinedDataBlock(leftBlock);
   }
 
   private void buildBroadcastHashTable()
@@ -221,23 +212,27 @@ public class HashJoinOperator extends MultiStageOperator {
       List<Object[]> container = rightBlock.getContainer();
       // Row based overflow check.
       if (container.size() + _currentRowsInHashTable > _maxRowsInHashTable) {
-        _resourceLimitExceededException =
-            new ProcessingException(QueryException.SERVER_RESOURCE_LIMIT_EXCEEDED_ERROR_CODE);
-        _resourceLimitExceededException.setMessage(
-            "Cannot build in memory hash table for join operator, reach number of rows limit: " + _maxRowsInHashTable);
         if (_joinOverflowMode == JoinOverFlowMode.THROW) {
-          throw _resourceLimitExceededException;
+          ProcessingException resourceLimitExceededException =
+              new ProcessingException(QueryException.SERVER_RESOURCE_LIMIT_EXCEEDED_ERROR_CODE);
+          resourceLimitExceededException.setMessage(
+              "Cannot build in memory hash table for join operator, reach number of rows limit: "
+                  + _maxRowsInHashTable);
+          throw resourceLimitExceededException;
         } else {
           // Just fill up the buffer.
           int remainingRows = _maxRowsInHashTable - _currentRowsInHashTable;
           container = container.subList(0, remainingRows);
+          OperatorStats operatorStats = _opChainStats.getOperatorStats(_context, _operatorId);
+          operatorStats.recordSingleStat(DataTable.MetadataKey.MAX_ROWS_IN_JOIN_REACHED.getName(), "true");
+          // setting only the rightTableOperator to be early terminated and awaits EOS block next.
+          _rightTableOperator.earlyTerminate();
         }
       }
       // put all the rows into corresponding hash collections keyed by the key selector function.
       for (Object[] row : container) {
-        ArrayList<Object[]> hashCollection =
-            _broadcastRightTable.computeIfAbsent(new Key(_rightKeySelector.getKey(row)),
-                k -> new ArrayList<>(INITIAL_HEURISTIC_SIZE));
+        ArrayList<Object[]> hashCollection = _broadcastRightTable.computeIfAbsent(_rightKeySelector.getKey(row),
+            k -> new ArrayList<>(INITIAL_HEURISTIC_SIZE));
         int size = hashCollection.size();
         if ((size & size - 1) == 0 && size < _maxRowsInHashTable && size < Integer.MAX_VALUE / 2) { // is power of 2
           hashCollection.ensureCapacity(Math.min(size << 1, _maxRowsInHashTable));
@@ -245,11 +240,6 @@ public class HashJoinOperator extends MultiStageOperator {
         hashCollection.add(row);
       }
       _currentRowsInHashTable += container.size();
-      if (_currentRowsInHashTable == _maxRowsInHashTable) {
-        // Early terminate right table operator.
-        _rightTableOperator.close();
-        break;
-      }
       rightBlock = _rightTableOperator.nextBlock();
     }
     if (rightBlock.isErrorBlock()) {
@@ -271,15 +261,18 @@ public class HashJoinOperator extends MultiStageOperator {
       // TODO: Moved to a different function.
       // Return remaining non-matched rows for non-inner join.
       List<Object[]> returnRows = new ArrayList<>();
-      for (Map.Entry<Key, ArrayList<Object[]>> entry : _broadcastRightTable.entrySet()) {
-        Set<Integer> matchedIdx = _matchedRightRows.getOrDefault(entry.getKey(), new HashSet<>());
+      for (Map.Entry<Object, ArrayList<Object[]>> entry : _broadcastRightTable.entrySet()) {
         List<Object[]> rightRows = entry.getValue();
-        if (rightRows.size() == matchedIdx.size()) {
-          continue;
-        }
-        for (int i = 0; i < rightRows.size(); i++) {
-          if (!matchedIdx.contains(i)) {
-            returnRows.add(joinRow(null, rightRows.get(i)));
+        BitSet matchedIndices = _matchedRightRows.get(entry.getKey());
+        if (matchedIndices == null) {
+          for (Object[] rightRow : rightRows) {
+            returnRows.add(joinRow(null, rightRow));
+          }
+        } else {
+          int numRightRows = rightRows.size();
+          int unmatchedIndex = 0;
+          while ((unmatchedIndex = matchedIndices.nextClearBit(unmatchedIndex)) < numRightRows) {
+            returnRows.add(joinRow(null, rightRows.get(unmatchedIndex++)));
           }
         }
       }
@@ -305,19 +298,12 @@ public class HashJoinOperator extends MultiStageOperator {
     return new TransferableBlock(rows, _resultSchema, DataBlock.Type.ROW);
   }
 
-  private TransferableBlock setPartialResultExceptionToBlock(TransferableBlock block) {
-    if (_resourceLimitExceededException != null) {
-      block.addException(_resourceLimitExceededException);
-    }
-    return block;
-  }
-
   private List<Object[]> buildJoinedDataBlockSemi(TransferableBlock leftBlock) {
     List<Object[]> container = leftBlock.getContainer();
     List<Object[]> rows = new ArrayList<>(container.size());
 
     for (Object[] leftRow : container) {
-      Key key = new Key(_leftKeySelector.getKey(leftRow));
+      Object key = _leftKeySelector.getKey(leftRow);
       // SEMI-JOIN only checks existence of the key
       if (_broadcastRightTable.containsKey(key)) {
         rows.add(joinRow(leftRow, null));
@@ -332,19 +318,20 @@ public class HashJoinOperator extends MultiStageOperator {
     ArrayList<Object[]> rows = new ArrayList<>(container.size());
 
     for (Object[] leftRow : container) {
-      Key key = new Key(_leftKeySelector.getKey(leftRow));
+      Object key = _leftKeySelector.getKey(leftRow);
       // NOTE: Empty key selector will always give same hash code.
-      List<Object[]> matchedRightRows = _broadcastRightTable.getOrDefault(key, null);
-      if (matchedRightRows == null) {
+      List<Object[]> rightRows = _broadcastRightTable.get(key);
+      if (rightRows == null) {
         if (needUnmatchedLeftRows()) {
           rows.add(joinRow(leftRow, null));
         }
         continue;
       }
       boolean hasMatchForLeftRow = false;
-      rows.ensureCapacity(rows.size() + matchedRightRows.size());
-      for (int i = 0; i < matchedRightRows.size(); i++) {
-        Object[] rightRow = matchedRightRows.get(i);
+      int numRightRows = rightRows.size();
+      rows.ensureCapacity(rows.size() + numRightRows);
+      for (int i = 0; i < numRightRows; i++) {
+        Object[] rightRow = rightRows.get(i);
         // TODO: Optimize this to avoid unnecessary object copy.
         Object[] resultRow = joinRow(leftRow, rightRow);
         if (_joinClauseEvaluators.isEmpty() || _joinClauseEvaluators.stream()
@@ -352,8 +339,7 @@ public class HashJoinOperator extends MultiStageOperator {
           rows.add(resultRow);
           hasMatchForLeftRow = true;
           if (_matchedRightRows != null) {
-            HashSet<Integer> matchedRows = _matchedRightRows.computeIfAbsent(key, k -> new HashSet<>());
-            matchedRows.add(i);
+            _matchedRightRows.computeIfAbsent(key, k -> new BitSet(numRightRows)).set(i);
           }
         }
       }
@@ -370,7 +356,7 @@ public class HashJoinOperator extends MultiStageOperator {
     List<Object[]> rows = new ArrayList<>(container.size());
 
     for (Object[] leftRow : container) {
-      Key key = new Key(_leftKeySelector.getKey(leftRow));
+      Object key = _leftKeySelector.getKey(leftRow);
       // ANTI-JOIN only checks non-existence of the key
       if (!_broadcastRightTable.containsKey(key)) {
         rows.add(joinRow(leftRow, null));
