@@ -45,7 +45,8 @@ public class LiteralValueOperator extends MultiStageOperator {
     super(context);
     _dataSchema = dataSchema;
     _rexLiteralBlock = constructBlock(rexLiteralRows);
-    _isLiteralBlockReturned = false;
+    // only return a single literal block when it is the 1st virtual server. otherwise, result will be duplicated.
+    _isLiteralBlockReturned = context.getId().getVirtualServerId() != 0;
   }
 
   @Override
@@ -61,7 +62,7 @@ public class LiteralValueOperator extends MultiStageOperator {
 
   @Override
   protected TransferableBlock getNextBlock() {
-    if (!_isLiteralBlockReturned) {
+    if (!_isLiteralBlockReturned && !_isEarlyTerminated) {
       _isLiteralBlockReturned = true;
       return _rexLiteralBlock;
     } else {
