@@ -163,22 +163,14 @@ public class WindowAggregateOperator extends MultiStageOperator {
 
   @Override
   protected TransferableBlock getNextBlock() {
-    try {
-      TransferableBlock finalBlock = consumeInputBlocks();
-      if (finalBlock.isErrorBlock()) {
-        return finalBlock;
-      }
-
-      if (!_hasReturnedWindowAggregateBlock) {
-        return produceWindowAggregatedBlock();
-      } else {
-        // TODO: Move to close call.
-        return TransferableBlockUtils.getEndOfStreamTransferableBlock();
-      }
-    } catch (Exception e) {
-      LOGGER.error("Caught exception while executing WindowAggregationOperator, returning an error block", e);
-      return TransferableBlockUtils.getErrorTransferableBlock(e);
+    if (_hasReturnedWindowAggregateBlock) {
+      return TransferableBlockUtils.getEndOfStreamTransferableBlock();
     }
+    TransferableBlock finalBlock = consumeInputBlocks();
+    if (finalBlock.isErrorBlock()) {
+      return finalBlock;
+    }
+    return produceWindowAggregatedBlock();
   }
 
   private void validateAggregationCalls(String functionName,
