@@ -20,7 +20,6 @@ package org.apache.pinot.query.runtime.plan;
 
 import java.util.ArrayList;
 import java.util.List;
-import org.apache.pinot.common.utils.DataSchema;
 import org.apache.pinot.query.planner.plannode.AggregateNode;
 import org.apache.pinot.query.planner.plannode.ExchangeNode;
 import org.apache.pinot.query.planner.plannode.FilterNode;
@@ -39,6 +38,7 @@ import org.apache.pinot.query.runtime.operator.AggregateOperator;
 import org.apache.pinot.query.runtime.operator.FilterOperator;
 import org.apache.pinot.query.runtime.operator.HashJoinOperator;
 import org.apache.pinot.query.runtime.operator.IntersectOperator;
+import org.apache.pinot.query.runtime.operator.LeafStageTransferableBlockOperator;
 import org.apache.pinot.query.runtime.operator.LiteralValueOperator;
 import org.apache.pinot.query.runtime.operator.MailboxReceiveOperator;
 import org.apache.pinot.query.runtime.operator.MailboxSendOperator;
@@ -50,6 +50,7 @@ import org.apache.pinot.query.runtime.operator.SortedMailboxReceiveOperator;
 import org.apache.pinot.query.runtime.operator.TransformOperator;
 import org.apache.pinot.query.runtime.operator.UnionOperator;
 import org.apache.pinot.query.runtime.operator.WindowAggregateOperator;
+import org.apache.pinot.query.runtime.plan.server.ServerPlanRequestContext;
 
 
 /**
@@ -68,6 +69,17 @@ public class PhysicalPlanVisitor implements PlanNodeVisitor<MultiStageOperator,
     return new OpChain(context, root);
   }
 
+  private <T extends PlanNode> MultiStageOperator visit(T node, OpChainExecutionContext context) {
+    if (context.getLeafStageContext() != null && context.getLeafStageContext().getLeafStageBoundaryNode() == node) {
+      ServerPlanRequestContext leafStageContext = context.getLeafStageContext();
+      return new LeafStageTransferableBlockOperator(context, leafStageContext.getServerQueryRequests(),
+          leafStageContext.getLeafStageBoundaryNode().getDataSchema(), leafStageContext.getLeafQueryExecutor(),
+          leafStageContext.getExecutorService());
+    } else {
+      return node.visit(this, context);
+    }
+  }
+
   @Override
   public MultiStageOperator visitMailboxReceive(MailboxReceiveNode node, OpChainExecutionContext context) {
     if (node.isSortOnReceiver()) {
@@ -81,24 +93,21 @@ public class PhysicalPlanVisitor implements PlanNodeVisitor<MultiStageOperator,
 
   @Override
   public MultiStageOperator visitMailboxSend(MailboxSendNode node, OpChainExecutionContext context) {
-    MultiStageOperator nextOperator = node.getInputs().get(0).visit(this, context);
-    return new MailboxSendOperator(context, nextOperator, node.getDistributionType(), node.getPartitionKeySelector(),
+    MultiStageOperator nextOperator = visit(node.getInputs().get(0), context);
+    return new MailboxSendOperator(context, nextOperator, node.getDistributionType(), node.getDistributionKeys(),
         node.getCollationKeys(), node.getCollationDirections(), node.isSortOnSender(), node.getReceiverStageId());
   }
 
   @Override
   public MultiStageOperator visitAggregate(AggregateNode node, OpChainExecutionContext context) {
-    MultiStageOperator nextOperator = node.getInputs().get(0).visit(this, context);
-    DataSchema inputSchema = node.getInputs().get(0).getDataSchema();
-    DataSchema resultSchema = node.getDataSchema();
-
-    return new AggregateOperator(context, nextOperator, resultSchema, inputSchema, node.getAggCalls(),
+    MultiStageOperator nextOperator = visit(node.getInputs().get(0), context);
+    return new AggregateOperator(context, nextOperator, node.getDataSchema(), node.getAggCalls(),
         node.getGroupSet(), node.getAggType(), node.getFilterArgIndices(), node.getNodeHint());
   }
 
   @Override
   public MultiStageOperator visitWindow(WindowNode node, OpChainExecutionContext context) {
-    MultiStageOperator nextOperator = node.getInputs().get(0).visit(this, context);
+    MultiStageOperator nextOperator = visit(node.getInputs().get(0), context);
     return new WindowAggregateOperator(context, nextOperator, node.getGroupSet(), node.getOrderSet(),
         node.getOrderSetDirection(), node.getOrderSetNullDirection(), node.getAggCalls(), node.getLowerBound(),
         node.getUpperBound(), node.getWindowFrameType(), node.getConstants(), node.getDataSchema(),
@@ -109,7 +118,7 @@ public class PhysicalPlanVisitor implements PlanNodeVisitor<MultiStageOperator,
   public MultiStageOperator visitSetOp(SetOpNode setOpNode, OpChainExecutionContext context) {
     List<MultiStageOperator> inputs = new ArrayList<>();
     for (PlanNode input : setOpNode.getInputs()) {
-      MultiStageOperator visited = input.visit(this, context);
+      MultiStageOperator visited = visit(input, context);
       inputs.add(visited);
     }
     switch (setOpNode.getSetOpType()) {
@@ -131,7 +140,7 @@ public class PhysicalPlanVisitor implements PlanNodeVisitor<MultiStageOperator,
 
   @Override
   public MultiStageOperator visitFilter(FilterNode node, OpChainExecutionContext context) {
-    MultiStageOperator nextOperator = node.getInputs().get(0).visit(this, context);
+    MultiStageOperator nextOperator = visit(node.getInputs().get(0), context);
     return new FilterOperator(context, nextOperator, node.getDataSchema(), node.getCondition());
   }
 
@@ -140,22 +149,22 @@ public class PhysicalPlanVisitor implements PlanNodeVisitor<MultiStageOperator,
     PlanNode left = node.getInputs().get(0);
     PlanNode right = node.getInputs().get(1);
 
-    MultiStageOperator leftOperator = left.visit(this, context);
-    MultiStageOperator rightOperator = right.visit(this, context);
+    MultiStageOperator leftOperator = visit(left, context);
+    MultiStageOperator rightOperator = visit(right, context);
 
     return new HashJoinOperator(context, leftOperator, rightOperator, left.getDataSchema(), node);
   }
 
   @Override
   public MultiStageOperator visitProject(ProjectNode node, OpChainExecutionContext context) {
-    MultiStageOperator nextOperator = node.getInputs().get(0).visit(this, context);
+    MultiStageOperator nextOperator = visit(node.getInputs().get(0), context);
     return new TransformOperator(context, nextOperator, node.getDataSchema(), node.getProjects(),
         node.getInputs().get(0).getDataSchema());
   }
 
   @Override
   public MultiStageOperator visitSort(SortNode node, OpChainExecutionContext context) {
-    MultiStageOperator nextOperator = node.getInputs().get(0).visit(this, context);
+    MultiStageOperator nextOperator = visit(node.getInputs().get(0), context);
     boolean isInputSorted = nextOperator instanceof SortedMailboxReceiveOperator;
     return new SortOperator(context, nextOperator, node.getCollationKeys(), node.getCollationDirections(),
         node.getCollationNullDirections(), node.getFetch(), node.getOffset(), node.getDataSchema(), isInputSorted);
