@@ -20,6 +20,7 @@ package org.apache.pinot.query.runtime.operator;
 
 import com.google.common.collect.ImmutableList;
 import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
 import java.util.stream.Collectors;
 import org.apache.calcite.rel.RelFieldCollation.Direction;
@@ -618,6 +619,32 @@ public class SortOperatorTest {
     Assert.assertTrue(block2.isEndOfStreamBlock(), "expected EOS block to propagate");
   }
 
+  @Test
+  public void shouldEarlyTerminateCorrectlyWithSignalingPropagateUpstream() {
+    // Given:
+    List<RexExpression> collation = Collections.emptyList();
+    List<Direction> directions = ImmutableList.of(Direction.ASCENDING);
+    List<NullDirection> nullDirections = ImmutableList.of(NullDirection.LAST);
+    DataSchema schema = new DataSchema(new String[]{"sort"}, new DataSchema.ColumnDataType[]{INT});
+    SortOperator op =
+        new SortOperator(OperatorTestUtil.getDefaultContext(), _input, collation, directions, nullDirections, 10, 0,
+            schema, false);
+
+    Mockito.when(_input.nextBlock()).thenReturn(block(schema, new Object[]{1}, new Object[]{2}, new Object[]{3},
+        new Object[]{4}, new Object[]{5}, new Object[]{6}, new Object[]{7}, new Object[]{8}, new Object[]{9},
+        new Object[]{10}, new Object[]{11}, new Object[]{12}))
+        .thenReturn(TransferableBlockUtils.getEndOfStreamTransferableBlock());
+
+    // When:
+    TransferableBlock block = op.nextBlock(); // construct
+    TransferableBlock block2 = op.nextBlock(); // eos
+
+    // Then:
+    Mockito.verify(_input).earlyTerminate();
+    Assert.assertEquals(block.getNumRows(), 10);
+    Assert.assertTrue(block2.isEndOfStreamBlock(), "expected EOS block to propagate");
+  }
+
   private static List<RexExpression> collation(int... indexes) {
     return Arrays.stream(indexes).mapToObj(RexExpression.InputRef::new).collect(Collectors.toList());
   }
