@@ -24,7 +24,7 @@ import java.util.Arrays;
 import java.util.Random;
 import java.util.stream.IntStream;
 import org.apache.commons.io.FileUtils;
-import org.apache.pinot.segment.local.io.writer.impl.FixedByteChunkSVForwardIndexWriter;
+import org.apache.pinot.segment.local.io.writer.impl.FixedByteChunkForwardIndexWriter;
 import org.apache.pinot.segment.local.segment.index.readers.forward.ChunkReaderContext;
 import org.apache.pinot.segment.local.segment.index.readers.forward.FixedByteChunkSVForwardIndexReader;
 import org.apache.pinot.segment.local.segment.index.readers.forward.FixedBytePower2ChunkSVForwardIndexReader;
@@ -38,14 +38,13 @@ import org.testng.annotations.Test;
 
 
 /**
- * Unit test for {@link FixedByteChunkSVForwardIndexReader} and {@link FixedByteChunkSVForwardIndexWriter} classes.
+ * Unit test for {@link FixedByteChunkSVForwardIndexReader} and {@link FixedByteChunkForwardIndexWriter} classes.
  *
- * This test writes {@link #NUM_VALUES} using {@link FixedByteChunkSVForwardIndexWriter}. It then reads
+ * This test writes {@link #NUM_VALUES} using {@link FixedByteChunkForwardIndexWriter}. It then reads
  * the values using {@link FixedByteChunkSVForwardIndexReader}, and asserts that what was written is the same as
  * what was read in.
  *
  * Number of docs and docs per chunk are chosen to generate complete as well partial chunks.
- *
  */
 public class FixedByteChunkSVForwardIndexTest {
   private static final int NUM_VALUES = 10009;
@@ -75,10 +74,10 @@ public class FixedByteChunkSVForwardIndexTest {
     FileUtils.deleteQuietly(outFileEightByte);
 
     // test both formats (4-byte chunk offsets and 8-byte chunk offsets)
-    try (FixedByteChunkSVForwardIndexWriter fourByteOffsetWriter = new FixedByteChunkSVForwardIndexWriter(
-        outFileFourByte, compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Integer.BYTES, version);
-        FixedByteChunkSVForwardIndexWriter eightByteOffsetWriter = new FixedByteChunkSVForwardIndexWriter(
-            outFileEightByte, compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Integer.BYTES, version)) {
+    try (FixedByteChunkForwardIndexWriter fourByteOffsetWriter = new FixedByteChunkForwardIndexWriter(outFileFourByte,
+        compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Integer.BYTES, version);
+        FixedByteChunkForwardIndexWriter eightByteOffsetWriter = new FixedByteChunkForwardIndexWriter(outFileEightByte,
+            compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Integer.BYTES, version)) {
       for (int value : expected) {
         fourByteOffsetWriter.putInt(value);
         eightByteOffsetWriter.putInt(value);
@@ -103,6 +102,24 @@ public class FixedByteChunkSVForwardIndexTest {
         Assert.assertEquals(fourByteOffsetReader.getInt(i, fourByteOffsetReaderContext), expected[i]);
         Assert.assertEquals(eightByteOffsetReader.getInt(i, eightByteOffsetReaderContext), expected[i]);
       }
+
+
+      Assert.assertTrue(fourByteOffsetReader.isBufferByteRangeInfoSupported());
+      Assert.assertTrue(eightByteOffsetReader.isBufferByteRangeInfoSupported());
+      // Validate byte range provider behaviour
+      if (compressionType == ChunkCompressionType.PASS_THROUGH) {
+        // For pass through compression, the buffer is fixed offset mapping type
+        Assert.assertTrue(fourByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(fourByteOffsetReader.getDocLength(), Integer.BYTES);
+        Assert.assertFalse(fourByteOffsetReader.isDocLengthInBits());
+
+        Assert.assertTrue(eightByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(eightByteOffsetReader.getDocLength(), Integer.BYTES);
+        Assert.assertFalse(eightByteOffsetReader.isDocLengthInBits());
+      } else {
+        Assert.assertFalse(eightByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertFalse(eightByteOffsetReader.isFixedOffsetMappingType());
+      }
     }
 
     FileUtils.deleteQuietly(outFileFourByte);
@@ -123,10 +140,10 @@ public class FixedByteChunkSVForwardIndexTest {
     FileUtils.deleteQuietly(outFileEightByte);
 
     // test both formats (4-byte chunk offsets and 8-byte chunk offsets)
-    try (FixedByteChunkSVForwardIndexWriter fourByteOffsetWriter = new FixedByteChunkSVForwardIndexWriter(
-        outFileFourByte, compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Long.BYTES, version);
-        FixedByteChunkSVForwardIndexWriter eightByteOffsetWriter = new FixedByteChunkSVForwardIndexWriter(
-            outFileEightByte, compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Long.BYTES, version)) {
+    try (FixedByteChunkForwardIndexWriter fourByteOffsetWriter = new FixedByteChunkForwardIndexWriter(outFileFourByte,
+        compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Long.BYTES, version);
+        FixedByteChunkForwardIndexWriter eightByteOffsetWriter = new FixedByteChunkForwardIndexWriter(outFileEightByte,
+            compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Long.BYTES, version)) {
       for (long value : expected) {
         fourByteOffsetWriter.putLong(value);
         eightByteOffsetWriter.putLong(value);
@@ -151,6 +168,23 @@ public class FixedByteChunkSVForwardIndexTest {
         Assert.assertEquals(fourByteOffsetReader.getLong(i, fourByteOffsetReaderContext), expected[i]);
         Assert.assertEquals(eightByteOffsetReader.getLong(i, eightByteOffsetReaderContext), expected[i]);
       }
+
+      // Validate byte range provider behaviour
+      Assert.assertTrue(fourByteOffsetReader.isBufferByteRangeInfoSupported());
+      Assert.assertTrue(eightByteOffsetReader.isBufferByteRangeInfoSupported());
+      if (compressionType == ChunkCompressionType.PASS_THROUGH) {
+        // For pass through compression, the buffer is fixed offset mapping type
+        Assert.assertTrue(fourByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(fourByteOffsetReader.getDocLength(), Long.BYTES);
+        Assert.assertFalse(fourByteOffsetReader.isDocLengthInBits());
+
+        Assert.assertTrue(eightByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(eightByteOffsetReader.getDocLength(), Long.BYTES);
+        Assert.assertFalse(eightByteOffsetReader.isDocLengthInBits());
+      } else {
+        Assert.assertFalse(fourByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertFalse(eightByteOffsetReader.isFixedOffsetMappingType());
+      }
     }
 
     FileUtils.deleteQuietly(outFileFourByte);
@@ -171,10 +205,10 @@ public class FixedByteChunkSVForwardIndexTest {
     FileUtils.deleteQuietly(outFileEightByte);
 
     // test both formats (4-byte chunk offsets and 8-byte chunk offsets)
-    try (FixedByteChunkSVForwardIndexWriter fourByteOffsetWriter = new FixedByteChunkSVForwardIndexWriter(
-        outFileFourByte, compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Float.BYTES, version);
-        FixedByteChunkSVForwardIndexWriter eightByteOffsetWriter = new FixedByteChunkSVForwardIndexWriter(
-            outFileEightByte, compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Float.BYTES, version)) {
+    try (FixedByteChunkForwardIndexWriter fourByteOffsetWriter = new FixedByteChunkForwardIndexWriter(outFileFourByte,
+        compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Float.BYTES, version);
+        FixedByteChunkForwardIndexWriter eightByteOffsetWriter = new FixedByteChunkForwardIndexWriter(outFileEightByte,
+            compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Float.BYTES, version)) {
       for (float value : expected) {
         fourByteOffsetWriter.putFloat(value);
         eightByteOffsetWriter.putFloat(value);
@@ -199,6 +233,23 @@ public class FixedByteChunkSVForwardIndexTest {
         Assert.assertEquals(fourByteOffsetReader.getFloat(i, fourByteOffsetReaderContext), expected[i]);
         Assert.assertEquals(eightByteOffsetReader.getFloat(i, eightByteOffsetReaderContext), expected[i]);
       }
+
+      // Validate byte range provider behaviour
+      Assert.assertTrue(fourByteOffsetReader.isBufferByteRangeInfoSupported());
+      Assert.assertTrue(eightByteOffsetReader.isBufferByteRangeInfoSupported());
+      if (compressionType == ChunkCompressionType.PASS_THROUGH) {
+        // For pass through compression, the buffer is fixed offset mapping type
+        Assert.assertTrue(fourByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(fourByteOffsetReader.getDocLength(), Float.BYTES);
+        Assert.assertFalse(fourByteOffsetReader.isDocLengthInBits());
+
+        Assert.assertTrue(eightByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(eightByteOffsetReader.getDocLength(), Float.BYTES);
+        Assert.assertFalse(eightByteOffsetReader.isDocLengthInBits());
+      } else {
+        Assert.assertFalse(fourByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertFalse(eightByteOffsetReader.isFixedOffsetMappingType());
+      }
     }
 
     FileUtils.deleteQuietly(outFileFourByte);
@@ -219,10 +270,10 @@ public class FixedByteChunkSVForwardIndexTest {
     FileUtils.deleteQuietly(outFileEightByte);
 
     // test both formats (4-byte chunk offsets and 8-byte chunk offsets)
-    try (FixedByteChunkSVForwardIndexWriter fourByteOffsetWriter = new FixedByteChunkSVForwardIndexWriter(
-        outFileFourByte, compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Double.BYTES, version);
-        FixedByteChunkSVForwardIndexWriter eightByteOffsetWriter = new FixedByteChunkSVForwardIndexWriter(
-            outFileEightByte, compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Double.BYTES, version)) {
+    try (FixedByteChunkForwardIndexWriter fourByteOffsetWriter = new FixedByteChunkForwardIndexWriter(outFileFourByte,
+        compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Double.BYTES, version);
+        FixedByteChunkForwardIndexWriter eightByteOffsetWriter = new FixedByteChunkForwardIndexWriter(outFileEightByte,
+            compressionType, NUM_VALUES, NUM_DOCS_PER_CHUNK, Double.BYTES, version)) {
       for (double value : expected) {
         fourByteOffsetWriter.putDouble(value);
         eightByteOffsetWriter.putDouble(value);
@@ -247,6 +298,39 @@ public class FixedByteChunkSVForwardIndexTest {
         Assert.assertEquals(fourByteOffsetReader.getDouble(i, fourByteOffsetReaderContext), expected[i]);
         Assert.assertEquals(eightByteOffsetReader.getDouble(i, eightByteOffsetReaderContext), expected[i]);
       }
+
+      // Validate byte range provider behaviour
+      Assert.assertTrue(fourByteOffsetReader.isBufferByteRangeInfoSupported());
+      Assert.assertTrue(eightByteOffsetReader.isBufferByteRangeInfoSupported());
+      if (compressionType == ChunkCompressionType.PASS_THROUGH) {
+        // For pass through compression, the buffer is fixed offset mapping type
+        Assert.assertTrue(fourByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(fourByteOffsetReader.getDocLength(), Double.BYTES);
+        Assert.assertFalse(fourByteOffsetReader.isDocLengthInBits());
+
+        Assert.assertTrue(eightByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(eightByteOffsetReader.getDocLength(), Double.BYTES);
+        Assert.assertFalse(eightByteOffsetReader.isDocLengthInBits());
+      } else {
+        Assert.assertFalse(fourByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertFalse(eightByteOffsetReader.isFixedOffsetMappingType());
+      }
+
+      Assert.assertTrue(fourByteOffsetReader.isBufferByteRangeInfoSupported());
+      Assert.assertTrue(eightByteOffsetReader.isBufferByteRangeInfoSupported());
+      if (compressionType == ChunkCompressionType.PASS_THROUGH) {
+        // For pass through compression, the buffer is fixed offset mapping type
+        Assert.assertTrue(fourByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(fourByteOffsetReader.getDocLength(), Double.BYTES);
+        Assert.assertFalse(fourByteOffsetReader.isDocLengthInBits());
+
+        Assert.assertTrue(eightByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertEquals(eightByteOffsetReader.getDocLength(), Double.BYTES);
+        Assert.assertFalse(eightByteOffsetReader.isDocLengthInBits());
+      } else {
+        Assert.assertFalse(fourByteOffsetReader.isFixedOffsetMappingType());
+        Assert.assertFalse(eightByteOffsetReader.isFixedOffsetMappingType());
+      }
     }
 
     FileUtils.deleteQuietly(outFileFourByte);
