@@ -23,30 +23,38 @@ import com.google.common.base.Preconditions;
 import java.io.File;
 import java.io.Serializable;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
+import java.util.TreeMap;
 import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
 import javax.annotation.Nullable;
-import org.apache.commons.collections.CollectionUtils;
 import org.apache.pinot.segment.spi.compression.ChunkCompressionType;
 import org.apache.pinot.segment.spi.creator.name.FixedSegmentNameGenerator;
+import org.apache.pinot.segment.spi.creator.name.NormalizedDateSegmentNameGenerator;
 import org.apache.pinot.segment.spi.creator.name.SegmentNameGenerator;
 import org.apache.pinot.segment.spi.creator.name.SimpleSegmentNameGenerator;
-import org.apache.pinot.segment.spi.index.creator.H3IndexConfig;
+import org.apache.pinot.segment.spi.index.FieldIndexConfigs;
+import org.apache.pinot.segment.spi.index.FieldIndexConfigsUtil;
+import org.apache.pinot.segment.spi.index.ForwardIndexConfig;
+import org.apache.pinot.segment.spi.index.IndexType;
+import org.apache.pinot.segment.spi.index.RangeIndexConfig;
+import org.apache.pinot.segment.spi.index.StandardIndexes;
 import org.apache.pinot.spi.config.table.FSTType;
 import org.apache.pinot.spi.config.table.FieldConfig;
+import org.apache.pinot.spi.config.table.IndexConfig;
 import org.apache.pinot.spi.config.table.IndexingConfig;
 import org.apache.pinot.spi.config.table.SegmentPartitionConfig;
 import org.apache.pinot.spi.config.table.SegmentZKPropsConfig;
 import org.apache.pinot.spi.config.table.StarTreeIndexConfig;
 import org.apache.pinot.spi.config.table.TableConfig;
-import org.apache.pinot.spi.config.table.TimestampIndexGranularity;
+import org.apache.pinot.spi.config.table.ingestion.IngestionConfig;
 import org.apache.pinot.spi.data.DateTimeFieldSpec;
 import org.apache.pinot.spi.data.DateTimeFormatSpec;
 import org.apache.pinot.spi.data.FieldSpec;
@@ -54,9 +62,10 @@ import org.apache.pinot.spi.data.FieldSpec.FieldType;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.data.readers.FileFormat;
 import org.apache.pinot.spi.data.readers.RecordReaderConfig;
+import org.apache.pinot.spi.ingestion.batch.BatchConfigProperties;
+import org.apache.pinot.spi.utils.IngestionConfigUtils;
+import org.apache.pinot.spi.utils.TimestampIndexUtils;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 
 
 /**
@@ -67,21 +76,12 @@ public class SegmentGeneratorConfig implements Serializable {
     EPOCH, SIMPLE_DATE
   }
 
-  private static final Logger LOGGER = LoggerFactory.getLogger(SegmentGeneratorConfig.class);
-  public static final double DEFAULT_NO_DICTIONARY_SIZE_RATIO_THRESHOLD = 0.85d;
-
-  private TableConfig _tableConfig;
-  private final Map<String, String> _customProperties = new HashMap<>();
+  public static final String GENERATE_INV_BEFORE_PUSH_DEPREC_PROP = "generate.inverted.index.before.push";
+  private final TableConfig _tableConfig;
+  // NOTE: Use TreeMap to guarantee the order. The custom properties will be written into the segment metadata.
+  private final TreeMap<String, String> _customProperties = new TreeMap<>();
   private final Set<String> _rawIndexCreationColumns = new HashSet<>();
   private final Map<String, ChunkCompressionType> _rawIndexCompressionType = new HashMap<>();
-  private final List<String> _invertedIndexCreationColumns = new ArrayList<>();
-  private final List<String> _bloomFilterCreationColumns = new ArrayList<>();
-  private final List<String> _rangeIndexCreationColumns = new ArrayList<>();
-  private final List<String> _textIndexCreationColumns = new ArrayList<>();
-  private final List<String> _fstIndexCreationColumns = new ArrayList<>();
-  private final List<String> _jsonIndexCreationColumns = new ArrayList<>();
-  private final Map<String, H3IndexConfig> _h3IndexConfigs = new HashMap<>();
-  private final Map<String, List<TimestampIndexGranularity>> _timestampIndexConfigs = new HashMap<>();
   private final List<String> _columnSortOrder = new ArrayList<>();
   private List<String> _varLengthDictionaryColumns = new ArrayList<>();
   private String _inputFilePath = null;
@@ -93,6 +93,7 @@ public class SegmentGeneratorConfig implements Serializable {
   private String _segmentNamePrefix = null;
   private String _segmentNamePostfix = null;
   private String _segmentTimeColumnName = null;
+  private FieldSpec.DataType _segmentTimeColumnDataType = null;
   private TimeUnit _segmentTimeUnit = null;
   private String _segmentCreationTime = null;
   private String _segmentStartTime = null;
@@ -111,37 +112,36 @@ public class SegmentGeneratorConfig implements Serializable {
   private DateTimeFormatSpec _dateTimeFormatSpec = null;
   // Use on-heap or off-heap memory to generate index (currently only affect inverted index and star-tree v2)
   private boolean _onHeap = false;
-  private boolean _skipTimeValueCheck = false;
   private boolean _nullHandlingEnabled = false;
+  private boolean _continueOnError = false;
+  private boolean _rowTimeValueCheck = false;
+  private boolean _segmentTimeValueCheck = true;
   private boolean _failOnEmptySegment = false;
+  private boolean _optimizeDictionary = false;
   private boolean _optimizeDictionaryForMetrics = false;
-  private double _noDictionarySizeRatioThreshold = DEFAULT_NO_DICTIONARY_SIZE_RATIO_THRESHOLD;
+  private double _noDictionarySizeRatioThreshold = IndexingConfig.DEFAULT_NO_DICTIONARY_SIZE_RATIO_THRESHOLD;
+  private final Map<String, FieldIndexConfigs> _indexConfigsByColName;
 
   // constructed from FieldConfig
   private Map<String, Map<String, String>> _columnProperties = new HashMap<>();
 
   private SegmentZKPropsConfig _segmentZKPropsConfig;
 
-  @Deprecated
-  public SegmentGeneratorConfig() {
-  }
-
   /**
-   * Construct the SegmentGeneratorConfig using schema and table config.
-   * If table config is passed, it will be used to initialize the time column details and the indexing config
-   * This constructor is used during offline data generation.
+   * Constructs the SegmentGeneratorConfig with table config and schema.
+   * NOTE: The passed in table config and schema might be changed.
+   *
    * @param tableConfig table config of the segment. Used for getting time column information and indexing information
    * @param schema schema of the segment to be generated. The time column information should be taken from table config.
    *               However, for maintaining backward compatibility, taking it from schema if table config is null.
    *               This will not work once we start supporting multiple time columns (DateTimeFieldSpec)
    */
   public SegmentGeneratorConfig(TableConfig tableConfig, Schema schema) {
-    Preconditions.checkNotNull(schema);
     Preconditions.checkNotNull(tableConfig);
-    _timestampIndexConfigs.putAll(extractTimestampIndexConfigsFromTableConfig(tableConfig));
-    setSchema(updateSchemaWithTimestampIndexes(schema, _timestampIndexConfigs));
-
+    Preconditions.checkNotNull(schema);
+    TimestampIndexUtils.applyTimestampIndex(tableConfig, schema);
     _tableConfig = tableConfig;
+    _schema = schema;
     setTableName(tableConfig.getTableName());
 
     // NOTE: SegmentGeneratorConfig#setSchema doesn't set the time column anymore. timeColumnName is expected to be
@@ -180,80 +180,79 @@ public class SegmentGeneratorConfig implements Serializable {
       setStarTreeIndexConfigs(indexingConfig.getStarTreeIndexConfigs());
       setEnableDefaultStarTree(indexingConfig.isEnableDefaultStarTree());
 
-      // NOTE: There are 2 ways to configure creating inverted index during segment generation:
-      //       - Set 'generate.inverted.index.before.push' to 'true' in custom config (deprecated)
-      //       - Enable 'createInvertedIndexDuringSegmentGeneration' in indexing config
-      // TODO: Clean up the table configs with the deprecated settings, and always use the one in the indexing config
-      if (indexingConfig.getInvertedIndexColumns() != null) {
-        Map<String, String> customConfigs = tableConfig.getCustomConfig().getCustomConfigs();
-        if ((customConfigs != null && Boolean.parseBoolean(customConfigs.get("generate.inverted.index.before.push")))
-            || indexingConfig.isCreateInvertedIndexDuringSegmentGeneration()) {
-          _invertedIndexCreationColumns.addAll(indexingConfig.getInvertedIndexColumns());
-        }
-      }
-
-      if (indexingConfig.getBloomFilterColumns() != null) {
-        _bloomFilterCreationColumns.addAll(indexingConfig.getBloomFilterColumns());
-      }
-
-      if (indexingConfig.getBloomFilterConfigs() != null) {
-        _bloomFilterCreationColumns.addAll(indexingConfig.getBloomFilterConfigs().keySet());
-      }
-
-      if (indexingConfig.getRangeIndexColumns() != null) {
-        _rangeIndexCreationColumns.addAll(indexingConfig.getRangeIndexColumns());
-      }
-
-      if (indexingConfig.getJsonIndexColumns() != null) {
-        _jsonIndexCreationColumns.addAll(indexingConfig.getJsonIndexColumns());
-      }
-
       List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
       if (fieldConfigList != null) {
         for (FieldConfig fieldConfig : fieldConfigList) {
-          _columnProperties.put(fieldConfig.getName(), fieldConfig.getProperties());
+          Map<String, String> properties = fieldConfig.getProperties();
+          if (properties != null) {
+            _columnProperties.put(fieldConfig.getName(), Collections.unmodifiableMap(properties));
+          }
         }
       }
 
-      extractTextIndexColumnsFromTableConfig(tableConfig);
-      extractFSTIndexColumnsFromTableConfig(tableConfig);
-      extractH3IndexConfigsFromTableConfig(tableConfig);
       extractCompressionCodecConfigsFromTableConfig(tableConfig);
 
-      _fstTypeForFSTIndex = tableConfig.getIndexingConfig().getFSTIndexType();
-
+      _fstTypeForFSTIndex = indexingConfig.getFSTIndexType();
       _nullHandlingEnabled = indexingConfig.isNullHandlingEnabled();
+
+      _optimizeDictionary = indexingConfig.isOptimizeDictionary();
       _optimizeDictionaryForMetrics = indexingConfig.isOptimizeDictionaryForMetrics();
       _noDictionarySizeRatioThreshold = indexingConfig.getNoDictionarySizeRatioThreshold();
     }
+
+    IngestionConfig ingestionConfig = tableConfig.getIngestionConfig();
+    if (ingestionConfig != null) {
+      _continueOnError = ingestionConfig.isContinueOnError();
+      _rowTimeValueCheck = ingestionConfig.isRowTimeValueCheck();
+      _segmentTimeValueCheck = ingestionConfig.isSegmentTimeValueCheck();
+    }
+
+    _indexConfigsByColName = FieldIndexConfigsUtil.createIndexConfigsByColName(tableConfig, schema);
+
+    if (indexingConfig != null) {
+      // NOTE: By default inverted indexes are not created during segment creation
+      // There are 2 ways to configure creating inverted index during segment generation:
+      //       - Set 'generate.inverted.index.before.push' to 'true' in custom config (deprecated)
+      //       - Enable 'createInvertedIndexDuringSegmentGeneration' in indexing config
+      // TODO: Clean up the table configs with the deprecated settings, and always use the one in the indexing config
+      // TODO 2: Decide what to do with this. Index-spi is based on the idea that TableConfig is the source of truth
+      if (indexingConfig.getInvertedIndexColumns() != null) {
+        Map<String, String> customConfigs = tableConfig.getCustomConfig().getCustomConfigs();
+        boolean customConfigEnabled =
+            customConfigs != null && Boolean.parseBoolean(customConfigs.get(GENERATE_INV_BEFORE_PUSH_DEPREC_PROP));
+        boolean indexingConfigEnable = indexingConfig.isCreateInvertedIndexDuringSegmentGeneration();
+        if (!customConfigEnabled && !indexingConfigEnable) {
+          setIndexOn(StandardIndexes.inverted(), IndexConfig.DISABLED, indexingConfig.getInvertedIndexColumns());
+        }
+      }
+    }
   }
 
-  public static Schema updateSchemaWithTimestampIndexes(Schema schema,
-      Map<String, List<TimestampIndexGranularity>> timestampIndexConfigs) {
-    if (timestampIndexConfigs.isEmpty()) {
-      return schema;
+  public <C extends IndexConfig> void setIndexOn(IndexType<C, ?, ?> indexType, C config, String... columns) {
+    setIndexOn(indexType, config, Arrays.asList(columns));
+  }
+
+  @VisibleForTesting
+  public <C extends IndexConfig> void setIndexOn(IndexType<C, ?, ?> indexType, C config,
+      @Nullable Iterable<String> columns) {
+    if (columns == null) {
+      return;
     }
-    List<FieldSpec> timestampColumnWithGranularityFieldSpecs = new ArrayList<>();
-    for (Map.Entry<String, List<TimestampIndexGranularity>> entry : timestampIndexConfigs.entrySet()) {
-      String columnName = entry.getKey();
-      Preconditions.checkState(schema.hasColumn(columnName),
-          "Cannot create Timestamp index for column: %s because it is not in schema", columnName);
-      entry.getValue().stream().filter(granularity -> !schema.hasColumn(
-          TimestampIndexGranularity.getColumnNameWithGranularity(columnName, granularity))).forEach(
-          granularity -> timestampColumnWithGranularityFieldSpecs.add(
-              TimestampIndexGranularity.getFieldSpecForTimestampColumnWithGranularity(
-                  schema.getFieldSpecFor(columnName), granularity)));
+    for (String column : columns) {
+      _indexConfigsByColName.compute(column, (key, old) -> {
+        FieldIndexConfigs.Builder builder;
+        if (old == null) {
+          builder = new FieldIndexConfigs.Builder();
+        } else {
+          builder = new FieldIndexConfigs.Builder(old);
+        }
+        return builder.add(indexType, config).build();
+      });
     }
-    if (timestampColumnWithGranularityFieldSpecs.isEmpty()) {
-      return schema;
-    }
-    Schema newSchema = schema.clone();
-    timestampColumnWithGranularityFieldSpecs.forEach(fieldSpec -> newSchema.addField(fieldSpec));
-    return newSchema;
   }
 
   public Map<String, Map<String, String>> getColumnProperties() {
-    return _columnProperties;
+    return Collections.unmodifiableMap(_columnProperties);
   }
 
   /**
@@ -263,70 +262,13 @@ public class SegmentGeneratorConfig implements Serializable {
     if (timeColumnName != null) {
       DateTimeFieldSpec dateTimeFieldSpec = schema.getSpecForTimeColumn(timeColumnName);
       if (dateTimeFieldSpec != null) {
+        _segmentTimeColumnDataType = dateTimeFieldSpec.getDataType();
         setTimeColumnName(dateTimeFieldSpec.getName());
         setDateTimeFormatSpec(dateTimeFieldSpec.getFormatSpec());
       }
     }
   }
 
-  /**
-   * Text index creation info for each column is specified
-   * using {@link FieldConfig} model of indicating per column
-   * encoding and indexing information. Since SegmentGeneratorConfig
-   * is created from TableConfig, we extract the text index info
-   * from fieldConfigList in TableConfig.
-   * @param tableConfig table config
-   */
-  private void extractTextIndexColumnsFromTableConfig(TableConfig tableConfig) {
-    List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
-    if (fieldConfigList != null) {
-      for (FieldConfig fieldConfig : fieldConfigList) {
-        if (fieldConfig.getIndexType() == FieldConfig.IndexType.TEXT) {
-          _textIndexCreationColumns.add(fieldConfig.getName());
-        }
-      }
-    }
-  }
-
-  private void extractFSTIndexColumnsFromTableConfig(TableConfig tableConfig) {
-    List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
-    if (fieldConfigList != null) {
-      for (FieldConfig fieldConfig : fieldConfigList) {
-        if (fieldConfig.getIndexType() == FieldConfig.IndexType.FST) {
-          _fstIndexCreationColumns.add(fieldConfig.getName());
-        }
-      }
-    }
-  }
-
-  private void extractH3IndexConfigsFromTableConfig(TableConfig tableConfig) {
-    List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
-    if (fieldConfigList != null) {
-      for (FieldConfig fieldConfig : fieldConfigList) {
-        if (fieldConfig.getIndexType() == FieldConfig.IndexType.H3) {
-          //noinspection ConstantConditions
-          _h3IndexConfigs.put(fieldConfig.getName(), new H3IndexConfig(fieldConfig.getProperties()));
-        }
-      }
-    }
-  }
-
-  public static Map<String, List<TimestampIndexGranularity>> extractTimestampIndexConfigsFromTableConfig(
-      TableConfig tableConfig) {
-    if (tableConfig == null) {
-      return Collections.emptyMap();
-    }
-    List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
-    Map<String, List<TimestampIndexGranularity>> timestampIndexConfigs = new HashMap<>();
-    if (CollectionUtils.isNotEmpty(fieldConfigList)) {
-      fieldConfigList.stream()
-          .filter(fieldConfig -> fieldConfig.getIndexTypes().contains(FieldConfig.IndexType.TIMESTAMP))
-          .forEach(fieldConfig -> timestampIndexConfigs.put(fieldConfig.getName(),
-              fieldConfig.getTimestampConfig().getGranularities()));
-    }
-    return timestampIndexConfigs;
-  }
-
   private void extractCompressionCodecConfigsFromTableConfig(TableConfig tableConfig) {
     List<FieldConfig> fieldConfigList = tableConfig.getFieldConfigList();
     if (fieldConfigList != null) {
@@ -334,15 +276,15 @@ public class SegmentGeneratorConfig implements Serializable {
         if (fieldConfig.getEncodingType() == FieldConfig.EncodingType.RAW
             && fieldConfig.getCompressionCodec() != null) {
           _rawIndexCreationColumns.add(fieldConfig.getName());
-          _rawIndexCompressionType
-              .put(fieldConfig.getName(), ChunkCompressionType.valueOf(fieldConfig.getCompressionCodec().name()));
+          _rawIndexCompressionType.put(fieldConfig.getName(),
+              ChunkCompressionType.valueOf(fieldConfig.getCompressionCodec().name()));
         }
       }
     }
   }
 
   public Map<String, String> getCustomProperties() {
-    return _customProperties;
+    return Collections.unmodifiableMap(_customProperties);
   }
 
   public void setCustomProperties(Map<String, String> properties) {
@@ -375,119 +317,55 @@ public class SegmentGeneratorConfig implements Serializable {
   }
 
   public Set<String> getRawIndexCreationColumns() {
-    return _rawIndexCreationColumns;
+    return Collections.unmodifiableSet(_rawIndexCreationColumns);
   }
 
-  public List<String> getInvertedIndexCreationColumns() {
-    return _invertedIndexCreationColumns;
-  }
-
-  public List<String> getBloomFilterCreationColumns() {
-    return _bloomFilterCreationColumns;
-  }
-
-  public List<String> getRangeIndexCreationColumns() {
-    return _rangeIndexCreationColumns;
-  }
-
-  /**
-   * Used by org.apache.pinot.core.segment.creator.impl.SegmentColumnarIndexCreator
-   * to get the list of text index columns.
-   * @return list of text index columns.
-   */
-  public List<String> getTextIndexCreationColumns() {
-    return _textIndexCreationColumns;
-  }
-
-  public List<String> getFSTIndexCreationColumns() {
-    return _fstIndexCreationColumns;
-  }
-
-  public List<String> getJsonIndexCreationColumns() {
-    return _jsonIndexCreationColumns;
-  }
-
-  public Map<String, H3IndexConfig> getH3IndexConfigs() {
-    return _h3IndexConfigs;
-  }
-
-  public Map<String, List<TimestampIndexGranularity>> getTimestampIndexConfigs() {
-    return _timestampIndexConfigs;
+  public void addInvertedIndexCreationColumns(Collection<String> newColumns) {
+    setIndexOn(StandardIndexes.inverted(), IndexConfig.ENABLED, newColumns);
   }
 
   public List<String> getColumnSortOrder() {
-    return _columnSortOrder;
+    return Collections.unmodifiableList(_columnSortOrder);
   }
 
+  /**
+   * Even when this method looks like a setter, it is in fact an adder.
+   */
   public void setRawIndexCreationColumns(List<String> rawIndexCreationColumns) {
     Preconditions.checkNotNull(rawIndexCreationColumns);
     _rawIndexCreationColumns.addAll(rawIndexCreationColumns);
   }
 
-  // NOTE: Should always be extracted from the table config
-  @Deprecated
-  public void setInvertedIndexCreationColumns(List<String> indexCreationColumns) {
-    Preconditions.checkNotNull(indexCreationColumns);
-    _invertedIndexCreationColumns.addAll(indexCreationColumns);
-  }
-
-  /**
-   * Used by org.apache.pinot.core.realtime.converter.RealtimeSegmentConverter
-   * and text search functional tests
-   * @param textIndexCreationColumns list of columns with text index creation enabled
-   */
-  public void setTextIndexCreationColumns(List<String> textIndexCreationColumns) {
-    if (textIndexCreationColumns != null) {
-      _textIndexCreationColumns.addAll(textIndexCreationColumns);
+  @VisibleForTesting
+  public void setRangeIndexCreationColumns(List<String> rangeIndexCreationColumns) {
+    if (rangeIndexCreationColumns != null) {
+      setIndexOn(StandardIndexes.range(), RangeIndexConfig.DEFAULT, rangeIndexCreationColumns);
     }
   }
 
   @VisibleForTesting
-  public void setColumnProperties(Map<String, Map<String, String>> columnProperties) {
-    _columnProperties = columnProperties;
-  }
-
-  public void setFSTIndexCreationColumns(List<String> fstIndexCreationColumns) {
-    if (fstIndexCreationColumns != null) {
-      _fstIndexCreationColumns.addAll(fstIndexCreationColumns);
+  public void setForwardIndexDisabledColumns(List<String> forwardIndexDisabledColumns) {
+    if (forwardIndexDisabledColumns != null) {
+      setIndexOn(StandardIndexes.forward(), ForwardIndexConfig.DISABLED, forwardIndexDisabledColumns);
     }
   }
 
+  /**
+   * Even when this method looks like a setter, it is in fact an adder.
+   */
   public void setColumnSortOrder(List<String> sortOrder) {
     Preconditions.checkNotNull(sortOrder);
     _columnSortOrder.addAll(sortOrder);
   }
 
   public List<String> getVarLengthDictionaryColumns() {
-    return _varLengthDictionaryColumns;
+    return Collections.unmodifiableList(_varLengthDictionaryColumns);
   }
 
   public void setVarLengthDictionaryColumns(List<String> varLengthDictionaryColumns) {
     _varLengthDictionaryColumns = varLengthDictionaryColumns;
   }
 
-  public void createInvertedIndexForColumn(String column) {
-    Preconditions.checkNotNull(column);
-    if (_schema != null && _schema.getFieldSpecFor(column) == null) {
-      LOGGER.warn("Cannot find column {} in schema, will not create inverted index.", column);
-      return;
-    }
-    if (_schema == null) {
-      LOGGER.warn("Schema has not been set, column {} might not exist in schema after all.", column);
-    }
-    _invertedIndexCreationColumns.add(column);
-  }
-
-  public void createInvertedIndexForAllColumns() {
-    if (_schema == null) {
-      LOGGER.warn("Schema has not been set, will not create inverted index for all columns.");
-      return;
-    }
-    for (FieldSpec spec : _schema.getAllFieldSpecs()) {
-      _invertedIndexCreationColumns.add(spec.getName());
-    }
-  }
-
   public String getInputFilePath() {
     return _inputFilePath;
   }
@@ -495,7 +373,7 @@ public class SegmentGeneratorConfig implements Serializable {
   public void setInputFilePath(String inputFilePath) {
     Preconditions.checkNotNull(inputFilePath);
     File inputFile = new File(inputFilePath);
-    Preconditions.checkState(inputFile.exists(), "Input path {} does not exist.", inputFilePath);
+    Preconditions.checkState(inputFile.exists(), "Input path %s does not exist.", inputFilePath);
     _inputFilePath = inputFile.getAbsolutePath();
   }
 
@@ -640,30 +518,21 @@ public class SegmentGeneratorConfig implements Serializable {
     _segmentVersion = segmentVersion;
   }
 
-  public Schema getSchema() {
-    return _schema;
-  }
-
+  /**
+   * Returns the {@link TableConfig} that was used to initialize this object.
+   *
+   * Remember that this object is mutable. Therefore it may have modified since the object was created. Changes on this
+   * object may or may not modify the initial table config, so the object returned by this method may not contain the
+   * same information stored on this SegmentGeneratorConfig. For example, if someone called
+   * {@link #setFSTIndexType(FSTType)} on the SegmentGeneratorConfig, the TableConfig returned by this method
+   * will not be modified accordingly.
+   */
   public TableConfig getTableConfig() {
     return _tableConfig;
   }
 
-  private void setSchema(Schema schema) {
-    Preconditions.checkNotNull(schema);
-    _schema = schema;
-
-    // Remove inverted index columns not in schema
-    // TODO: add a validate() method to perform all validations
-    if (_invertedIndexCreationColumns != null) {
-      Iterator<String> iterator = _invertedIndexCreationColumns.iterator();
-      while (iterator.hasNext()) {
-        String column = iterator.next();
-        if (_schema.getFieldSpecFor(column) == null) {
-          LOGGER.warn("Cannot find column {} in schema, will not create inverted index.", column);
-          iterator.remove();
-        }
-      }
-    }
+  public Schema getSchema() {
+    return _schema;
   }
 
   public RecordReaderConfig getReaderConfig() {
@@ -676,7 +545,10 @@ public class SegmentGeneratorConfig implements Serializable {
 
   @Nullable
   public List<StarTreeIndexConfig> getStarTreeIndexConfigs() {
-    return _starTreeIndexConfigs;
+    if (_starTreeIndexConfigs == null) {
+      return null;
+    }
+    return Collections.unmodifiableList(_starTreeIndexConfigs);
   }
 
   public void setStarTreeIndexConfigs(List<StarTreeIndexConfig> starTreeIndexConfigs) {
@@ -695,14 +567,36 @@ public class SegmentGeneratorConfig implements Serializable {
     if (_segmentNameGenerator != null) {
       return _segmentNameGenerator;
     }
+
+    String segmentNameGeneratorType = inferSegmentNameGeneratorType();
+    switch (segmentNameGeneratorType) {
+      case BatchConfigProperties.SegmentNameGeneratorType.FIXED:
+        return new FixedSegmentNameGenerator(_segmentName);
+      case BatchConfigProperties.SegmentNameGeneratorType.NORMALIZED_DATE:
+        return new NormalizedDateSegmentNameGenerator(_rawTableName, _segmentNamePrefix, false,
+            IngestionConfigUtils.getBatchSegmentIngestionType(_tableConfig),
+            IngestionConfigUtils.getBatchSegmentIngestionFrequency(_tableConfig), _dateTimeFormatSpec,
+            _segmentNamePostfix);
+      default:
+        return new SimpleSegmentNameGenerator(_segmentNamePrefix != null ? _segmentNamePrefix : _rawTableName,
+            _segmentNamePostfix);
+    }
+  }
+
+  /**
+   * Infers the segment name generator type based on segment generator config properties. Will default to simple
+   * SegmentNameGeneratorType.
+   */
+  public String inferSegmentNameGeneratorType() {
     if (_segmentName != null) {
-      return new FixedSegmentNameGenerator(_segmentName);
+      return BatchConfigProperties.SegmentNameGeneratorType.FIXED;
     }
-    if (_segmentNamePrefix != null) {
-      return new SimpleSegmentNameGenerator(_segmentNamePrefix, _segmentNamePostfix);
-    } else {
-      return new SimpleSegmentNameGenerator(_rawTableName, _segmentNamePostfix);
+
+    if (_segmentTimeColumnDataType == FieldSpec.DataType.STRING && _timeColumnType == TimeColumnType.SIMPLE_DATE) {
+      return BatchConfigProperties.SegmentNameGeneratorType.NORMALIZED_DATE;
     }
+
+    return BatchConfigProperties.SegmentNameGeneratorType.SIMPLE;
   }
 
   public void setSegmentNameGenerator(SegmentNameGenerator segmentNameGenerator) {
@@ -718,15 +612,15 @@ public class SegmentGeneratorConfig implements Serializable {
   }
 
   public boolean isSkipTimeValueCheck() {
-    return _skipTimeValueCheck;
+    return !_segmentTimeValueCheck;
   }
 
   public void setSkipTimeValueCheck(boolean skipTimeValueCheck) {
-    _skipTimeValueCheck = skipTimeValueCheck;
+    _segmentTimeValueCheck = !skipTimeValueCheck;
   }
 
   public Map<String, ChunkCompressionType> getRawIndexCompressionType() {
-    return _rawIndexCompressionType;
+    return Collections.unmodifiableMap(_rawIndexCompressionType);
   }
 
   public void setRawIndexCompressionType(Map<String, ChunkCompressionType> rawIndexCompressionType) {
@@ -784,6 +678,38 @@ public class SegmentGeneratorConfig implements Serializable {
     _nullHandlingEnabled = nullHandlingEnabled;
   }
 
+  public boolean isContinueOnError() {
+    return _continueOnError;
+  }
+
+  public void setContinueOnError(boolean continueOnError) {
+    _continueOnError = continueOnError;
+  }
+
+  public boolean isRowTimeValueCheck() {
+    return _rowTimeValueCheck;
+  }
+
+  public void setRowTimeValueCheck(boolean rowTimeValueCheck) {
+    _rowTimeValueCheck = rowTimeValueCheck;
+  }
+
+  public boolean isSegmentTimeValueCheck() {
+    return _segmentTimeValueCheck;
+  }
+
+  public void setSegmentTimeValueCheck(boolean segmentTimeValueCheck) {
+    _segmentTimeValueCheck = segmentTimeValueCheck;
+  }
+
+  public boolean isOptimizeDictionary() {
+    return _optimizeDictionary;
+  }
+
+  public void setOptimizeDictionary(boolean optimizeDictionary) {
+    _optimizeDictionary = optimizeDictionary;
+  }
+
   public boolean isOptimizeDictionaryForMetrics() {
     return _optimizeDictionaryForMetrics;
   }
@@ -815,4 +741,8 @@ public class SegmentGeneratorConfig implements Serializable {
   public void setSegmentZKPropsConfig(SegmentZKPropsConfig segmentZKPropsConfig) {
     _segmentZKPropsConfig = segmentZKPropsConfig;
   }
+
+  public Map<String, FieldIndexConfigs> getIndexConfigsByColName() {
+    return _indexConfigsByColName;
+  }
 }
