@@ -18,10 +18,15 @@
  */
 package org.apache.pinot.segment.spi.index.reader;
 
-import java.io.Closeable;
 import java.math.BigDecimal;
+import java.util.List;
 import javax.annotation.Nullable;
+import lombok.AllArgsConstructor;
+import lombok.EqualsAndHashCode;
+import lombok.Getter;
 import org.apache.pinot.segment.spi.compression.ChunkCompressionType;
+import org.apache.pinot.segment.spi.compression.DictIdCompressionType;
+import org.apache.pinot.segment.spi.index.IndexReader;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
 import org.apache.pinot.spi.utils.BigDecimalUtils;
 
@@ -31,7 +36,7 @@ import org.apache.pinot.spi.utils.BigDecimalUtils;
  *
  * @param <T> Type of the ReaderContext
  */
-public interface ForwardIndexReader<T extends ForwardIndexReaderContext> extends Closeable {
+public interface ForwardIndexReader<T extends ForwardIndexReaderContext> extends IndexReader {
 
   /**
    * Returns {@code true} if the forward index is dictionary-encoded, {@code false} if it is raw.
@@ -53,12 +58,20 @@ public interface ForwardIndexReader<T extends ForwardIndexReaderContext> extends
   /**
    * Returns the compression type (if valid). Only valid for RAW forward index columns implemented in
    * BaseChunkForwardIndexReader.
-   * @return
    */
+  @Nullable
   default ChunkCompressionType getCompressionType() {
     return null;
   }
 
+  /**
+   * Returns the compression type for dictionary encoded forward index.
+   */
+  @Nullable
+  default DictIdCompressionType getDictIdCompressionType() {
+    return null;
+  }
+
   /**
    * Returns the length of the longest entry. Only valid for RAW forward index columns implemented in
    * BaseChunkForwardIndexReader. Returns -1 otherwise.
@@ -916,4 +929,77 @@ public interface ForwardIndexReader<T extends ForwardIndexReaderContext> extends
   default int getNumValuesMV(int docId, T context) {
     throw new UnsupportedOperationException();
   }
+
+  // Functions for recording absolute buffer byte ranges accessed while reading a given docId
+
+  /**
+   * Returns whether the forward index supports recording the byte ranges accessed while reading a given docId.
+   * For readers that do support this info, caller should check if the buffer is a {@link isFixedOffsetMappingType()}.
+   * If yes, the byte range mapping for a docId can be calculated using the {@link getRawDataStartOffset()} and the
+   * {@link getDocLength()} functions.
+   * if not, caller should use the {@link recordDocIdByteRanges()} function to get the list of byte ranges accessed
+   * for a docId.
+   */
+  default boolean isBufferByteRangeInfoSupported() {
+    return false;
+  }
+
+  /**
+   * Returns a list of {@link ByteRange} that represents all the distinct
+   * buffer byte ranges (absolute offset, sizeInBytes) that are accessed when reading the given (@param docId}
+   * @param docId to find the range for
+   * @param context Reader context
+   * @param ranges List of {@link ByteRange} to which the applicable value ranges will be added
+   */
+  default void recordDocIdByteRanges(int docId, T context, List<ByteRange> ranges) {
+    throw new UnsupportedOperationException();
+  }
+
+  /**
+   * Returns whether the forward index is of fixed length type, and therefore the docId -> byte range mapping is fixed
+   * @return true if forward index has a fixed mapping of docId -> buffer offsets
+   * (eg: FixedBitSVForwardIndexReader, FixedByteChunkSVForwardIndexReader (if buffer is uncompressed) etc), false
+   * otherwise
+   */
+  default boolean isFixedOffsetMappingType() {
+    throw new UnsupportedOperationException();
+  }
+
+  /**
+   * Returns the base offset of raw data start within the fwd index buffer, if it's of fixed offset mapping type
+   * @return raw data start offset if the reader is of fixed offset mapping type
+   */
+  default long getRawDataStartOffset() {
+    throw new UnsupportedOperationException();
+  }
+
+  /**
+   * Returns the length of each entry in the forward index, if it's of fixed offset mapping type
+   */
+  default int getDocLength() {
+    throw new UnsupportedOperationException();
+  }
+
+  /**
+   * Returns whether the length of each entry in the forward index is in bits, if it's of fixed offset mapping type
+   */
+  default boolean isDocLengthInBits() {
+    return false;
+  }
+
+  /**
+   * This class represents the buffer byte ranges accessed while reading a given docId.
+   */
+  @AllArgsConstructor
+  @EqualsAndHashCode
+  @Getter
+  class ByteRange {
+    private final long _offset;
+    private final int _sizeInBytes;
+
+    @Override
+    public String toString() {
+      return "Range{" + "_offset=" + _offset + ", _size=" + _sizeInBytes + '}';
+    }
+  }
 }
