@@ -18,8 +18,13 @@
  */
 package org.apache.pinot.server.starter.helix;
 
+import com.google.common.collect.Maps;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
 import java.util.Optional;
-import org.apache.commons.configuration.ConfigurationException;
+import org.apache.commons.configuration2.ex.ConfigurationException;
 import org.apache.pinot.common.utils.TarGzCompressionUtils;
 import org.apache.pinot.segment.spi.loader.SegmentDirectoryLoaderRegistry;
 import org.apache.pinot.spi.config.instance.InstanceDataManagerConfig;
@@ -37,8 +42,6 @@ import static org.apache.pinot.spi.utils.CommonConstants.Server.DEFAULT_READ_MOD
 
 /**
  * The config used for HelixInstanceDataManager.
- *
- *
  */
 public class HelixInstanceDataManagerConfig implements InstanceDataManagerConfig {
   private static final Logger LOGGER = LoggerFactory.getLogger(HelixInstanceDataManagerConfig.class);
@@ -56,10 +59,6 @@ public class HelixInstanceDataManagerConfig implements InstanceDataManagerConfig
   public static final String INSTANCE_SEGMENT_TAR_DIR = "segmentTarDir";
   // Key of segment directory
   public static final String INSTANCE_BOOTSTRAP_SEGMENT_DIR = "bootstrap.segment.dir";
-  // Key of table data directory
-  public static final String kEY_OF_TABLE_DATA_DIRECTORY = "directory";
-  // Key of table data directory
-  public static final String kEY_OF_TABLE_NAME = "name";
   // Key of instance level segment read mode
   public static final String READ_MODE = "readMode";
   // Key of the segment format this server can read
@@ -68,6 +67,14 @@ public class HelixInstanceDataManagerConfig implements InstanceDataManagerConfig
   public static final String INSTANCE_RELOAD_CONSUMING_SEGMENT = "reload.consumingSegment";
   // Key of segment directory loader
   public static final String SEGMENT_DIRECTORY_LOADER = "segment.directory.loader";
+  // Prefix for upsert config
+  public static final String UPSERT_CONFIG_PREFIX = "upsert";
+  // Prefix for auth config
+  public static final String AUTH_CONFIG_PREFIX = "auth";
+  // Prefix for tier configs
+  public static final String TIER_CONFIGS_PREFIX = "tierConfigs";
+  // Key of tier names
+  public static final String TIER_NAMES = "tierNames";
 
   // Key of how many parallel realtime segments can be built.
   // A value of <= 0 indicates unlimited.
@@ -86,20 +93,15 @@ public class HelixInstanceDataManagerConfig implements InstanceDataManagerConfig
   // Key of server segment download rate limit
   // limit the rate to write download-untar stream to disk, in bytes
   // -1 for no disk write limit, 0 for limit the writing to min(untar, download) rate
-  private static final String STREAM_SEGMENT_DOWNLOAD_UNTAR_RATE_LIMIT
-      = "segment.stream.download.untar.rate.limit.bytes.per.sec";
-  private static final long DEFAULT_STREAM_SEGMENT_DOWNLOAD_UNTAR_RATE_LIMIT
-      = TarGzCompressionUtils.NO_DISK_WRITE_RATE_LIMIT;
+  private static final String STREAM_SEGMENT_DOWNLOAD_UNTAR_RATE_LIMIT =
+      "segment.stream.download.untar.rate.limit.bytes.per.sec";
+  private static final long DEFAULT_STREAM_SEGMENT_DOWNLOAD_UNTAR_RATE_LIMIT =
+      TarGzCompressionUtils.NO_DISK_WRITE_RATE_LIMIT;
 
   // Key of whether to use streamed server segment download-untar
   private static final String ENABLE_STREAM_SEGMENT_DOWNLOAD_UNTAR = "segment.stream.download.untar";
   private static final boolean DEFAULT_ENABLE_STREAM_SEGMENT_DOWNLOAD_UNTAR = false;
 
-  // Key of whether to enable split commit
-  private static final String ENABLE_SPLIT_COMMIT = "enable.split.commit";
-  // Key of whether to enable split commit end with segment metadata files.
-  private static final String ENABLE_SPLIT_COMMIT_END_WITH_METADATA = "enable.commitend.metadata";
-
   // Whether memory for realtime consuming segments should be allocated off-heap.
   private static final String REALTIME_OFFHEAP_ALLOCATION = "realtime.alloc.offheap";
   // And whether the allocation should be direct (default is to allocate via mmap)
@@ -143,181 +145,187 @@ public class HelixInstanceDataManagerConfig implements InstanceDataManagerConfig
   public static final long DEFAULT_EXTERNAL_VIEW_DROPPED_MAX_WAIT_MS = 20 * 60_000L;
   public static final long DEFAULT_EXTERNAL_VIEW_DROPPED_CHECK_INTERVAL_MS = 1_000L;
 
-  private final PinotConfiguration _instanceDataManagerConfiguration;
+  private final PinotConfiguration _serverConfig;
+  private final PinotConfiguration _upsertConfig;
+  private final PinotConfiguration _authConfig;
+  private final Map<String, Map<String, String>> _tierConfigs;
 
   public HelixInstanceDataManagerConfig(PinotConfiguration serverConfig)
       throws ConfigurationException {
-    _instanceDataManagerConfiguration = serverConfig;
+    _serverConfig = serverConfig;
 
     for (String key : serverConfig.getKeys()) {
       LOGGER.info("InstanceDataManagerConfig, key: {} , value: {}", key, serverConfig.getProperty(key));
     }
 
     checkRequiredKeys();
+
+    _authConfig = serverConfig.subset(AUTH_CONFIG_PREFIX);
+    _upsertConfig = serverConfig.subset(UPSERT_CONFIG_PREFIX);
+
+    PinotConfiguration tierConfigs = getConfig().subset(TIER_CONFIGS_PREFIX);
+    List<String> tierNames = tierConfigs.getProperty(TIER_NAMES, Collections.emptyList());
+    if (tierNames.isEmpty()) {
+      _tierConfigs = Collections.emptyMap();
+    } else {
+      _tierConfigs = Maps.newHashMapWithExpectedSize(tierNames.size());
+      for (String tierName : tierNames) {
+        Map<String, String> tierConfigMap = new HashMap<>();
+        tierConfigs.subset(tierName).toMap().forEach((k, v) -> tierConfigMap.put(k, String.valueOf(v)));
+        _tierConfigs.put(tierName, tierConfigMap);
+      }
+    }
   }
 
   private void checkRequiredKeys()
       throws ConfigurationException {
     for (String keyString : REQUIRED_KEYS) {
-      Optional.ofNullable(_instanceDataManagerConfiguration.getProperty(keyString))
+      Optional.ofNullable(_serverConfig.getProperty(keyString))
           .orElseThrow(() -> new ConfigurationException("Cannot find required key : " + keyString));
     }
   }
 
   @Override
   public PinotConfiguration getConfig() {
-    return _instanceDataManagerConfiguration;
+    return _serverConfig;
   }
 
   @Override
   public String getInstanceId() {
-    return _instanceDataManagerConfiguration.getProperty(INSTANCE_ID);
+    return _serverConfig.getProperty(INSTANCE_ID);
   }
 
   @Override
   public String getInstanceDataDir() {
-    return _instanceDataManagerConfiguration.getProperty(INSTANCE_DATA_DIR, DEFAULT_INSTANCE_DATA_DIR);
+    return _serverConfig.getProperty(INSTANCE_DATA_DIR, DEFAULT_INSTANCE_DATA_DIR);
   }
 
   @Override
   public String getConsumerDir() {
-    return _instanceDataManagerConfiguration.getProperty(CONSUMER_DIR);
+    return _serverConfig.getProperty(CONSUMER_DIR);
   }
 
   @Override
   public String getInstanceSegmentTarDir() {
-    return _instanceDataManagerConfiguration.getProperty(INSTANCE_SEGMENT_TAR_DIR, DEFAULT_INSTANCE_SEGMENT_TAR_DIR);
+    return _serverConfig.getProperty(INSTANCE_SEGMENT_TAR_DIR, DEFAULT_INSTANCE_SEGMENT_TAR_DIR);
   }
 
   @Override
   public String getInstanceBootstrapSegmentDir() {
-    return _instanceDataManagerConfiguration.getProperty(INSTANCE_BOOTSTRAP_SEGMENT_DIR);
+    return _serverConfig.getProperty(INSTANCE_BOOTSTRAP_SEGMENT_DIR);
   }
 
   @Override
   public String getSegmentStoreUri() {
-    return _instanceDataManagerConfiguration.getProperty(CONFIG_OF_SEGMENT_STORE_URI);
+    return _serverConfig.getProperty(CONFIG_OF_SEGMENT_STORE_URI);
   }
 
   @Override
   public ReadMode getReadMode() {
-    return ReadMode.valueOf(_instanceDataManagerConfiguration.getProperty(READ_MODE, DEFAULT_READ_MODE));
+    return ReadMode.valueOf(_serverConfig.getProperty(READ_MODE, DEFAULT_READ_MODE));
   }
 
   @Override
   public String getSegmentFormatVersion() {
-    return _instanceDataManagerConfiguration.getProperty(SEGMENT_FORMAT_VERSION);
-  }
-
-  @Override
-  public boolean isEnableSplitCommit() {
-    return _instanceDataManagerConfiguration.getProperty(ENABLE_SPLIT_COMMIT, true);
-  }
-
-  @Override
-  public boolean isEnableSplitCommitEndWithMetadata() {
-    return _instanceDataManagerConfiguration.getProperty(ENABLE_SPLIT_COMMIT_END_WITH_METADATA, true);
+    return _serverConfig.getProperty(SEGMENT_FORMAT_VERSION);
   }
 
   @Override
   public boolean isRealtimeOffHeapAllocation() {
-    return _instanceDataManagerConfiguration.getProperty(REALTIME_OFFHEAP_ALLOCATION, true);
+    return _serverConfig.getProperty(REALTIME_OFFHEAP_ALLOCATION, true);
   }
 
   @Override
   public boolean isDirectRealtimeOffHeapAllocation() {
-    return _instanceDataManagerConfiguration.getProperty(DIRECT_REALTIME_OFFHEAP_ALLOCATION, false);
+    return _serverConfig.getProperty(DIRECT_REALTIME_OFFHEAP_ALLOCATION, false);
   }
 
   public boolean shouldReloadConsumingSegment() {
-    return _instanceDataManagerConfiguration
-        .getProperty(INSTANCE_RELOAD_CONSUMING_SEGMENT, Server.DEFAULT_RELOAD_CONSUMING_SEGMENT);
+    return _serverConfig.getProperty(INSTANCE_RELOAD_CONSUMING_SEGMENT, Server.DEFAULT_RELOAD_CONSUMING_SEGMENT);
   }
 
   @Override
   public String getAvgMultiValueCount() {
-    return _instanceDataManagerConfiguration.getProperty(AVERAGE_MV_COUNT);
+    return _serverConfig.getProperty(AVERAGE_MV_COUNT);
   }
 
   public int getMaxParallelRefreshThreads() {
-    return _instanceDataManagerConfiguration.getProperty(MAX_PARALLEL_REFRESH_THREADS, 1);
+    return _serverConfig.getProperty(MAX_PARALLEL_REFRESH_THREADS, 1);
   }
 
   public int getMaxSegmentPreloadThreads() {
-    return _instanceDataManagerConfiguration.getProperty(MAX_SEGMENT_PRELOAD_THREADS, 0);
+    return _serverConfig.getProperty(MAX_SEGMENT_PRELOAD_THREADS, 0);
   }
 
   public int getMaxParallelSegmentBuilds() {
-    return _instanceDataManagerConfiguration
-        .getProperty(MAX_PARALLEL_SEGMENT_BUILDS, DEFAULT_MAX_PARALLEL_SEGMENT_BUILDS);
+    return _serverConfig.getProperty(MAX_PARALLEL_SEGMENT_BUILDS, DEFAULT_MAX_PARALLEL_SEGMENT_BUILDS);
   }
 
   @Override
   public int getMaxParallelSegmentDownloads() {
-    return _instanceDataManagerConfiguration.getProperty(MAX_PARALLEL_SEGMENT_DOWNLOADS,
-        DEFAULT_MAX_PARALLEL_SEGMENT_DOWNLOADS);
+    return _serverConfig.getProperty(MAX_PARALLEL_SEGMENT_DOWNLOADS, DEFAULT_MAX_PARALLEL_SEGMENT_DOWNLOADS);
   }
 
   public String getSegmentDirectoryLoader() {
-    return _instanceDataManagerConfiguration.getProperty(SEGMENT_DIRECTORY_LOADER,
+    return _serverConfig.getProperty(SEGMENT_DIRECTORY_LOADER,
         SegmentDirectoryLoaderRegistry.DEFAULT_SEGMENT_DIRECTORY_LOADER_NAME);
   }
 
   @Override
   public long getErrorCacheSize() {
-    return _instanceDataManagerConfiguration.getProperty(ERROR_CACHE_SIZE, DEFAULT_ERROR_CACHE_SIZE);
+    return _serverConfig.getProperty(ERROR_CACHE_SIZE, DEFAULT_ERROR_CACHE_SIZE);
   }
 
   @Override
   public boolean isStreamSegmentDownloadUntar() {
-    return _instanceDataManagerConfiguration.getProperty(ENABLE_STREAM_SEGMENT_DOWNLOAD_UNTAR,
+    return _serverConfig.getProperty(ENABLE_STREAM_SEGMENT_DOWNLOAD_UNTAR,
         DEFAULT_ENABLE_STREAM_SEGMENT_DOWNLOAD_UNTAR);
   }
 
   @Override
   public long getStreamSegmentDownloadUntarRateLimit() {
-    return _instanceDataManagerConfiguration.getProperty(STREAM_SEGMENT_DOWNLOAD_UNTAR_RATE_LIMIT,
+    return _serverConfig.getProperty(STREAM_SEGMENT_DOWNLOAD_UNTAR_RATE_LIMIT,
         DEFAULT_STREAM_SEGMENT_DOWNLOAD_UNTAR_RATE_LIMIT);
   }
 
   @Override
   public int getDeletedSegmentsCacheSize() {
-    return _instanceDataManagerConfiguration.getProperty(DELETED_SEGMENTS_CACHE_SIZE,
-        DEFAULT_DELETED_SEGMENTS_CACHE_SIZE);
+    return _serverConfig.getProperty(DELETED_SEGMENTS_CACHE_SIZE, DEFAULT_DELETED_SEGMENTS_CACHE_SIZE);
   }
 
   @Override
   public int getDeletedSegmentsCacheTtlMinutes() {
-    return _instanceDataManagerConfiguration.getProperty(DELETED_SEGMENTS_CACHE_TTL_MINUTES,
-        DEFAULT_DELETED_SEGMENTS_CACHE_TTL_MINUTES);
+    return _serverConfig.getProperty(DELETED_SEGMENTS_CACHE_TTL_MINUTES, DEFAULT_DELETED_SEGMENTS_CACHE_TTL_MINUTES);
   }
 
   @Override
   public String getSegmentPeerDownloadScheme() {
-    return _instanceDataManagerConfiguration.getProperty(PEER_DOWNLOAD_SCHEME);
+    return _serverConfig.getProperty(PEER_DOWNLOAD_SCHEME);
   }
 
   @Override
   public long getExternalViewDroppedMaxWaitMs() {
-    return _instanceDataManagerConfiguration.getProperty(EXTERNAL_VIEW_DROPPED_MAX_WAIT_MS,
-        DEFAULT_EXTERNAL_VIEW_DROPPED_MAX_WAIT_MS);
+    return _serverConfig.getProperty(EXTERNAL_VIEW_DROPPED_MAX_WAIT_MS, DEFAULT_EXTERNAL_VIEW_DROPPED_MAX_WAIT_MS);
   }
 
   @Override
   public long getExternalViewDroppedCheckIntervalMs() {
-    return _instanceDataManagerConfiguration.getProperty(EXTERNAL_VIEW_DROPPED_CHECK_INTERVAL_MS,
+    return _serverConfig.getProperty(EXTERNAL_VIEW_DROPPED_CHECK_INTERVAL_MS,
         DEFAULT_EXTERNAL_VIEW_DROPPED_CHECK_INTERVAL_MS);
   }
 
   @Override
-  public String toString() {
-    String configString = "";
-    configString += "Instance Id: " + getInstanceId();
-    configString += "\n\tInstance Data Dir: " + getInstanceDataDir();
-    configString += "\n\tInstance Segment Tar Dir: " + getInstanceSegmentTarDir();
-    configString += "\n\tBootstrap Segment Dir: " + getInstanceBootstrapSegmentDir();
-    configString += "\n\tRead Mode: " + getReadMode();
-    configString += "\n\tSegment format version: " + getSegmentFormatVersion();
-    return configString;
+  public PinotConfiguration getUpsertConfig() {
+    return _upsertConfig;
+  }
+
+  @Override
+  public PinotConfiguration getAuthConfig() {
+    return _authConfig;
+  }
+
+  @Override
+  public Map<String, Map<String, String>> getTierConfigs() {
+    return _tierConfigs;
   }
 }
