@@ -18,11 +18,13 @@
  */
 package org.apache.pinot.spi.config.instance;
 
+import java.util.Map;
 import org.apache.pinot.spi.env.PinotConfiguration;
 import org.apache.pinot.spi.utils.ReadMode;
 
 
 public interface InstanceDataManagerConfig {
+
   PinotConfiguration getConfig();
 
   String getInstanceId();
@@ -43,10 +45,6 @@ public interface InstanceDataManagerConfig {
 
   String getAvgMultiValueCount();
 
-  boolean isEnableSplitCommit();
-
-  boolean isEnableSplitCommitEndWithMetadata();
-
   boolean isRealtimeOffHeapAllocation();
 
   boolean isDirectRealtimeOffHeapAllocation();
@@ -72,4 +70,10 @@ public interface InstanceDataManagerConfig {
   long getExternalViewDroppedMaxWaitMs();
 
   long getExternalViewDroppedCheckIntervalMs();
+
+  PinotConfiguration getUpsertConfig();
+
+  PinotConfiguration getAuthConfig();
+
+  Map<String, Map<String, String>> getTierConfigs();
 }
