@@ -31,8 +31,6 @@ import org.apache.pinot.spi.config.table.assignment.InstanceAssignmentConfig;
 import org.apache.pinot.spi.config.table.assignment.InstancePartitionsType;
 import org.apache.pinot.spi.config.table.assignment.SegmentAssignmentConfig;
 import org.apache.pinot.spi.config.table.ingestion.IngestionConfig;
-import org.apache.pinot.spi.stream.StreamConfig;
-import org.apache.pinot.spi.utils.IngestionConfigUtils;
 import org.apache.pinot.spi.utils.builder.TableNameBuilder;
 
 
@@ -394,6 +392,17 @@ public class TableConfig extends BaseJsonConfig {
     return _upsertConfig == null ? null : _upsertConfig.getDeleteRecordColumn();
   }
 
+  @JsonIgnore
+  @Nullable
+  public String getOutOfOrderRecordColumn() {
+    return _upsertConfig == null ? null : _upsertConfig.getOutOfOrderRecordColumn();
+  }
+
+  @JsonIgnore
+  public boolean isDropOutOfOrderRecord() {
+    return _upsertConfig != null && _upsertConfig.isDropOutOfOrderRecord();
+  }
+
   @JsonProperty(TUNER_CONFIG_LIST_KEY)
   public List<TunerConfig> getTunerConfigsList() {
     return _tunerConfigList;
@@ -415,27 +424,13 @@ public class TableConfig extends BaseJsonConfig {
 
   @JsonIgnore
   public int getReplication() {
-    int replication = 0;
     if (_tableType == TableType.REALTIME) {
-      StreamConfig streamConfig = new StreamConfig(_tableName, IngestionConfigUtils.getStreamConfigMap(this));
-      if (streamConfig.hasHighLevelConsumerType()) {
-        // In case of HLC, we read from "replication"
-        replication = Integer.parseInt(_validationConfig.getReplication());
-      } else {
-        // To keep the backward compatibility, we read from "replicasPerPartition" in case of LLC
-        String replicasPerPartitionStr = _validationConfig.getReplicasPerPartition();
-        try {
-          replication = Integer.parseInt(replicasPerPartitionStr);
-        } catch (NumberFormatException e) {
-          // If numReplicasPerPartition is not being used or specified, read the value from replication
-          String replicationStr = _validationConfig.getReplication();
-          replication = Integer.parseInt(replicationStr);
-        }
+      // Use replicasPerPartition for real-time table if exists
+      String replicasPerPartition = _validationConfig.getReplicasPerPartition();
+      if (replicasPerPartition != null) {
+        return Integer.parseInt(replicasPerPartition);
       }
-    } else {
-      // In case of OFFLINE tables, we read from "replication"
-      replication = Integer.parseInt(_validationConfig.getReplication());
     }
-    return replication;
+    return Integer.parseInt(_validationConfig.getReplication());
   }
 }
