@@ -57,12 +57,18 @@ public class UpsertConfig extends BaseJsonConfig {
   @JsonPropertyDescription("Boolean column to indicate whether a records should be deleted")
   private String _deleteRecordColumn;
 
+  @JsonPropertyDescription("Boolean column to indicate whether a records is out-of-order")
+  private String _outOfOrderRecordColumn;
+
   @JsonPropertyDescription("Whether to use snapshot for fast upsert metadata recovery")
   private boolean _enableSnapshot;
 
   @JsonPropertyDescription("Whether to use TTL for upsert metadata cleanup, it uses the same unit as comparison col")
   private double _metadataTTL;
 
+  @JsonPropertyDescription("TTL for upsert metadata cleanup for deleted keys, it uses the same unit as comparison col")
+  private double _deletedKeysTTL;
+
   @JsonPropertyDescription("Whether to preload segments for fast upsert metadata recovery")
   private boolean _enablePreload;
 
@@ -72,6 +78,9 @@ public class UpsertConfig extends BaseJsonConfig {
   @JsonPropertyDescription("Custom configs for upsert metadata manager")
   private Map<String, String> _metadataManagerConfigs;
 
+  @JsonPropertyDescription("Whether to drop out-of-order record")
+  private boolean _dropOutOfOrderRecord;
+
   public UpsertConfig(Mode mode) {
     _mode = mode;
   }
@@ -110,6 +119,11 @@ public class UpsertConfig extends BaseJsonConfig {
     return _deleteRecordColumn;
   }
 
+  @Nullable
+  public String getOutOfOrderRecordColumn() {
+    return _outOfOrderRecordColumn;
+  }
+
   public boolean isEnableSnapshot() {
     return _enableSnapshot;
   }
@@ -118,10 +132,18 @@ public class UpsertConfig extends BaseJsonConfig {
     return _metadataTTL;
   }
 
+  public double getDeletedKeysTTL() {
+    return _deletedKeysTTL;
+  }
+
   public boolean isEnablePreload() {
     return _enablePreload;
   }
 
+  public boolean isDropOutOfOrderRecord() {
+    return _dropOutOfOrderRecord;
+  }
+
   @Nullable
   public String getMetadataManagerClass() {
     return _metadataManagerClass;
@@ -177,9 +199,11 @@ public class UpsertConfig extends BaseJsonConfig {
   }
 
   public void setDeleteRecordColumn(String deleteRecordColumn) {
-    if (deleteRecordColumn != null) {
-      _deleteRecordColumn = deleteRecordColumn;
-    }
+    _deleteRecordColumn = deleteRecordColumn;
+  }
+
+  public void setOutOfOrderRecordColumn(String outOfOrderRecordColumn) {
+    _outOfOrderRecordColumn = outOfOrderRecordColumn;
   }
 
   public void setEnableSnapshot(boolean enableSnapshot) {
@@ -190,10 +214,18 @@ public class UpsertConfig extends BaseJsonConfig {
     _metadataTTL = metadataTTL;
   }
 
+  public void setDeletedKeysTTL(double deletedKeysTTL) {
+    _deletedKeysTTL = deletedKeysTTL;
+  }
+
   public void setEnablePreload(boolean enablePreload) {
     _enablePreload = enablePreload;
   }
 
+  public void setDropOutOfOrderRecord(boolean dropOutOfOrderRecord) {
+    _dropOutOfOrderRecord = dropOutOfOrderRecord;
+  }
+
   public void setMetadataManagerClass(String metadataManagerClass) {
     _metadataManagerClass = metadataManagerClass;
   }
