@@ -19,14 +19,13 @@
 package org.apache.pinot.spi.stream;
 
 import com.google.common.base.Preconditions;
-import java.util.ArrayList;
 import java.util.HashMap;
-import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Optional;
 import java.util.concurrent.TimeUnit;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.pinot.spi.utils.DataSizeUtils;
-import org.apache.pinot.spi.utils.EqualityUtils;
 import org.apache.pinot.spi.utils.TimeUtils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -38,14 +37,6 @@ import org.slf4j.LoggerFactory;
 public class StreamConfig {
   private static final Logger LOGGER = LoggerFactory.getLogger(StreamConfig.class);
 
-  /**
-   * The type of the stream consumer either HIGHLEVEL or LOWLEVEL. For backward compatibility, adding SIMPLE which is
-   * equivalent to LOWLEVEL
-   */
-  public enum ConsumerType {
-    HIGHLEVEL, LOWLEVEL
-  }
-
   public static final int DEFAULT_FLUSH_THRESHOLD_ROWS = 5_000_000;
   public static final long DEFAULT_FLUSH_THRESHOLD_TIME_MILLIS = TimeUnit.MILLISECONDS.convert(6, TimeUnit.HOURS);
   public static final long DEFAULT_FLUSH_THRESHOLD_SEGMENT_SIZE_BYTES = 200 * 1024 * 1024; // 200M
@@ -57,16 +48,14 @@ public class StreamConfig {
 
   public static final long DEFAULT_STREAM_CONNECTION_TIMEOUT_MILLIS = 30_000;
   public static final int DEFAULT_STREAM_FETCH_TIMEOUT_MILLIS = 5_000;
+  public static final int DEFAULT_STREAM_FETCH_TIMEOUT_MILLIS_KINESIS = 600_000;
   public static final int DEFAULT_IDLE_TIMEOUT_MILLIS = 3 * 60 * 1000;
 
-  private static final String SIMPLE_CONSUMER_TYPE_STRING = "simple";
-
   private static final double CONSUMPTION_RATE_LIMIT_NOT_SPECIFIED = -1;
 
   private final String _type;
   private final String _topicName;
   private final String _tableNameWithType;
-  private final List<ConsumerType> _consumerTypes = new ArrayList<>();
   private final String _consumerFactoryClassName;
   private final String _decoderClass;
   private final Map<String, String> _decoderProperties = new HashMap<>();
@@ -110,7 +99,6 @@ public class StreamConfig {
     _tableNameWithType = tableNameWithType;
 
     validateConsumerType(_type, streamConfigMap);
-    _consumerTypes.add(ConsumerType.LOWLEVEL);
 
     String consumerFactoryClassKey =
         StreamConfigProperties.constructStreamProperty(_type, StreamConfigProperties.STREAM_CONSUMER_FACTORY_CLASS);
@@ -136,8 +124,8 @@ public class StreamConfig {
         StreamConfigProperties.constructStreamProperty(_type, StreamConfigProperties.DECODER_PROPS_PREFIX);
     for (String key : streamConfigMap.keySet()) {
       if (key.startsWith(streamDecoderPropPrefix)) {
-        _decoderProperties
-            .put(StreamConfigProperties.getPropertySuffix(key, streamDecoderPropPrefix), streamConfigMap.get(key));
+        _decoderProperties.put(StreamConfigProperties.getPropertySuffix(key, streamDecoderPropPrefix),
+            streamConfigMap.get(key));
       }
     }
 
@@ -155,7 +143,11 @@ public class StreamConfig {
     }
     _connectionTimeoutMillis = connectionTimeoutMillis;
 
-    int fetchTimeoutMillis = DEFAULT_STREAM_FETCH_TIMEOUT_MILLIS;
+    // For Kinesis, we need to set a higher fetch timeout to avoid getting stuck in empty records loop
+    // TODO: Remove this once we have a better way to handle empty records in Kinesis
+    int fetchTimeoutMillis =
+        _consumerFactoryClassName.contains("KinesisConsumerFactory") ? DEFAULT_STREAM_FETCH_TIMEOUT_MILLIS_KINESIS
+            : DEFAULT_STREAM_FETCH_TIMEOUT_MILLIS;
     String fetchTimeoutKey =
         StreamConfigProperties.constructStreamProperty(_type, StreamConfigProperties.STREAM_FETCH_TIMEOUT_MILLIS);
     String fetchTimeoutValue = streamConfigMap.get(fetchTimeoutKey);
@@ -188,7 +180,7 @@ public class StreamConfig {
     _flushThresholdSegmentSizeBytes = extractFlushThresholdSegmentSize(streamConfigMap);
     _serverUploadToDeepStore = Boolean.parseBoolean(
         streamConfigMap.getOrDefault(StreamConfigProperties.SERVER_UPLOAD_TO_DEEPSTORE,
-        DEFAULT_SERVER_UPLOAD_TO_DEEPSTORE));
+            DEFAULT_SERVER_UPLOAD_TO_DEEPSTORE));
 
     int autotuneInitialRows = 0;
     String initialRowsValue = streamConfigMap.get(StreamConfigProperties.SEGMENT_FLUSH_AUTOTUNE_INITIAL_ROWS);
@@ -216,13 +208,15 @@ public class StreamConfig {
     String consumerTypesKey =
         StreamConfigProperties.constructStreamProperty(streamType, StreamConfigProperties.STREAM_CONSUMER_TYPES);
     String consumerTypes = streamConfigMap.get(consumerTypesKey);
-    Preconditions.checkNotNull(consumerTypes, consumerTypesKey + " cannot be null");
-    for (String consumerType : consumerTypes.split(",")) {
-      Preconditions.checkState(ConsumerType.LOWLEVEL.name().equalsIgnoreCase(consumerType)
-              || SIMPLE_CONSUMER_TYPE_STRING.equalsIgnoreCase(consumerType),
+    if (consumerTypes == null) {
+      return;
+    }
+    for (String consumerType : StringUtils.split(consumerTypes, ',')) {
+      Preconditions.checkState(!consumerType.equalsIgnoreCase("highlevel"),
           "Realtime tables with HLC consumer (consumer.type=highlevel) is no longer supported in Apache Pinot");
     }
   }
+
   public boolean isServerUploadToDeepStore() {
     return _serverUploadToDeepStore;
   }
@@ -260,6 +254,11 @@ public class StreamConfig {
       key = StreamConfigProperties.DEPRECATED_SEGMENT_FLUSH_THRESHOLD_ROWS;
       flushThresholdRowsStr = streamConfigMap.get(key);
     }
+    if (flushThresholdRowsStr == null) {
+      // for backward compatibility with older property
+      key = StreamConfigProperties.DEPRECATED_SEGMENT_FLUSH_THRESHOLD_ROWS + StreamConfigProperties.LLC_SUFFIX;
+      flushThresholdRowsStr = streamConfigMap.get(key);
+    }
     if (flushThresholdRowsStr != null) {
       try {
         int flushThresholdRows = Integer.parseInt(flushThresholdRowsStr);
@@ -267,8 +266,8 @@ public class StreamConfig {
         Preconditions.checkState(flushThresholdRows >= 0);
         return flushThresholdRows;
       } catch (Exception e) {
-        LOGGER
-            .warn("Invalid config {}: {}, defaulting to: {}", key, flushThresholdRowsStr, DEFAULT_FLUSH_THRESHOLD_ROWS);
+        LOGGER.warn("Invalid config {}: {}, defaulting to: {}", key, flushThresholdRowsStr,
+            DEFAULT_FLUSH_THRESHOLD_ROWS);
         return DEFAULT_FLUSH_THRESHOLD_ROWS;
       }
     } else {
@@ -277,7 +276,13 @@ public class StreamConfig {
   }
 
   protected long extractFlushThresholdTimeMillis(Map<String, String> streamConfigMap) {
-    String flushThresholdTimeStr = streamConfigMap.get(StreamConfigProperties.SEGMENT_FLUSH_THRESHOLD_TIME);
+    String key = StreamConfigProperties.SEGMENT_FLUSH_THRESHOLD_TIME;
+    String flushThresholdTimeStr = streamConfigMap.get(key);
+    if (flushThresholdTimeStr == null) {
+      // for backward compatibility with older property
+      key = StreamConfigProperties.SEGMENT_FLUSH_THRESHOLD_TIME + StreamConfigProperties.LLC_SUFFIX;
+      flushThresholdTimeStr = streamConfigMap.get(key);
+    }
     if (flushThresholdTimeStr != null) {
       try {
         return TimeUtils.convertPeriodToMillis(flushThresholdTimeStr);
@@ -286,8 +291,8 @@ public class StreamConfig {
           // For backward-compatibility, parse it as milliseconds value
           return Long.parseLong(flushThresholdTimeStr);
         } catch (NumberFormatException nfe) {
-          LOGGER.warn("Invalid config {}: {}, defaulting to: {}", StreamConfigProperties.SEGMENT_FLUSH_THRESHOLD_TIME,
-              flushThresholdTimeStr, DEFAULT_FLUSH_THRESHOLD_TIME_MILLIS);
+          LOGGER.warn("Invalid config {}: {}, defaulting to: {}", key, flushThresholdTimeStr,
+              DEFAULT_FLUSH_THRESHOLD_TIME_MILLIS);
           return DEFAULT_FLUSH_THRESHOLD_TIME_MILLIS;
         }
       }
@@ -304,16 +309,14 @@ public class StreamConfig {
     return _topicName;
   }
 
-  public List<ConsumerType> getConsumerTypes() {
-    return _consumerTypes;
-  }
-
+  @Deprecated
   public boolean hasHighLevelConsumerType() {
-    return _consumerTypes.contains(ConsumerType.HIGHLEVEL);
+    return false;
   }
 
+  @Deprecated
   public boolean hasLowLevelConsumerType() {
-    return _consumerTypes.contains(ConsumerType.LOWLEVEL);
+    return true;
   }
 
   public String getConsumerFactoryClassName() {
@@ -383,67 +386,46 @@ public class StreamConfig {
 
   @Override
   public String toString() {
-    return "StreamConfig{" + "_type='" + _type + '\'' + ", _topicName='" + _topicName + '\'' + ", _consumerTypes="
-        + _consumerTypes + ", _consumerFactoryClassName='" + _consumerFactoryClassName + '\'' + ", _offsetCriteria='"
-        + _offsetCriteria + '\'' + ", _connectionTimeoutMillis=" + _connectionTimeoutMillis + ", _fetchTimeoutMillis="
-        + _fetchTimeoutMillis + ", _idleTimeoutMillis=" + _idleTimeoutMillis + ", _flushThresholdRows="
-        + _flushThresholdRows + ", _flushThresholdTimeMillis=" + _flushThresholdTimeMillis
-        + ", _flushSegmentDesiredSizeBytes=" + _flushThresholdSegmentSizeBytes + ", _flushAutotuneInitialRows="
-        + _flushAutotuneInitialRows + ", _decoderClass='" + _decoderClass + '\'' + ", _decoderProperties="
-        + _decoderProperties + ", _groupId='" + _groupId + "', _topicConsumptionRateLimit=" + _topicConsumptionRateLimit
-        + ", _tableNameWithType='" + _tableNameWithType + ", _serverUploadToDeepStore=" + _serverUploadToDeepStore
-        + "}";
+    return "StreamConfig{" + "_type='" + _type + '\'' + ", _topicName='" + _topicName + '\'' + ", _tableNameWithType='"
+        + _tableNameWithType + '\'' + ", _consumerFactoryClassName='" + _consumerFactoryClassName + '\''
+        + ", _decoderClass='" + _decoderClass + '\'' + ", _decoderProperties=" + _decoderProperties
+        + ", _connectionTimeoutMillis=" + _connectionTimeoutMillis + ", _fetchTimeoutMillis=" + _fetchTimeoutMillis
+        + ", _idleTimeoutMillis=" + _idleTimeoutMillis + ", _flushThresholdRows=" + _flushThresholdRows
+        + ", _flushThresholdTimeMillis=" + _flushThresholdTimeMillis + ", _flushThresholdSegmentSizeBytes="
+        + _flushThresholdSegmentSizeBytes + ", _flushAutotuneInitialRows=" + _flushAutotuneInitialRows + ", _groupId='"
+        + _groupId + '\'' + ", _topicConsumptionRateLimit=" + _topicConsumptionRateLimit + ", _streamConfigMap="
+        + _streamConfigMap + ", _offsetCriteria=" + _offsetCriteria + ", _serverUploadToDeepStore="
+        + _serverUploadToDeepStore + '}';
   }
 
   @Override
   public boolean equals(Object o) {
-    if (EqualityUtils.isSameReference(this, o)) {
+    if (this == o) {
       return true;
     }
-
-    if (EqualityUtils.isNullOrNotSameClass(this, o)) {
+    if (o == null || getClass() != o.getClass()) {
       return false;
     }
-
     StreamConfig that = (StreamConfig) o;
-
-    return EqualityUtils.isEqual(_connectionTimeoutMillis, that._connectionTimeoutMillis) && EqualityUtils.isEqual(
-        _fetchTimeoutMillis, that._fetchTimeoutMillis) && EqualityUtils.isEqual(_idleTimeoutMillis,
-        that._idleTimeoutMillis) && EqualityUtils.isEqual(_flushThresholdRows, that._flushThresholdRows)
-        && EqualityUtils.isEqual(_flushThresholdTimeMillis, that._flushThresholdTimeMillis) && EqualityUtils.isEqual(
-        _flushThresholdSegmentSizeBytes, that._flushThresholdSegmentSizeBytes) && EqualityUtils.isEqual(
-        _flushAutotuneInitialRows, that._flushAutotuneInitialRows) && EqualityUtils.isEqual(_type, that._type)
-        && EqualityUtils.isEqual(_topicName, that._topicName) && EqualityUtils.isEqual(_consumerTypes,
-        that._consumerTypes) && EqualityUtils.isEqual(_consumerFactoryClassName, that._consumerFactoryClassName)
-        && EqualityUtils.isEqual(_offsetCriteria, that._offsetCriteria) && EqualityUtils.isEqual(_decoderClass,
-        that._decoderClass) && EqualityUtils.isEqual(_decoderProperties, that._decoderProperties)
-        && EqualityUtils.isEqual(_groupId, that._groupId) && EqualityUtils.isEqual(_tableNameWithType,
-        that._tableNameWithType) && EqualityUtils.isEqual(_topicConsumptionRateLimit, that._topicConsumptionRateLimit)
-        && EqualityUtils.isEqual(_streamConfigMap, that._streamConfigMap)
-        && _serverUploadToDeepStore == that._serverUploadToDeepStore;
+    return _connectionTimeoutMillis == that._connectionTimeoutMillis && _fetchTimeoutMillis == that._fetchTimeoutMillis
+        && _idleTimeoutMillis == that._idleTimeoutMillis && _flushThresholdRows == that._flushThresholdRows
+        && _flushThresholdTimeMillis == that._flushThresholdTimeMillis
+        && _flushThresholdSegmentSizeBytes == that._flushThresholdSegmentSizeBytes
+        && _flushAutotuneInitialRows == that._flushAutotuneInitialRows
+        && Double.compare(_topicConsumptionRateLimit, that._topicConsumptionRateLimit) == 0
+        && _serverUploadToDeepStore == that._serverUploadToDeepStore && Objects.equals(_type, that._type)
+        && Objects.equals(_topicName, that._topicName) && Objects.equals(_tableNameWithType, that._tableNameWithType)
+        && Objects.equals(_consumerFactoryClassName, that._consumerFactoryClassName) && Objects.equals(_decoderClass,
+        that._decoderClass) && Objects.equals(_decoderProperties, that._decoderProperties) && Objects.equals(_groupId,
+        that._groupId) && Objects.equals(_streamConfigMap, that._streamConfigMap) && Objects.equals(_offsetCriteria,
+        that._offsetCriteria);
   }
 
   @Override
   public int hashCode() {
-    int result = EqualityUtils.hashCodeOf(_type);
-    result = EqualityUtils.hashCodeOf(result, _topicName);
-    result = EqualityUtils.hashCodeOf(result, _consumerTypes);
-    result = EqualityUtils.hashCodeOf(result, _consumerFactoryClassName);
-    result = EqualityUtils.hashCodeOf(result, _offsetCriteria);
-    result = EqualityUtils.hashCodeOf(result, _connectionTimeoutMillis);
-    result = EqualityUtils.hashCodeOf(result, _fetchTimeoutMillis);
-    result = EqualityUtils.hashCodeOf(result, _idleTimeoutMillis);
-    result = EqualityUtils.hashCodeOf(result, _flushThresholdRows);
-    result = EqualityUtils.hashCodeOf(result, _flushThresholdTimeMillis);
-    result = EqualityUtils.hashCodeOf(result, _flushThresholdSegmentSizeBytes);
-    result = EqualityUtils.hashCodeOf(result, _flushAutotuneInitialRows);
-    result = EqualityUtils.hashCodeOf(result, _decoderClass);
-    result = EqualityUtils.hashCodeOf(result, _decoderProperties);
-    result = EqualityUtils.hashCodeOf(result, _groupId);
-    result = EqualityUtils.hashCodeOf(result, _topicConsumptionRateLimit);
-    result = EqualityUtils.hashCodeOf(result, _streamConfigMap);
-    result = EqualityUtils.hashCodeOf(result, _tableNameWithType);
-    result = EqualityUtils.hashCodeOf(result, _serverUploadToDeepStore);
-    return result;
+    return Objects.hash(_type, _topicName, _tableNameWithType, _consumerFactoryClassName, _decoderClass,
+        _decoderProperties, _connectionTimeoutMillis, _fetchTimeoutMillis, _idleTimeoutMillis, _flushThresholdRows,
+        _flushThresholdTimeMillis, _flushThresholdSegmentSizeBytes, _flushAutotuneInitialRows, _groupId,
+        _topicConsumptionRateLimit, _streamConfigMap, _offsetCriteria, _serverUploadToDeepStore);
   }
 }
