@@ -74,7 +74,10 @@ public class JsonUtils {
   public static final String KEY_SEPARATOR = ".";
   public static final String ARRAY_PATH = "[*]";
   public static final String ARRAY_INDEX_KEY = ".$index";
+  public static final String SKIPPED_VALUE_REPLACEMENT = "$SKIPPED$";
   public static final int MAX_COMBINATIONS = 100_000;
+  private static final List<Map<String, String>> SKIPPED_FLATTENED_RECORD =
+      Collections.singletonList(Collections.singletonMap(VALUE_KEY, SKIPPED_VALUE_REPLACEMENT));
 
   // For querying
   public static final String WILDCARD = "*";
@@ -355,7 +358,7 @@ public class JsonUtils {
    * ]
    * </pre>
    */
-  public static List<Map<String, String>> flatten(JsonNode node, JsonIndexConfig jsonIndexConfig) {
+  protected static List<Map<String, String>> flatten(JsonNode node, JsonIndexConfig jsonIndexConfig) {
     try {
       return flatten(node, jsonIndexConfig, 0, "$", false);
     } catch (OutOfMemoryError oom) {
@@ -377,7 +380,12 @@ public class JsonUtils {
 
     // Value
     if (node.isValueNode()) {
-      return Collections.singletonList(Collections.singletonMap(VALUE_KEY, node.asText()));
+      String valueAsText = node.asText();
+      int maxValueLength = jsonIndexConfig.getMaxValueLength();
+      if (0 < maxValueLength && maxValueLength < valueAsText.length()) {
+        valueAsText = SKIPPED_VALUE_REPLACEMENT;
+      }
+      return Collections.singletonList(Collections.singletonMap(VALUE_KEY, valueAsText));
     }
 
     Preconditions.checkArgument(node.isArray() || node.isObject(), "Unexpected node type: %s", node.getNodeType());
@@ -641,7 +649,7 @@ public class JsonUtils {
             fieldTypeMap, timeUnit, fieldsToUnnest, delimiter, collectionNotUnnestedToJson);
       }
     } else {
-      throw new IllegalArgumentException(String.format("Unsupported json node type", jsonNode.getClass()));
+      throw new IllegalArgumentException(String.format("Unsupported json node type for class %s", jsonNode.getClass()));
     }
   }
 
@@ -713,4 +721,19 @@ public class JsonUtils {
       }
     }
   }
+
+  public static List<Map<String, String>> flatten(String jsonString, JsonIndexConfig jsonIndexConfig)
+      throws IOException {
+    JsonNode jsonNode;
+    try {
+      jsonNode = JsonUtils.stringToJsonNode(jsonString);
+    } catch (JsonProcessingException e) {
+      if (jsonIndexConfig.getSkipInvalidJson()) {
+        return SKIPPED_FLATTENED_RECORD;
+      } else {
+        throw e;
+      }
+    }
+    return JsonUtils.flatten(jsonNode, jsonIndexConfig);
+  }
 }
