@@ -20,23 +20,17 @@ package org.apache.pinot.tools.admin.command;
 
 import java.io.File;
 import java.io.IOException;
-import java.util.HashMap;
-import java.util.LinkedList;
 import java.util.List;
-import java.util.Map;
 import java.util.concurrent.TimeUnit;
 import org.apache.commons.lang.math.IntRange;
+import org.apache.pinot.controller.recommender.data.DataGenerationHelpers;
 import org.apache.pinot.controller.recommender.data.generator.DataGenerator;
 import org.apache.pinot.controller.recommender.data.generator.DataGeneratorSpec;
 import org.apache.pinot.controller.recommender.data.generator.SchemaAnnotation;
-import org.apache.pinot.spi.data.FieldSpec;
 import org.apache.pinot.spi.data.FieldSpec.DataType;
-import org.apache.pinot.spi.data.FieldSpec.FieldType;
 import org.apache.pinot.spi.data.Schema;
 import org.apache.pinot.spi.data.Schema.SchemaBuilder;
-import org.apache.pinot.spi.data.TimeFieldSpec;
 import org.apache.pinot.spi.data.TimeGranularitySpec;
-import org.apache.pinot.spi.data.readers.FileFormat;
 import org.apache.pinot.spi.utils.JsonUtils;
 import org.apache.pinot.tools.Command;
 import org.slf4j.Logger;
@@ -54,6 +48,7 @@ public class GenerateDataCommand extends AbstractBaseAdminCommand implements Com
 
   private final static String FORMAT_AVRO = "avro";
   private final static String FORMAT_CSV = "csv";
+  private final static String FORMAT_JSON = "json";
 
   @CommandLine.Option(names = {"-numRecords"}, required = true, description = "Number of records to generate.")
   private int _numRecords = 0;
@@ -79,7 +74,7 @@ public class GenerateDataCommand extends AbstractBaseAdminCommand implements Com
   private boolean _help = false;
 
   @CommandLine.Option(names = {"-format"}, required = false, help = true,
-      description = "Output format ('AVRO' or 'CSV').")
+      description = "Output format ('AVRO' or 'CSV' or 'JSON').")
   private String _format = FORMAT_AVRO;
 
   @Override
@@ -125,31 +120,20 @@ public class GenerateDataCommand extends AbstractBaseAdminCommand implements Com
     }
 
     Schema schema = Schema.fromFile(new File(_schemaFile));
-
-    List<String> columns = new LinkedList<>();
-    final HashMap<String, DataType> dataTypes = new HashMap<>();
-    final HashMap<String, FieldType> fieldTypes = new HashMap<>();
-    final HashMap<String, TimeUnit> timeUnits = new HashMap<>();
-
-    final HashMap<String, Integer> cardinality = new HashMap<>();
-    final HashMap<String, IntRange> range = new HashMap<>();
-    final HashMap<String, Map<String, Object>> pattern = new HashMap<>();
-    final HashMap<String, Double> mvCountMap = new HashMap<>();
-    final HashMap<String, Integer> lengthMap = new HashMap<>();
-
-    buildCardinalityRangeMaps(_schemaAnnFile, cardinality, range, pattern);
-
     final DataGeneratorSpec spec =
-        buildDataGeneratorSpec(schema, columns, dataTypes, fieldTypes, timeUnits, cardinality, range, pattern,
-            mvCountMap, lengthMap);
+        DataGenerationHelpers.buildDataGeneratorSpec(schema);
+    buildCardinalityRangeMaps(_schemaAnnFile, spec);
+
 
     final DataGenerator gen = new DataGenerator();
     gen.init(spec);
 
     if (FORMAT_AVRO.equalsIgnoreCase(_format)) {
-      gen.generateAvro(_numRecords, _numFiles);
+      DataGenerationHelpers.generateAvro(gen, _numRecords, _numFiles, _outDir, _overwrite);
     } else if (FORMAT_CSV.equalsIgnoreCase(_format)) {
-      gen.generateCsv(_numRecords, _numFiles);
+      DataGenerationHelpers.generateCsv(gen, _numRecords, _numFiles, _outDir, _overwrite);
+    } else if (FORMAT_JSON.equalsIgnoreCase(_format)) {
+      DataGenerationHelpers.generateJson(gen, _numRecords, _numFiles, _outDir, _overwrite);
     } else {
       throw new IllegalArgumentException(String.format("Invalid output format '%s'", _format));
     }
@@ -157,8 +141,7 @@ public class GenerateDataCommand extends AbstractBaseAdminCommand implements Com
     return true;
   }
 
-  private void buildCardinalityRangeMaps(String file, HashMap<String, Integer> cardinality,
-      HashMap<String, IntRange> range, Map<String, Map<String, Object>> pattern)
+  private void buildCardinalityRangeMaps(String file, DataGeneratorSpec spec)
       throws IOException {
     if (file == null) {
       return; // Nothing to do here.
@@ -170,61 +153,15 @@ public class GenerateDataCommand extends AbstractBaseAdminCommand implements Com
       String column = sa.getColumn();
 
       if (sa.isRange()) {
-        range.put(column, new IntRange(sa.getRangeStart(), sa.getRangeEnd()));
+        spec.getRangeMap().put(column, new IntRange(sa.getRangeStart(), sa.getRangeEnd()));
       } else if (sa.getPattern() != null) {
-        pattern.put(column, sa.getPattern());
+        spec.getPatternMap().put(column, sa.getPattern());
       } else {
-        cardinality.put(column, sa.getCardinality());
+        spec.getCardinalityMap().put(column, sa.getCardinality());
       }
     }
   }
 
-  private DataGeneratorSpec buildDataGeneratorSpec(Schema schema, List<String> columns,
-      HashMap<String, DataType> dataTypes, HashMap<String, FieldType> fieldTypes, HashMap<String, TimeUnit> timeUnits,
-      HashMap<String, Integer> cardinality, HashMap<String, IntRange> range,
-      HashMap<String, Map<String, Object>> pattern, Map<String, Double> mvCountMap, Map<String, Integer> lengthMap) {
-    for (final FieldSpec fs : schema.getAllFieldSpecs()) {
-      String col = fs.getName();
-
-      columns.add(col);
-      dataTypes.put(col, fs.getDataType());
-      fieldTypes.put(col, fs.getFieldType());
-
-      switch (fs.getFieldType()) {
-        case DIMENSION:
-          if (cardinality.get(col) == null) {
-            cardinality.put(col, 1000);
-          }
-          break;
-
-        case METRIC:
-          if (!range.containsKey(col)) {
-            range.put(col, new IntRange(1, 1000));
-          }
-          break;
-
-        case TIME:
-          if (!range.containsKey(col)) {
-            range.put(col, new IntRange(1, 1000));
-          }
-          TimeFieldSpec tfs = (TimeFieldSpec) fs;
-          timeUnits.put(col, tfs.getIncomingGranularitySpec().getTimeType());
-          break;
-
-        // forward compatibility with pattern generator
-        case DATE_TIME:
-        case COMPLEX:
-          break;
-
-        default:
-          throw new RuntimeException("Invalid field type.");
-      }
-    }
-
-    return new DataGeneratorSpec(columns, cardinality, range, pattern, mvCountMap, lengthMap, dataTypes, fieldTypes,
-        timeUnits, FileFormat.AVRO, _outDir, _overwrite);
-  }
-
   public static void main(String[] args)
       throws IOException {
     SchemaBuilder schemaBuilder = new SchemaBuilder();
